############
# a command line function that performs the same functionality as
# the Load Model menu item under the File menu
# modelData is the actual data (this will be loaded in the global environment)
# dataName is the name of the new model (MVC)
# typeModel is the type of data to load (either "data.frame", "exprSet", or 
#  "graph")
############
loadData<-function(modelData, dataName, typeModel)
{
  # dataName must be a character string and it must be unique (uniqueness will 
  # be checked in the handleMessage method)
  if (dataName != "")
  {
    curData<-list(data=modelData, linkData=NULL)
    newMessage<-new("gAddDataMessage", data=curData, dataName=dataName, 
                        type=typeModel)
    handleMessage(newMessage)
    cat("The", dataName, "data set has been loaded.", "\n")
  }
  else
    cat("The", dataName, "data set could not be loaded.", "\n")
}

############
# added 6/26/06
# let users add model variables to an already loaded MVC (model)
# modelName is the name of the model (MVC) to add this variable to
# modelVarName is the name of the model variable (how this model variable will
#  be referenced in the modelVar list)
# modelVar is the actual model variable
############
loadModelVar<-function(modelName, modelVarName, modelVar)
{
  if (modelName %in% getModelNames())
  {
    curMVC<-getMVC(modelName)
    curModel<-model(curMVC)
    curModVar<-modelVar(curModel)
    curLength<-length(curModVar)
    modVarNValid<-TRUE

    if (modelVarName == "")
    {
      print("The model variable name must not be an empty string.")
      modVarNValid<-FALSE
    }  

    if (curLength > 0)
    {
      # check if the model variable name is already used
      if (modelVarName %in% names(curModVar))
      {
        print(paste("That model variable name is already in use. ", 
                    "Please pick a different model variable name."))
        modVarNValid<-FALSE
      }
    }
    # only continue if the model variable name is valid
    if (modVarNValid)
    {
      curModVar[[curLength+1]]<-modelVar
      names(curModVar)[curLength+1]<-modelVarName
      # now need to update everything in the MVC list
      # first update the model and MVC
      modelVar(curModel)<-curModVar
      model(curMVC)<-curModel

      # now update the MVC list
      curIndex<-which(getModelNames(sort=FALSE) %in% modelName)
      curMVCList<-get("MVCList", mvcEnv)
      curMVCList[[curIndex]]<-curMVC
      assign("MVCList", curMVCList, mvcEnv)
    }
  }
  else
    print("The modelName parameter must be the name of a loaded model (MVC).")
}

#########
# 6/27/06
# return the model variables for a model 
# if mName is left blank, then the active MVC is used
#########
getModelVar<-function(mName="")
{
  if (length(getMVCList()) > 0)
  {
    if (mName == "")
      mName<-get("activeMVC", mvcEnv)
    if (mName %in% getModelNames())
    {
      curMVC<-getMVC(mName)
      curmVar<-modelVar(model(curMVC))
      return(curmVar)
    }
  }
}

#############
# setActiveMVC is a command line function that performs the same functionality 
# as the Set Active MVC menu item under the Edit menu
# aMVC is the name of the MVC that will be set to active
#############
setActiveMVC<-function(aMVC)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    mNames<-getModelNames()
    if (aMVC %in% mNames)
    {
      assign("oldActiveMVC", get("activeMVC", mvcEnv), mvcEnv)
      assign("activeMVC", aMVC, mvcEnv)

      if (length(get("controlWin", mvcEnv)) > 0)
      {
        # need to reset the display menu
        setDisplayMenu(aMVC)

        # set the control window to empty because the old action may be
        # for a different model type
        setEmptyView()
      }
      
      # need to refresh the model graph if there is one
      if (length(get("modelWin", mvcEnv)) > 0)
      {
        Sys.sleep(0.5)
        plotModelGraph()
      }
    }
    else
      print(paste(aMVC, "is not the name of a loaded MVC."))
  }
  else
    print("There are no MVCs loaded.")
}

############
# 6/22/06
# set the model name for a new child model (used when creating a heatmap
# from a graph node)
# childName is the name of the new child model (that the heatmap will be made
#  from) 
# parentName is the name of the parent model that the child is made from -
#  this only needs to be set if the parent is not the active MVC
############
setModelName<-function(childName, parentName="")
{
  # parentName must be one of the loaded models
  if (!(parentName %in% getModelNames()))
    parentName<-""
 
  # use the control environment that pertains to the active window
  # this doesn't actually work because the active window will be the window
  # with the R prompt (so it is either the active MVC's controller or
  # the controller of the parentName MVC)
  if (parentName == "")
    curMVC<-getMVCFromWinNum()
  else
    curMVC<-getMVC(parentName)
  controlEnv<-controller(curMVC)

  # first check if childName is unique and not ""
  if (!(childName %in% getModelNames()) && childName != "")
    assign("heatmapModName", childName, controlEnv)  
  else
    print(paste("The parameter, childName, must be a unique name that is",
                "currently not used by other models."))
}

############
# returns the methods to create a child model
# (problem: need to notify user that these methods need extra parameters)
############
getMethodsToCreateChild<-function()
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    controlEnv<-controller(curMVC)
    # these are the methods to create a child model without a GUI
    createChild<-get("createChildNoGUI", controlEnv)
    if (length(createChild) > 0)
      return(sort(createChild))
    else
      print("There are no methods to create a child model for the active MVC.")
  }
}

##########
# a function that creates a child model that is the gene ontology (GO) graph
# that corresponds to the affy ids from the parent model
# newModelName is the name of the new child model
# annotationSlot is the annotation slot of the exprSet (the Affymetrix chip 
# type, such as "hu6800", "hgu95av2", etc.)
# ontology is the ontology type (can be "MF", "BP", or "CC")
# affyids are the Affymetrix ids from the parent model that are used to create
# the GO graph (if affyids is left empty, then all Affymetrix ids from the
# parent model will be used)
# mVar is the model variable to be used to subset the parent model (if both
#  affyids and mVar are used, then the intersection of the affyids will be 
#  used) 
##########
createGoGraph<-function(newModelName, annotationSlot, ontology="MF",
				      affyids="", mVar="")
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    controlEnv<-controller(curMVC)
    createChild<-get("createChildNoGUI", controlEnv)
    # make sure this is a valid function to call for the active MVC
    if ("createGoGraph" %in% names(createChild))
    {
      curData<-getData(aMVC)

      # make sure the user has the correct libraries installed
      require("GOstats") || stop("The GOstats package is required to continue")

      if (length(affyids) == 1 && affyids == "")
      {
        if (is(curData, "exprSet"))
          affyids<-geneNames(curData)
        if (is(curData, "data.frame") || is(curData, "matrix"))
          affyids<-rownames(curData)
        if (is(curData, "graph"))
          affyids<-nodes(curData)
      }
      if (mVar != "")
      {
        if (mVar %in% names(getModelVar()))
        {
          curModVar<-modelVar(model(curMVC))[[mVar]]
          if (is(curModVar, "logical"))
          {
            if (is(curData, "exprSet"))
              mvAffyids<-geneNames(curData)[curModVar]
            if (is(curData, "data.frame") || is(curData, "matrix"))
              mvAffyids<-rownames(curData)[curModVar]
            if (is(curData, "graph"))
              mvAffyids<-nodes(curData)[curModVar]

            # now need to get the joint affyids with mvAffyids
            affyids<-affyids[affyids %in% mvAffyids]
          }
        }
      }

      if ((ontology %in% c("MF", "BP", "CC")) && (length(affyids) > 0))
      {
        # need to load the library
        require(annotationSlot, character.only=TRUE) || stop(paste("The", 
             annotationSlot, "package is required to continue")) 
    
        # need to convert affy ids to locus link ids
        LLenv<-paste(annotationSlot, "LOCUSID", sep="") 
        LLids<-as.character(unlist(mget(affyids, eval(as.name(LLenv))), 
                 use.names=FALSE))
        
        newGraph<-makeGOGraph(x=LLids, Ontology=ontology)

        if (length(nodes(newGraph)) == 0)
        {
          print("There are no GO terms annotated with those Affy Ids.")
        }
        else
        {
          # the chipType variable is needed in the createGoGraphVirData 
          # function
          assign("chipType", annotationSlot, controlEnv)

          newGraph<-setGoGraphAttr(newGraph, annotationSlot, LLids, ontology)

          # create the virtual data slot
          newVirData<-createGoGraphVirData(curMVC, affyids, newGraph, 
                            newModelName)

          newAddChildMessage<-new("gAddChildMessage", dataName=newModelName,
                               type="graph", data=list(data=newGraph,
                               virtualData=newVirData))
          handleMessage(newAddChildMessage, subsetParentData=affyids,
                          toParent=linkNodeToRowName,
                          fromParent=linkRowNameToNode)

          # should set the chipType in the new child MVC's controller (right 
          # now it is just set in the parent controller)
          childMVC<-getMVC(newModelName)
          childControlEnv<-controller(childMVC)
          assign("chipType", annotationSlot, childControlEnv)
          # need to also set variables that refer to environments to change
          # from GO term to affy id and vice versa
          assign("nodeToRNEnv", paste(annotationSlot, "GO2ALLPROBES", sep=""), 
                   childControlEnv)
          assign("rnToNodeEnv", paste(annotationSlot, "GO", sep=""), 
                   childControlEnv) 
          assign("nodeParentsEnv", paste("GO", ontology, "ANCESTOR", sep=""), 
                   childControlEnv)

          print(paste(newModelName, "data set has been loaded."))
        }
      }
    }
  }
}

############
# added 7/9/06
# create a child model by performing gene set enrichment (GSE) using the
# chromosome location as the gene sets using t-tests as the test statistics
############
createChrLocGSE<-function(newModelName, fac, annot="", affyids="", mVar="",
                          noGenes=5)
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    controlEnv<-controller(curMVC)
    createChild<-get("createChildNoGUI", controlEnv)
    # make sure this is a valid function to call for the active MVC
    if ("createChrLocGSE" %in% names(createChild))
    {
      # this will be the exprSet data
      curData<-getData(aMVC)

      if (annot == "")
        annot<-annotation(curData)
      if (annot == "")
        annot<-get("chipType", controlEnv)
  
      require(annot, character.only=TRUE) || 
            stop("The annotation package is required to continue")

      # make sure the user has the correct libraries installed
      require("genefilter") || 
            stop("The genefilter package is required to continue")

      # subset by a model variable
      if (mVar != "")
      {
        if (mVar %in% names(getModelVar()))
        {
          curModVar<-modelVar(model(curMVC))[[mVar]]
          if (is(curModVar, "logical"))
          {
            curData<-curData[curModVar, ]            
          }
        }
      }

      # subset by affyids
      if (length(affyids) > 1 || affyids != "")
      {
        if (is(curData, "exprSet"))
          if (any(affyids %in% geneNames(curData)))
            curData<-curData[which(geneNames(curData) %in% affyids), ]
        if (is(curData, "data.frame") || is(curData, "matrix"))
          if (any(affyids %in% rownames(curData)))
            curData<-curData[which(rownames(curData) %in% affyids), ]
      }

      ttests<-rowttests(curData, fac)
      if (is(curData, "exprSet"))
        fL<-findLargest(geneNames(curData), abs(ttests$statistic), annot)
      if (is(curData, "data.frame") || is(curData, "matrix"))
        fL<-findLargest(rownames(curData), abs(ttests$statistic), annot)

       # make sure the user has the correct libraries installed
      require(Category) || 
            stop("The Category package is required to continue")

      mapIncMat<-createMAPIncMat(annot)

      # only use the genes that have chromosome locations
      fL2<-fL[names(fL) %in% colnames(mapIncMat)]
   
      # then need to subset this incidence matrix based on number of genes
      # and the names of our genes
      incMat<-mapIncMat[, which(colnames(mapIncMat) %in% names(fL))]

      rs<-rowSums(incMat)
      rs2<-rs[rs > noGenes]
      # only take pathways that have a certain number of genes in them
      incMat2<-incMat[rs > noGenes, ]

      # need to recalculate the t-tests with the subset of data
      if (is(curData, "exprSet"))
        curDatasub<-curData[which(geneNames(curData) %in% fL2), ]
      if (is(curData, "data.frame") || is(curData, "matrix"))
        curDatasub<-curData[which(rownames(curData) %in% fL2), ]

      tsub<-rowttests(curDatasub, fac)
      gTS<-tsub$statistic

      gsTS<-(incMat2 %*% gTS)/sqrt(rs2)
      expD<-curDatasub
      des<-"Chromosome Location"
      newGSE<-new("GSE", incidMat=incMat2, gTestStat=gTS, 
                         gsTestStat=as.vector(gsTS),
                         expData=expD, descr=des)

      # now can create the child model
      # need to think about linking!!!!

      # need to create a function to set the virtualData slot so it
      # matches the virData slot of the parent
      newVirData<-createKEGGgseVirData(curMVC, fL2, newGSE)

      newAddChildMessage<-new("gAddChildMessage", dataName=newModelName,
                               type="GSE", data=list(data=newGSE,
                               virtualData=newVirData))

      handleMessage(newAddChildMessage, subsetParentData=fL,
                          toParent=linkGeneSetToGenes,
                          fromParent=linkGeneToGeneSets)

      # should set the chipType in the new child MVC's controller (right 
      # now it is just set in the parent controller)
      childMVC<-getMVC(newModelName)
      childControlEnv<-controller(childMVC)
      assign("chipType", annot, childControlEnv)

      # next add model variables 
      # add the normal p-values
      curPvals<-round(dnorm(as.vector(gsTS)), 4)
      loadModelVar(newModelName, "Normal.p.value", curPvals)

      if (length(get("controlWin", mvcEnv)) > 0)
        showLabel(paste(newModelName, "data set has been loaded."))
      else      
        print(paste(newModelName, "data set has been loaded."))
    }
  }
}

########
# create a child model by performing gene set enrichment (GSE) using the
# KEGG pathways as the gene sets using t-tests as the test statistics
#
# newModelName is the name of the new model
# fac is the factor to use for the t-tests
# annot is the annotation
# affyids are the affy ids to subset the data
# mVar is the model variable to subset the data
# noGenes is the number of genes expected in a pathway (to include it)
########
createKEGGGSE<-function(newModelName, fac, annot="", affyids="", mVar="", 
                        noGenes=5)
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    controlEnv<-controller(curMVC)
    createChild<-get("createChildNoGUI", controlEnv)
    # make sure this is a valid function to call for the active MVC
    if ("createKEGGGSE" %in% names(createChild))
    {
      # this will be the exprSet data
      curData<-getData(aMVC)

      if (annot == "")
        annot<-annotation(curData)
      if (annot == "")
        annot<-get("chipType", controlEnv)
  
      require(annot, character.only=TRUE) || 
            stop("The annotation package is required to continue")

      # make sure the user has the correct libraries installed
      require("genefilter") || 
            stop("The genefilter package is required to continue")

      # subset by a model variable
      if (mVar != "")
      {
        if (mVar %in% names(getModelVar()))
        {
          curModVar<-modelVar(model(curMVC))[[mVar]]
          if (is(curModVar, "logical"))
          {
            curData<-curData[curModVar, ]            
          }
        }
      }

      # subset by affyids
      if (length(affyids) > 1 || affyids != "")
      {
        if (is(curData, "exprSet"))
          if (any(affyids %in% geneNames(curData)))
            curData<-curData[which(geneNames(curData) %in% affyids), ]
        if (is(curData, "data.frame") || is(curData, "matrix"))
          if (any(affyids %in% rownames(curData)))
            curData<-curData[which(rownames(curData) %in% affyids), ]
      }

      ttests<-rowttests(curData, fac)
      if (is(curData, "exprSet"))
        fL<-findLargest(geneNames(curData), abs(ttests$statistic), annot)
      if (is(curData, "data.frame") || is(curData, "matrix"))
        fL<-findLargest(rownames(curData), abs(ttests$statistic), annot)

       # make sure the user has the correct libraries installed
      require(Category) || 
            stop("The Category package is required to continue")

      fL2<-probes2Path(fL, annot)

      fL<-fL[fL %in% names(fL2)]

      Amat<-t(PWAmat(annot))
      incMat<-Amat[, names(fL)]     
      rs<-rowSums(incMat)
      rs2<-rs[rs > noGenes]
      # only take pathways that have a certain number of genes in them
      incMat2<-incMat[rs > noGenes, ]

      # need to recalculate the t-tests with the subset of data
      if (is(curData, "exprSet"))
        curDatasub<-curData[which(geneNames(curData) %in% names(fL2)), ]
      if (is(curData, "data.frame") || is(curData, "matrix"))
        curDatasub<-curData[which(rownames(curData) %in% names(fL2)), ]
      tsub<-rowttests(curDatasub, fac)
      gTS<-tsub$statistic

      gsTS<-(incMat2 %*% gTS)/sqrt(rs2)
      expD<-curDatasub
      des<-"KEGG Pathway"
      newGSE<-new("GSE", incidMat=incMat2, gTestStat=gTS, 
                         gsTestStat=as.vector(gsTS),
                         expData=expD, descr=des)

      # now can create the child model
      # need to think about linking!!!!

      # need to create a function to set the virtualData slot so it
      # matches the virData slot of the parent
      newVirData<-createKEGGgseVirData(curMVC, fL, newGSE)

      newAddChildMessage<-new("gAddChildMessage", dataName=newModelName,
                               type="GSE", data=list(data=newGSE,
                               virtualData=newVirData))

      handleMessage(newAddChildMessage, subsetParentData=fL,
                          toParent=linkGeneSetToGenes,
                          fromParent=linkGeneToGeneSets)

      # should set the chipType in the new child MVC's controller (right 
      # now it is just set in the parent controller)
      childMVC<-getMVC(newModelName)
      childControlEnv<-controller(childMVC)
      assign("chipType", annot, childControlEnv)

      # next add model variables 
      # get the pathway names
      pathIds<-rownames(incMat2)
      pathNames<-mget(pathIds, KEGGPATHID2NAME)
      if (all(unlist(lapply(pathNames, length))==1) &&
            all(unlist(lapply(pathNames, is.na))==FALSE))
      {
        pathNames<-unlist(pathNames)
        loadModelVar(newModelName, "KEGGPathwayName", pathNames)
      }
      # add the normal p-values
      curPvals<-round(dnorm(as.vector(gsTS)), 4)
      loadModelVar(newModelName, "Normal.p.value", curPvals)

      if (length(get("controlWin", mvcEnv)) > 0)
        showLabel(paste(newModelName, "data set has been loaded."))
      else      
        print(paste(newModelName, "data set has been loaded."))
    }
  }
}

###########
# 6/6/06
# to set attributes for the GO graph nodes
# goGraph is the new GO graph
# annotation is the annotation slot (e.g. "hgu95av2")
# LLids are the locus link ids that were used to make this graph
# ont is the ontology
###########
setGoGraphAttr<-function(goGraph, annotation, LLids, ont)
{
  nodeNames<-nodes(goGraph)
  # the node names are GO ids
  # want to get GO terms - the GO library has already been loaded
  allGOterms<-mget(nodeNames, GOTERM)
  goTerms<-unlist(lapply(allGOterms, function(x) {x@Term}), use.names=FALSE)

  nodeDataDefaults(goGraph, "Term")<-""
  # for some reason this doesn't work as expected so instead use a for loop
#  nodeData(goGraph, n=nodeNames, attr="Term")<-goTerms
  for (i in 1:length(nodeNames))
  {
    nodeData(goGraph, n=nodeNames[i], attr="Term")<-goTerms[i]
  }
  goHyper<-GOHyperG(unique(LLids), lib=annotation, what=ont)
  nodeDataDefaults(goGraph, "HGpvalue")<-0
  nodeDataDefaults(goGraph, "intLLCounts")<-0
  nodeDataDefaults(goGraph, "allLLCounts")<-0
  for (i in 1:length(nodeNames))
  {
    nodeData(goGraph, n=nodeNames[i], attr="HGpvalue")<-
      signif(goHyper$pvalues[which(names(goHyper$pvalues) %in% 
                             nodes(goGraph)[i])], 3)
    nodeData(goGraph, n=nodeNames[i], attr="intLLCounts")<-
      goHyper$intCounts[which(names(goHyper$intCounts) %in% nodes(goGraph)[i])]
    nodeData(goGraph, n=nodeNames[i], attr="allLLCounts")<-
      goHyper$goCounts[which(names(goHyper$goCounts) %in% nodes(goGraph)[i])]
  }

  return(goGraph)
}

############
# a function that creates a child model that is a subset of the parent model
# newModelName is the name of the new child model
# rowNames are the names of the rows to use from the parent model (currently,
# this function only works with data frame models)
#
# note: this function could work with exprSet models (if you used gene names
# as the row names)
############
createSubset<-function(newModelName, rowNames)
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    controlEnv<-controller(curMVC)
    createChild<-get("createChildNoGUI", controlEnv)
    # make sure that this is a valid function to call for the active MVC
    if ("createSubset" %in% names(createChild))
    {
      curData<-getData(aMVC)
      if (any(rowNames %in% rownames(curData)))
      {
        curIndex<-match(rowNames, rownames(curData))
        # remove any NAs
        curIndex<-curIndex[!is.na(curIndex)]
        subData<-curData[curIndex,]
        subVirData<-virtualData(model(curMVC))[curIndex,]

        newAddChildMessage<-new("gAddChildMessage", dataName=newModelName,
                             type="data.frame", data=list(data=subData, 
                             virtualData=subVirData))
        handleMessage(newAddChildMessage, toParent=linkToParentByRowName, 
                    fromParent=linkToChildByRowName, 
                    subsetParentData=rowNames) 
        assign("childName", "", controlEnv)
        print(paste(newModelName, "data set has been loaded."))
      }
    }
  }
}

############
# a function that creates a child model by performing Multidimensional Scaling
# (MDS) on the parent model
# newModelName is the name of the new child model
# k is the number of dimensions for the MDS
# transpose determines whether the matrix should be transposed before
# performing MDS (T or F)
#
# transpose parameter is not currently used because it makes the link data
# functions too complicated (want to be able to link on row names)
############
performMDS<-function(newModelName, k=2, transpose=FALSE)
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    controlEnv<-controller(curMVC)
    createChild<-get("createChildNoGUI", controlEnv)
    if ("performMDS" %in% names(createChild))
    {
      curData<-getData(aMVC)
      isDf<-is(curData, "data.frame")
      # need a matrix (or data.frame) to perform the distance function on
      if (!isDf)
        clusterData<-exprs(curData)
      else
        clusterData<-curData      

#      if (transpose)
#        clusterData<-t(clusterData)

      # now need to create the child model
      newModelData<-cmdscale(dist(clusterData), k=k)

      # will need to link this back to the parent model
      newAddChildMessage<-new("gAddChildMessage", dataName=newModelName,
                             type="data.frame", data=list(data=newModelData))

      # need to add toParent, fromParent, and subsetParentData to the
      # handleMessage call
      # currently not subsetting the data so use all row names
      subRowNames<-rownames(clusterData)

      handleMessage(newAddChildMessage, subsetParentData=subRowNames,
                     toParent=linkToParentByRowName, 
                     fromParent=linkToChildByRowName)

      assign("childName", "", controlEnv)
      print(paste(newModelName, "data set has been loaded."))
    }
  }
}

###########
# quitiSNetwork is a command line function to perform the same functionality
# that is available under the Quit menu item under the File menu
###########
quitiSNetwork<-function()
{
  clearAllWindows()
}

###########
# createSpreadsheet is a command line function that performs the same 
# functionality as the View Data menu item under the Display menu
# this only works for a dfModel object
###########
createSpreadsheet<-function()
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    curModel<-model(curMVC)
    curData<-getData(aMVC)
    # can only create a spreadsheet view of a data frame model
    if (is(curModel, "dfModel"))
    {
      # then need to check if a spreadsheet has already been created for 
      # this model
      curVList<-viewList(curMVC)
      booSpreadView<-unlist(lapply(curVList, function(x) {is(x, 
                                 "spreadView")}))
      # if there are any spread views then this data set is already shown
      # in a spreadsheet
      if (any(booSpreadView))
        booCreate<-FALSE
      else
        booCreate<-TRUE

      if (booCreate)
      {
        sData<-cbind(rownames(curData))
        sData<-cbind(sData, curData)
        # create and handle a message to create a view
        vMessage<-new("gAddViewMessage", dataName=aMVC, type="spreadView",
                        mData=list(spreadData=sData, 
                                   colNames=c("Row.Names", colnames(curData))))
        handleMessage(vMessage)
      }      
    }
    # can create a spreadsheet view if the active model is a graph and the
    # the nodes have attributes
    if (is(curModel, "graphModel"))
    {
      nattr<-getNodeAttr()
      # only build the spreadsheet if there are some node attributes
      if (!is.null(nattr))
      {
        # need to make the data frame to show
        nd<-nodeData(curData)
        sData<-cbind(nodes(curData))
        # now need to add all the node attributes
        for (i in 1:length(nattr))
        {
          curVec<-unlist(lapply(nd, function(x) {x[[i]]}))
          sData<-cbind(sData, curVec)
        }        
        cNames<-c("Node Names", nattr)

        vMessage<-new("gAddViewMessage", dataName=aMVC, type="spreadView",
                        mData=list(spreadData=sData, colNames=cNames))
        handleMessage(vMessage)
      }
      else
        print("There are no node attributes to show in a spreadsheet.")
    }
    if (is(curModel, "gseModel"))
    {
      # decide what to put in the spreadsheet - gene set name (from the 
      # row names of the incidMat), gene set test stat (from the
      # gsTestStat GSE slot), and then all model variables
      geneSetNames<-rownames(incidMat(curData))
      geneSetTestStats<-round(gsTestStat(curData), 3)
      # get the number of genes per gene set
      numberGenes<-apply(incidMat(curData), 1, sum)
      sData<-cbind(geneSetNames, numberGenes, geneSetTestStats)

      # next look at model variables
      curMVar<-modelVar(model(curMVC))
      if (length(curMVar) > 0)
      {
        for (i in 1:length(curMVar))
        {
          sData<-cbind(sData, curMVar[[i]])
        }
        cNames<-c("Gene Set Names", "Number of Genes in Gene Set", 
                  "Gene Set Test Stats", names(curMVar))
      }
      else
        cNames<-c("Gene Set Names", "Number of Genes in Gene Set", 
                  "Gene Set Test Stats")

      vMessage<-new("gAddViewMessage", dataName=showDF, 
                        type="spreadView",
                        mData=list(spreadData=sData, colNames=cNames))
      handleMessage(vMessage)
    }
  }
}

##########
# createQQplot is a command line function to create a qq plot
##########
createQQplot<-function()
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    curModel<-model(curMVC)
    # for now create a qq plot of GSE data
    if (is(curModel, "gseModel"))
    {
      # create a message to create a view
      vMessage<-new("gAddViewMessage", dataName=aMVC, type="qqPlotView",
            mData=list())
      handleMessage(vMessage)       
    }
  }
}

###########
# createSPlot is a command line function that performs the same functionality 
# as choosing the Plot Data menu item from the Display menu on the 
# control window
# 
# can only create plot of the active MVC and the model in the active MVC must
# be of class dfModel
#
# will need several parameters - 
#  the variable (column) name for the x axis
#  the variable (column) name for the y axis
#  the row names (if the user only wants to plot a subset)
###########
createSPlot<-function(varx, vary, rowNames="")
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    curModel<-model(curMVC)
    # can only create a spreadsheet view of a data frame model
    if (is(curModel, "dfModel"))
    {
      curData<-getData(aMVC)
      varNames<-colnames(curData)
      numRows<-dim(curData)[1]
      Xindex<-match(varx, varNames)
      Yindex<-match(vary, varNames)
      if (rowNames == "")
        DFrows<-rownames(curData)
      else
        DFrows<-rowNames

      # create a message to create a view
      vMessage<-new("gAddViewMessage", dataName=aMVC, type="sPlotView",
            mData=list(dfRows=DFrows, xvar=varx, yvar=vary))
      handleMessage(vMessage)      
    }
  }
}

############
# createGraphPlot is a command line function that performs the same 
# functionality as the Plot Graph menu item under the Display menu
#
# this only works when the model in the active MVC is of class graphModel
#
# there are two parameters for plotting a graph: layout and nodeShape
# the options for layout are "dot", "neato", and "twopi"
# the options for nodeShape are "circle", "ellipse", and "rectangle"
############
createGraphPlot<-function(layout="dot", nodeShape="circle")
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    curModel<-model(curMVC)
    if (is(curModel, "graphModel"))
    {
      vMessage<-new("gAddViewMessage", dataName=aMVC, type="graphView", 
                      mData=list(glayout=layout, nShape=nodeShape))
      handleMessage(vMessage)
    }
  }
}

############
# createHeatmap is a command line function that performs the same
# functionality as the Plot Heatmap menu item under the Display menu
#
# this only works when the model in the active MVC is of class exprModel 
#
# modelVar is the model variable from the active model (MVC) that will subset
# the data so that only certain rows from the exprSet are used
############
createHeatmap<-function(mVar="")
{
  aMVC<-get("activeMVC", mvcEnv)
  if (aMVC != "")
  {
    curMVC<-getMVC(aMVC)
    curModel<-model(curMVC)
    if (is(curModel, "exprModel"))
    {
      modData<-modelData(curModel)
      if (length(geneNames(modData)) > 1)
      {
        modVar<-modelVar(curModel)
        if (length(modVar) > 0 && mVar != "")
        {
          if (mVar %in% names(modVar))
          {
            modelVarVal<-modVar[[mVar]]
            if (is(modelVarVal, "logical"))
              curmData<-list(rNames=geneNames(modData)[modelVarVal])
            else
              curmData<-list()
          }
          else
            curmData<-list()
        }
        else
          curmData<-list()

        vMessage<-new("gAddViewMessage", dataName=aMVC,
                    type="heatmapView", mData=curmData)
        handleMessage(vMessage)
      }
      else
      {
        if (length(get("controlWin", mvcEnv)) > 0)
          showLabel(paste("This model has only one row and",
                         "the heatmap plot needs at least two rows."))
        else
          print(paste("This model has only one row and",
                         "the heatmap plot needs at least two rows."))
      }      
    }
  }
}

###########
# this function will provide the same functionality as choosing
# one of the menu items from the Event menu (ex. choosing Set Mouse Over,
# Set Left Button Click, etc.)
#
# will need two parameters:
#  event will be an item from "possEvents" in mvcEnv (right now the options
#    are mouseOver, leftButtonClick, middleButtonClick, and rightButtonClick)
#  shortName will be a character string referring to the shortName slot from
#    an object of class gEventFun (i.e. shortName is a description of the 
#    callback function) 
#  if shortName is left blank, then that means that there will be no active
#    active callback function for the event
###########
setCallFunc<-function(event, shortName="")
{
  if (event %in% get("possEvents", mvcEnv))
  {
    aMVC<-get("activeMVC", mvcEnv)
    if (aMVC != "")
    { 
      curMVC<-getMVC(aMVC)
      controlEnv<-controller(curMVC)

      # now need to make sure that callName is a valid value
      curEvents<-get(event, controlEnv)
      curShorts<-unlist(lapply(curEvents, function(x) {shortName(x$eventFun)}))
      if (shortName %in% curShorts || shortName=="")
      {
        # first need to remove any old active function
        activeFun<-unlist(lapply(curEvents, function(y)
                            {y$assigned==TRUE}))
        if (any(activeFun))
        {
          curActiveIndex<-which(activeFun)
          curEvents[[curActiveIndex]][["assigned"]]<-FALSE
        }              

        # then assign this callback function to be active
        if (shortName != "")
        {
          curIndex<-match(shortName, curShorts)
          curEvents[[curIndex]][["assigned"]]<-TRUE
        }
        assign(event, curEvents, controlEnv)

        # need to call the preprocessing function
        if (shortName != "")
        {
          preprocess<-preprocessFun(curEvents[[curIndex]][["eventFun"]])
          if (length(preprocess) > 0)
            for (i in 1:length(preprocess))
            {
              do.call(preprocess[[i]], list())
            }
        }
        if (shortName != "")
          print(paste(shortName, "has been set as the active function for", 
                      event))
        else
          print(paste("No action is active for", event))
      }
      else
      {
        print("The description of the callback function you gave as a parameter is not a valid value.")
      }
    }
  }
  else
  {
    print("Please use the function, getEvents, to find valid values for the event parameter")
  }
}

############
# this function will return all of the possible callback function descriptions
# (shortName slot) for this event
#
# event will be an item from "possEvents" in mvcEnv (right now the options
#   are mouseOver, leftButtonClick, middleButtonClick, and rightButtonClick)
############
getDescForEvent<-function(event)
{
  if (event %in% get("possEvents", mvcEnv))
  {
    aMVC<-get("activeMVC", mvcEnv)
    if (aMVC != "")
    {
      curMVC<-getMVC(aMVC)
      controlEnv<-controller(curMVC)
      curEvents<-get(event, controlEnv)

      if (length(curEvents) > 0)
      {
        curDesc<-unlist(lapply(curEvents, function(x) {shortName(x$eventFun)}))
        curFunc<-unlist(lapply(curEvents, function(x) {callFun(x$eventFun)}))

        return(curDesc)
      }
      else
        print("There are currently no callback functions for this event.")
    }
  }
  else
  {
    print("Please use the function, getEvents, to find valid values for the event parameter")
  }
}

#############
# this function will return all possible events that can be linked to
# callback functions (so it will return the values from "possEvents" in
# mvcEnv
#############
getEvents<-function()
{
  return(get("possEvents", mvcEnv))
}

##########
# this function will replace the createColorBrowser function when
# the user is not using the GUI, but wants to color points as a callback
# function (see functions createColorBrowser and colorPoint)
#
# newColor is the character string describing the new color 
##########
setColor<-function(newColor)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
    
    assign("currentColor", newColor, controlEnv)
  }
}

##########
# 6/5/06
# this function lets the user set the annotation slot on an exprSet if it
# is empty (this is necessary to convert the affy ids to SYMBOLS, GO, etc.)
##########
setChipType<-function(chipType)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
 
    assign("chipType", chipType, controlEnv)
  }
}

##########
# a function to return the current color stored in the active MVC's controller 
##########
getColor<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    controlEnv<-controller(getMVC(activeMVC))
    curColor<-get("currentColor", controlEnv)
    return(curColor)
  }
  else
    print("There is no active MVC.")
}

##########
# 6/9/06
# a function to let users show values that are hidden in the model of the
# active MVC
##########
showHiddenValues<-function(hidValues)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0 && !is.null(hidValues))
  {
    allHidValues<-getHiddenValues()
    if (all(hidValues %in% allHidValues) && !is.null(allHidValues))
    {
      activeMVC<-get("activeMVC", mvcEnv)
      curMVC<-getMVC(activeMVC)
      curModel<-model(curMVC)

      if (is(curModel, "graphModel"))
        hideNode(hidValues)
      if (is(curModel, "dfModel"))
        hidePoint(hidValues)
      if (is(curModel, "exprModel"))
        hideGene(hidValues)      
      if (is(curModel, "gseModel"))
        hidePoint(hidValues)
    }
    else
      print("Please call getHiddenValues to find out which values are hidden.")
  }
}

##########
# 6/8/06 
# let users see what values are hidden so they can show these values
# returns either row names, genes, or node names depending on the model 
# (in order: dfModel, exprModel, and graphModel)
##########
getHiddenValues<-function(printState=TRUE)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    vData<-virtualData(model(curMVC))
    # now need to look at the hidden values
    hidVal<-vData$hide
    if (any(hidVal))
    {
      # need to get the row names 
      return(rownames(vData)[which(hidVal)])
    }
    else
    {
      if (printState==TRUE)
        print("There are no hidden values for the active MVC.")
      return(NULL)
    }
  }
  else
  {
    if (printState==TRUE)
      print("There is no active MVC.")
    return(NULL)
  }
}

############
# this function will replace the setTooltipsGUI function when the user
# is not using the GUI, but wants tooltips as a callback function (see
# functions tooltips and setTooltipsGUI)
#
# for a data frame model:
# name is the name of the column for tooltips or "rowNames" if the user wants
# the row names as the tooltips
#
# for a graph model:
# name currently can only be "nodeNames" so the only tooltips for a graph are
# the node names
############
setTooltips<-function(name)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
    curModel<-model(curMVC)
    curData<-modelData(curModel)

    if (is(curModel, "dfModel"))
    {
      colNames<-colnames(curData)
      if (name %in% c(colNames, "rowNames"))
      {
        rNames<-rownames(curData)

        if (name=="rowNames")
        {
          # set the tooltips to the row name
          ttVec<-rNames
          names(ttVec)<-rNames
          assign("ttToShow", ttVec, controlEnv)        
        }
        else
        {
          cNames<-colnames(curData)
          ttVec<-curData[,match(name, cNames)]
          names(ttVec)<-rNames
          assign("ttToShow", ttVec, controlEnv)        
        }
      }
      else
        print(paste("The parameter must either be a column name from the", 
                 "active data set or it must be 'rowNames' to indicate",
                 " the row names."))    
    }
    if (is(curModel, "gseModel"))
    {
      gsNames<-rownames(incidMat(curData))
      if (name=="geneSetNames")
      {
        ttVec<-gsNames
        names(ttVec)<-gsNames
        assign("ttToShow", ttVec, controlEnv)
      }
      else
      {
        gsStats<-round(gsTestStat(curData), 2)
        if (name=="geneSetTestStats")
        {
          ttVec<-gsStats
          names(ttVec)<-gsNames
          assign("ttToShow", ttVec, controlEnv)
        }
        else
        {
          # check if the name is a model variable name
          modelVarNames<-names(modelVar(curModel))
          if (!is.null(modelVarNames))
          {
            if (name %in% modelVarNames)
            {
              ttVec<-modelVar(curModel)[[name]]
              names(ttVec)<-gsNames
              assign("ttToShow", ttVec, controlEnv)              
            } 
            else
              print(paste("The parameter must either be a model variable name",
               "from the active data set or it must be 'geneSetNames' to",
               "indicate the gene set names or it must be",
               "'geneSetTestStats' to indicate the gene set test statistics."))
          }
          else
            print(paste("The parameter must either be a model variable name",
               "from the active data set or it must be 'geneSetNames' to",
               "indicate the gene set names or it must be",
               "'geneSetTestStats' to indicate the gene set test statistics."))
        }
      }
    }
    if (is(curModel, "graphModel"))
    {
      # check if name is either nodeNames or an attribute name
      nodDat<-nodeData(curData)
      if (length(nodDat) > 0)
      {
        # for now assume that all nodes have the same attributes!
        allAttr<-unique(unlist(lAttr<-lapply(nodDat, names)))
        possTool<-c("nodeNames", allAttr)
      }
      else
        possTool<-c("nodeNames")

      if (name %in% possTool)
      {
        if (name == "nodeNames")
        {
          # set the tooltips to the node names
          ttVec<-nodes(curData)
          names(ttVec)<-nodes(curData)
          assign("ttToShow", ttVec, controlEnv)
        }
        else
        {
          # we are using one of the attributes
          ttVec<-unlist(lapply(nodDat, function(x) {x[[name]]}))
          names(ttVec)<-nodes(curData)
          assign("ttToShow", ttVec, controlEnv)
        }
      }
      else
        print(paste("Currently the only available tooltips for a graph model",
                    "are the node names ('nodeName') or an attribute of the",
                    "nodes."))
    }
  }
}

###########
# 6/6/06
# a function to let users access node attributes for the active graph
###########
getNodeAttr<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
    curModel<-model(curMVC)
    curData<-modelData(curModel)
    if (is(curModel, "graphModel"))
    {
      # check if name is either nodeNames or an attribute name
      nodDat<-nodeData(curData)
      if (length(nodDat) > 0)
      {
        # for now assume that all nodes have the same attributes!
        allAttr<-unique(unlist(lAttr<-lapply(nodDat, names)))
        return(allAttr)
      }
    }
  }
}

##########
# a function to return the current tooltips stored in the active MVC's 
# controller 
##########
getTooltips<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    controlEnv<-controller(getMVC(activeMVC))
    curTooltips<-get("ttToShow", controlEnv)
    return(curTooltips)
  }
  else
    print("There is no active MVC.")
}

##########
# returns the number of views currently open
##########
getNumberofViews<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    numViews<-unlist(lapply(MVCList, function(x) {
                   length(viewList(x))
                   }))
    totalnumViews<-sum(numViews)
  }
  else 
    totalnumViews<-0

  return(totalnumViews)
}

#############
# when a user adds a new method to create a child MVC, then this function
# will make sure that when users call getMethodsToCreateChild function, they
# will see that this is an option for creating a child MVC
#############
addNewChildMethod<-function(methodName, shortName, typeModel)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  { 
    for (i in 1:length(MVCList))
    {
      curMVC<-MVCList[[i]]
      if (is(model(curMVC), typeModel))
      {
        # need to add this method to createChildNoGUI variable in controller
        controlEnv<-controller(curMVC)
        curcrChild<-get("createChildNoGUI", controlEnv)
        curLength<-length(curcrChild)
        curcrChild[(curLength+1)]<-shortName
        names(curcrChild)[(curLength+1)]<-methodName

        assign("createChildNoGUI", curcrChild, controlEnv)

        # now need to reassign to MVCList
        controller(curMVC)<-controlEnv
        MVCList[[i]]<-curMVC
        assign("MVCList", MVCList, mvcEnv)
      }
    }
  }

  # now need to have it available for future MVCs
  # add to a mvcEnv variable that will be checked when the controller defaults 
  # are set
  newChildMethods<-get("newChildMethods", mvcEnv)
  newChildMethods[[length(newChildMethods)+1]]<-list(methodName=methodName,
                         shortName=shortName, typeModel=typeModel)
  assign("newChildMethods", newChildMethods, mvcEnv)
}

############
# want to add a callback function from the command line
# will need to be added for 2 situations: 1. for MVC objects that are 
#  already loaded and 2. for MVC objects that will be added in the 
#  future
# will need to tell which typeModel this callback function works with - for
# now the options are "graphModel" and "dfModel"
############
addCBFunction<-function(callFunction, shortName, preprocessingFun, 
                              eventLists, typeModel)
{
  MVCList<-get("MVCList", mvcEnv)
   # then need to add this new event to each loaded MVC object
  if (length(MVCList) > 0)
  {
    for (i in 1:length(MVCList))
    {
      if (is(model(MVCList[[i]]), "graphModel") && "graphModel" %in% typeModel)
        addCallEvent(callFunction, shortName, preprocessingFun, eventLists, 
                       MVCList[[i]])
      if (is(model(MVCList[[i]]), "dfModel") && "dfModel" %in% typeModel)
        addCallEvent(callFunction, shortName, preprocessingFun, eventLists, 
                       MVCList[[i]])
    }
  }
  # add to a mvcEnv variable that createDefaultCallEvents will check when a
  # new MVC object is loaded
  # the mvcEnv variable is called newCallFuns
  newCallFuns<-get("newCallFuns", mvcEnv)
  newCallFuns[[length(newCallFuns)+1]]<-list(callFunction=callFunction,
		shortName=shortName, preprocessingFun=preprocessingFun,
		eventLists=eventLists, modelType=typeModel)
  assign("newCallFuns", newCallFuns, mvcEnv)
}

##########
# need a function that creates a mvcEnv variable that will create the
# main menu items and sets the keyVals variable in mvcEnv
#
# this will need to be loaded in the onLoad function
#
# this is needed because we need the keyVals variable to be set even if 
# the user does not open the control window (so things can be done from
# the command line only)
#
# any key vals that need to be used from views created from the command line
# need to be set in this function - see the example for quit (need to call
# the function addToKeyVals)
##########
createMenuItemVariables<-function()
{
  # create a mvcEnv variable for each menu 
  fileItems<-list()
  fileItems[[1]]<-list(text="_Load Model   Ctrl+L", action="loadM")
  fileItems[[2]]<-list(text="Lo_ad Model Variables  Ctrl+A", action="loadMV")
  fileItems[[3]]<-list(text="_Create Child Model  Ctrl+C", action="createCh")
  fileItems[[4]]<-list(text="_Quit          Ctrl+Q", action="quit")
  assign("fileItems", fileItems, mvcEnv)

  # add a key press event for the Quit command even if the 
  # control window is not created
  addToKeyVals(text="_Quit       Ctrl+Q", action="quit")

  editItems<-list()
  editItems[[1]]<-list(text="Set Active M_VC   Ctrl+V", 
                       action="setActiveMVCGUI")
  editItems[[2]]<-list(text="D_elete MVC       Ctrl+E", action="deleteMVCGUI")
#  editItems[[3]]<-list(text="_Add Event Function  Ctrl+A", 
#                       action="addEventFun")
  editItems[[3]]<-list(text="Create Model _Graph  Ctrl+G", 
                        action="plotModelGraph")
  editItems[[4]]<-list(text="Show _Hidden Values  Ctrl+H",
                        action="showHide")
  assign("editItems", editItems, mvcEnv)

  eventItems<-list()
  eventItems[[1]]<-list(text="_Set Mouse Over          Ctrl+S", 
			action="setMouseOver")
  eventItems[[2]]<-list(text="Set Left _Button Click   Ctrl+B", 
			action="setLeftBC")
  eventItems[[3]]<-list(text="Set _Middle Button Click Ctrl+M",
                        action="setMiddleBC")
  eventItems[[4]]<-list(text="Set R_ight Button Click  Ctrl+I", 
			action="setRightBC")
  assign("eventItems", eventItems, mvcEnv)
}

##########
# This function will add to the keyVals variable stored in mvcEnv.
# It is this variable, keyVals, that determines what key press events
# the Gtk windows will respond to.  Thus, the keyVals variable will be 
# different if the views are created through the command line functions or
# through the user interface.  
#
# If the control window is created, then there will be key press events for
# all menu items (this is done through two function calls, including
# createMenuItem and createView).
# If the views are created through the command line functions, then there
# will be key press events for quitting and for setting the focus on a view
# (this is done through the functions createMenuItemVariables and createView).
#
# text is the text that would be placed on the menu item label
# action is the response that should occur when the key press event occurs
# (may either be a function name or the text used in updateControlWindow)
# ... is only used when the action corresponds to focusing on a view (so
# ... will contain the window number of the view)
##########
addToKeyVals<-function(text, action, ...)
{
  newLabel<-gtkAccelLabelNew(text)
  newKey<-gtkLabelParseUline(newLabel, text)

  # get the accelerator character
  labelSplit<-unlist(strsplit(text,"_"))
  if (length(labelSplit) > 1)
    accelChar<-substr(labelSplit[2],1,1)
  else
    accelChar<-""

  keyVals<-get("keyVals", mvcEnv)
  if (length(keyVals) > 0)
  {
    # need to check that this key val hasn't already been added
    # test all three characteristics (keyval, action, and accelChar)
    booMatch<-FALSE
    for (i in 1:length(keyVals))
    {
      if (keyVals[[i]]$keyval==newKey & keyVals[[i]]$action==action &
          keyVals[[i]]$accelChar==accelChar)
        booMatch<-TRUE
    }
    # only add if this is not already in keyVals variable
    if (!booMatch)
    {
      lenKV<-length(keyVals)
      keyVals[[lenKV+1]]<-list(keyval=newKey, action=action,
                           accelChar=accelChar, ...)
    }
  }
  else
  {
    lenKV<-length(keyVals)
    keyVals[[lenKV+1]]<-list(keyval=newKey, action=action,
                           accelChar=accelChar, ...)
  }
  assign("keyVals", keyVals, mvcEnv)
}

############
# to remove key vals when the display menu is changed (because the active mvc
# changed)
############
removeKeyVals<-function(accelChar, action)
{
  keyVals<-get("keyVals", mvcEnv)
  if (length(keyVals) > 0)
  {
    # this means that keyVal does not include an underscore character
    if (length(grep("_", accelChar)) == 0)
      text<-paste("_", accelChar, sep="")
    else
      text<-accelChar

    newLabel<-gtkAccelLabelNew(text)
    newKey<-gtkLabelParseUline(newLabel, text)

    for (i in length(keyVals):1)
    {
      # look for a match and if one is found, then remove that accelerator
      if (keyVals[[i]]$keyval==newKey & keyVals[[i]]$action==action &
          keyVals[[i]]$accelChar==accelChar)
      {
        # this item matches so it must be removed
        keyVals[[i]]<-c()
      }    
    }
    assign("keyVals", keyVals, mvcEnv)
  }
}

#############
# a function to return the active MVC object name
#############
getActiveMVC<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    return(activeMVC)
  }
  else
  {
    return(c())
  }
}

############
# a function to return the MVCList variable in mvcEnv
# this lets the user see all of the MVC objects currently loaded
############
getMVCList<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  return(MVCList)
}

###########
# a function that returns the key vals stored in mvcEnv
###########
getKeyVals<-function()
{
  keyVals<-get("keyVals", mvcEnv)
  return(keyVals)
}

##########
# a command line function that performs the same functionality as
# the Delete MVC menu item (under the Edit menu) - note that
# the active MVC cannot be deleted
#
# will need to reset the parentMVC and childMVCList slots if this is a 
# linked MVC object
#
# the one parameter is the name of the MVC that should be deleted
##########
deleteMVC<-function(dMVC)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    mNames<-getModelNames()
    if (dMVC %in% mNames)
    {
      activeMVC<-get("activeMVC", mvcEnv)
      if (dMVC != activeMVC)
      {
        # now need to remove the views for this MVC 
        delMVC<-getMVC(dMVC)
        vList<-viewList(delMVC)
        
        if (length(vList) > 0)
          for (i in 1:length(vList))
            setDeleteEvents(vList[[i]])        

        # next need to check the parentMVC and childMVCList slots
        curParent<-parentMVC(delMVC)
        curChildren<-childMVCList(delMVC)

        if (curParent != "")
        {
          # then need to remove this MVC object from the parent's 
          # childMVCList slot
          unsortMod<-getModelNames(sort=FALSE)
          # there can only be one parent
          changeMVC<-getMVC(curParent)
          # get the parent's child list
          parentChild<-childMVCList(changeMVC)
          # find the child that matches the soon to be deleted MVC
          curChildIndex<-match(dMVC, parentChild)
          # reset the child list without the deleted MVC
          newChildList<-parentChild[-curChildIndex]
          childMVCList(changeMVC)<-newChildList

          # now need to reassign this MVC to the MVCList variable
          MVCList<-get("MVCList", mvcEnv)
          curIndex<-match(curParent, unsortMod)
          MVCList[[curIndex]]<-changeMVC
          assign("MVCList", MVCList, mvcEnv)           
        }
          if (length(curChildren) > 0)
          {
            # then need to remove this MVC object from the child's parentMVC 
            # slot
            for (i in 1:length(curChildren))
            {
              unsortMod<-getModelNames(sort=FALSE)
              mvcToChange<-curChildren[[i]]
              changeMVC<-getMVC(mvcToChange)
              parentMVC(changeMVC)<-""
              # now need to reassign this MVC to the MVCList variable
              MVCList<-get("MVCList", mvcEnv)
              curIndex<-match(mvcToChange, unsortMod)
              MVCList[[curIndex]]<-changeMVC
              assign("MVCList", MVCList, mvcEnv)
            }
          }

        # then remove the MVC from MVCList in mvcEnv
        # find the index of the item to remove    
        allNames<-getModelNames(sort=FALSE)
        index<-match(dMVC, allNames)

        newMVCList<-MVCList[-index]
        assign("MVCList", newMVCList, mvcEnv)

        # need to refresh the model graph if there is one
        if (length(get("modelWin", mvcEnv)) > 0)
        {
          Sys.sleep(0.5)
          plotModelGraph()
        }        
      }
      else
        print("The active MVC cannot be deleted.")
    }
    else
      print(paste(dMVC, "is not the name of a loaded MVC."))
  }
  else
    print("There is no MVC loaded to delete.")
}

##########
# create a graph of the model relationships
# returns the graph
##########
createModelGraph<-function()
{
  mn<-getModelNames()

  if (!is.null(mn))
  {
    newNodes<-mn
    newEdgeMode<-"directed"
    # now need to make edge list based on parent-child relationships

    newEdgeL<-vector(mode="list", length=length(newNodes))
    names(newEdgeL)<-mn

#    x<-getMVCList()
#    MVCnames<-unlist(lapply(x, function(y) {modelName(model(y))}))

    for (i in 1:length(mn))
    {
      curNode<-mn[i]
      
      curMVC<-getMVC(curNode)
#      curIndex<-match(curNode, MVCnames)
#      curMVC<-x[[curIndex]]

      curChildList<-childMVCList(curMVC)
      if (length(curChildList) > 0)
      {
        curEdgeL<-newEdgeL[[curNode]]
        curMatches<-match(unlist(curChildList), newNodes)
        curEdgeL<-list(edges=curMatches, weights=rep(1, length(curMatches)))
        newEdgeL[[curNode]]<-curEdgeL
      }
    }
    # need to change any NULL elements into empty lists
    newEdgeL<-lapply(newEdgeL, function(x) 
              {
                if (is.null(x))
                  return(list())
                else
                  return(x)
              }
            )
    newModelGraph<-new("graphNEL", nodes=newNodes, edgeL=newEdgeL, 
                       edgemode=newEdgeMode)
    return(newModelGraph)
  }
  else
    return(NULL)
}

##########
# create a plot of the model graph
###########
plotModelGraph<-function()
{
  oldActive<-dev.cur()
  modelGraph<-createModelGraph()
  assign("modelGraph", modelGraph, mvcEnv)

  if (!is.null(modelGraph))
  {
    # first check if the plot has already been made
    modelWin<-get("modelWin", mvcEnv)
    if (length(modelWin) == 0)
    {
      # need to create the window with the drawing area
      win<-gtkWindow(show=FALSE)

      # this will create a gtk device on the window
      retList<-createGtkDev(win)
      win<-retList$win
      drArea<-retList$drArea
      win$Show()
    
      Sys.sleep(0.5)

      newActive<-dev.cur()

      assign("modelWin", win, mvcEnv)
      assign("modelDevice", newActive, mvcEnv)

      activeMVC<-get("activeMVC", mvcEnv)
      activeIndex<-match(activeMVC, nodes(modelGraph))
      curFillColor<-rep("transparent", length(nodes(modelGraph)))
      curFillColor[activeIndex]<-"red"

      # use a layout of neato because then I don't have as many problems with
      # font size (I have problems when I use dot)
      curGrPlot<-agopen(graph=modelGraph, name="modelGraph", 
                 layoutType="neato",
                 nodeAttrs=makeNodeAttrs(modelGraph,
                 shape="circle", fillcolor=curFillColor, color="black"))
      # set a few parameters that don't seem to be set correctly for a gtk 
      # device
      par("bg"="transparent")
      par("col"="black")
      par("fg"="black") 

      # now plot the graph
      plot(curGrPlot, main="Model Graph")

      # need to add a destroy event and key press accelerators
      # should also add to the window menu

      # add the key press events for pseudo-accelerators
      createKeyPressEvent(win)

      curCounter<-get("winCounter", mvcEnv)+1

      # need to add a callback for focus_in_event
      gtkAddCallback(win, "focus_in_event",
        function(obj, ev)
        {
          dev.set(get("modelDevice", mvcEnv))
          assign("activeWinNum", curCounter, mvcEnv)
          return(TRUE)
        }
      )

      # currently not used
      gtkAddCallback(win, "focus_out_event",
        function(obj, ev)
        {
          assign("activeWinNum", c(), mvcEnv)
          return(TRUE)
        }
      )

      # need to also add an item to the window menu
      # add a new sub menu item to the control window (if it's open, otherwise 
      # add a key press event)
      winCounter<-get("winCounter", mvcEnv)
      assign("winCounter", winCounter+1, mvcEnv)

      curLabel<-paste("Window _", get("winCounter", mvcEnv),
                  "  Ctrl+", get("winCounter", mvcEnv), sep="")
      if (length(get("controlWin", mvcEnv)) > 0)
        newMenuItem<-addSubMenuItem(labelText=curLabel, 
                  action="activateWindow",
                  menuName="window", winNum=get("winCounter", mvcEnv))
      else
        addToKeyVals(text=curLabel, action="activateWindow", 
                   winNum=get("winCounter", mvcEnv))
    
      assign("modelWinNum", get("winCounter", mvcEnv), mvcEnv)

      # set the title of the window
      curTitle<-paste("Window ", get("winCounter", mvcEnv), sep="")
      gtkWindowSetTitle(win, curTitle)

      win$AddCallback("delete_event", 
        function(x, y)
        {
          # need to turn off the device in R's device list
          dev.off(newActive)
 
          # reset environment variables
          assign("modelGraph", c(), mvcEnv)
          assign("modelWin", c(), mvcEnv)
          assign("modelDevice", c(), mvcEnv) 
          assign("modelWinNum", 0, mvcEnv)
          assign("activeWinNum", c(), mvcEnv)

          # need to reset the window menu and/or the accelerator keys
          if (length(get("controlWin", mvcEnv)) > 0)
          {
            # will also need to remove the menu item pertaining to this window
            # need to remove the menu item from the menu
            gtkMenuItemRemoveSubmenu(get("windowItem", mvcEnv))
            # need to remove the accelerators from keyVal in controlEnv
            removeAccelerators()
            # now add back the window menu to the control window
            addWindowMenu()  
          }
          else
          {
            # still need to update the accelerator keys
            removeAccelerators()

            # need to add to the window menu for the plots
            MVCList<-get("MVCList", mvcEnv)
            if (length(MVCList) > 0)
            {
              allViews<-lapply(MVCList, viewList)
              for (i in 1:length(allViews))
              {
                curViewList<-allViews[[i]]
                if (length(curViewList) > 0)
                {
                  allWinNums<-unlist(lapply(curViewList, winNum))
                  for (j in 1:length(allWinNums))
                  {
                    curLabel<-paste("Window _", allWinNums[j],
                      "  Ctrl+", allWinNums[j], sep="")
                    addToKeyVals(text=curLabel, action="activateWindow", 
                      winNum=allWinNums[j]) 
                  }
                }
              }
            }
          }

          # finally destroy the window
          win$Destroy() 
        }
      )
    
      # reset the original device
      if (oldActive != 1)
        dev.set(oldActive)
    }
    else
    {
      # the window has already been created
      modelDevice<-get("modelDevice", mvcEnv)
      dev.set(modelDevice)

      activeMVC<-get("activeMVC", mvcEnv)
      activeIndex<-match(activeMVC, nodes(modelGraph))
      curFillColor<-rep("transparent", length(nodes(modelGraph)))
      curFillColor[activeIndex]<-"red"

      # use a layout of neato because then I don't have as many problems with
      # font size (I have problems when I use dot)
      curGrPlot<-agopen(graph=modelGraph, name="modelGraph", 
                 layoutType="neato",
                 nodeAttrs=makeNodeAttrs(modelGraph,
                 shape="circle", fillcolor=curFillColor, color="black"))
      # set a few parameters that don't seem to be set correctly for a gtk 
      # device
      par("bg"="transparent")
      par("col"="black")
      par("fg"="black") 

      # now plot the graph
      plot(curGrPlot, main="Model Graph")

      # now reset the old device to be active
      if (oldActive != 1)
        dev.set(oldActive)    
    }
  }
}

