
#############
# remove the devices from R's device list
#############
removeDevices<-function(curDev)
{
  # remove the devices from R's device list
  sapply(curDev, function(x){dev.off(x)}, USE.NAMES=FALSE)
}

################
# addDrawingArea adds a drawing area to a gtk widget
# this is called by createGtkDev (in iSPlotfuns.R)
#
# wid is the widget, dr is the drawing area,
# width is the drawing area width, height is the drawing area height,
# devVec is the device vector for the gtk window that will contain this
# drawing area,
# addType can be either 1,2,3,or 4 depending on which widget the drawing
# area is being added to (1=window,2=2nd pane,3=notebook,4=scrolled window,
# 5=hbox or vbox)
# lab is the label, devName is the device name
################
addDrawingArea<-function(wid, dr, width, height, devVec, addType=1, lab="", 
                         devName="")
{
  # set up the drawing area
  asGtkDevice(dr)
  dr$SetUsize(width,height)

  # 1 is for normal addition to a window, 
  # 2 is for addition to vertical or horizontal pane (2nd pane),
  # 3 is for addition to a notebook, 4 is for addition to a scrolled window
  # 5 for adding to box (horizontal or vertical)
  switch(addType,
         wid$Add(dr),
         wid$Add2(dr),
         wid$AppendPage(dr,gtkLabel(lab)),
         wid$AddWithViewport(dr),
         gtkBoxPackStart(wid,dr))

  # add to the device vector for the widget
  # right now assuming only one device per Gtk window
  devVec<-dev.cur()

  # set the events that the drawing area will respond to
  dr$SetEvents(c("pointer-motion-mask", "pointer-motion-hint-mask", 
                   "button-press-mask", "button-release-mask", 
                   "key-press-mask", "key-release-mask"))

  # return the updated device vector
  return(devVec)
}

################
# removeEnvPlot removes plots from viewList based on the
# device number
# plotItem is the plot object
################
removeEnvPlot<-function(plotItem)
{
  isPlot<-is(plotItem, "plotView")
  isSpread<-is(plotItem, "spreadView")

  dataName<-dataName(plotItem)
  win<-win(plotItem)

  MVCList<-get("MVCList", mvcEnv)
  allNames<-getModelNames(sort=FALSE)
  mvcindex<-match(dataName, allNames)
  curMVC<-getMVC(dataName)
  curviewList<-viewList(curMVC)

  if (isPlot)
  {
    devVec<-plotDevice(plotItem)

    # need to find the indices of the plots to remove
    dev.num<-getPlotDev(curviewList)

    # sort will remove any NAs and need to make the list decreasing
    # otherwise, the indices won't work because as elements are 
    # removed from the list, the list length decreases (so upper
    # elements have their indices change)
    indicesToRemove<-sort(match(devVec, dev.num), decreasing=TRUE)

    if (length(curviewList)>0 && length(indicesToRemove)>0)
      curviewList[indicesToRemove]<-c()
  }
  if (isSpread)
  {
    booSpread<-unlist(lapply(curviewList, function(x) {is(x, "spreadView")}))
    if (any(booSpread))
    {
      indicesToRemove<-sort(which(booSpread), decreasing=TRUE)
      curviewList[indicesToRemove]<-c()
    }
  }
  # update the plot list
  viewList(curMVC)<-curviewList
  # need to reassign this MVC to MVCList
  MVCList[[mvcindex]]<-curMVC
  assign("MVCList", MVCList, mvcEnv)
}

################
# setDeleteEvents removes the plots from plotList
# and removes the control window
# plotItem is the plot object
################
setDeleteEvents<-function(plotItem)
{
  if (is(plotItem, "plotView"))
  {
    devVec<-plotDevice(plotItem)
    # remove from R's device list
    removeDevices(devVec)
  }

  dataName<-dataName(plotItem)
  win<-win(plotItem)
      
  # need to also remove plots from viewList in MVC
  removeEnvPlot(plotItem)

  if (length(get("controlWin", mvcEnv)) > 0)
  {
    # will also need to remove the menu item pertaining to this window
    # need to remove the menu item from the menu
    gtkMenuItemRemoveSubmenu(get("windowItem", mvcEnv))
    # need to remove the accelerators from keyVal in controlEnv
    removeAccelerators()
    # now add back the window menu to the control window
    addWindowMenu()  
  }
  else
  {
    # still need to update the accelerator keys
    removeAccelerators()

    # need to add to the window menu for the plots
    MVCList<-get("MVCList", mvcEnv)
    if (length(MVCList) > 0)
    {
      allViews<-lapply(MVCList, viewList)
      for (i in 1:length(allViews))
      {
        curViewList<-allViews[[i]]
        if (length(curViewList) > 0)
        {
          allWinNums<-unlist(lapply(curViewList, winNum))
          for (j in 1:length(allWinNums))
          {
            curLabel<-paste("Window _", allWinNums[j],
                    "  Ctrl+", allWinNums[j], sep="")
            addToKeyVals(text=curLabel, action="activateWindow", 
                     winNum=allWinNums[j]) 
          }
        }
      }
    }
    if (length(get("modelWin", mvcEnv)) > 0)
    {
      # need to also add a menu item for the plot of the model graph
      modelWinNum<-get("modelWinNum", mvcEnv)
      curLabel<-paste("Window _", modelWinNum, "  Ctrl+", modelWinNum, sep="")
      addToKeyVals(text=curLabel, action="activateWindow", winNum=modelWinNum)
    }
  }

  win$Destroy()     
}

#######
# will remove all accelerators that have an element winNum
#######
removeAccelerators<-function()
{
  keyVals<-get("keyVals", mvcEnv)

  for (i in length(keyVals):1)
  {
    if ("winNum" %in% names(keyVals[[i]]))
    {
      keyVals[[i]]<-c()
    }
  }
  assign("keyVals", keyVals, mvcEnv)
}

#####
# addWindowMenu is used after calling gtkMenuItemRemoveSubmenu(windowItem)
# to reset the window menu
#####
addWindowMenu<-function()
{
  windowMenu<-gtkMenuNew()

  windowItem<-get("windowItem", mvcEnv)

  menus<-get("menus", mvcEnv)
  menus[["window"]]<-windowMenu
  assign("menus", menus, mvcEnv)

  controlWinItem<-addSubMenuItem(labelText="Control _Window  Ctrl+W",
  action="activateWindow", menuName="window",winNum=0)

  # will need to go through the viewList in every MVC
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {  
    # first get all view lists
    allViewLists<-lapply(MVCList, function(x) {viewList(x)})
    # then get all winNums
    allWinNums<-lapply(allViewLists, function(x) {lapply(x, function(y)
  {winNum(y)})})
    allWinNums<-unlist(allWinNums)
  }
  else
    allWinNums<-c()

  # need to also add a menu item for the plot of the model graph
  if (length(get("modelWin", mvcEnv)) > 0)
  {
    modelWinNum<-get("modelWinNum", mvcEnv)
    allWinNums[length(allWinNums) + 1]<-modelWinNum
  }
  # so the number ordering is correct
  if (length(allWinNums) > 0)
  {
    allWinNums<-sort(allWinNums)
    for (i in 1:length(allWinNums))
    {
      curLabel<-paste("Window _", allWinNums[i], "  Ctrl+", allWinNums[i], 
                    sep="")
      newMenuItem<-addSubMenuItem(labelText=curLabel, action="activateWindow",
                             menuName="window", winNum=allWinNums[i])
    }
  }
  gtkMenuItemSetSubmenu(windowItem, windowMenu)

  menus<-get("menus", mvcEnv)
  menus[["window"]]<-windowMenu
  assign("menus", menus, mvcEnv)
}

################
# closeWin adds a callback to the window for the delete
# event, which closes all devices open in the window
################
closeWin<-function(plotItem)
{
  win<-win(plotItem)

  win$AddCallback("delete_event",
    function(x,y)
    {    
      # x is the widget and y is the event type
      setDeleteEvents(plotItem)    
      # 6/1/06 need to set active win num to NULL
      assign("activeWinNum", c(), mvcEnv)        
    }
  )
}

##########
# 6/1/06
# this function gets the MVC that the view with focus belongs to - does this
# using the winNum slot (which is stored in mvcEnv as activeWinNum)
#
# returns the MVC that corresponds with the view that has focus or it returns
# the active MVC if the MVC corresponding to the view with focus can't be 
# found
##########
getMVCFromWinNum<-function()
{
  aWinNum<-get("activeWinNum", mvcEnv)
  if (length(aWinNum)==0)
  {
    return(getMVC(get("activeMVC", mvcEnv)))
  }
  else
  {
    allMVCs<-getMVCList()
    # this will be a list of lists
    allViewLists<-lapply(allMVCs, viewList)

    winNumValues<-lapply(allViewLists, function(x)
     { unlist(lapply(x, function(y) 
       {         
         y@winNum
       } ))
     })
    winIndex<-unlist(lapply(winNumValues, function(x) aWinNum %in% x))
    if (any(winIndex))
    {
      curMVC<-allMVCs[[which(winIndex)]]
      return(curMVC)
    }
    else
      return(getMVC(get("activeMVC", mvcEnv)))
  }
}

##########
# this is the function to call to set the tooltips in the control environment
##########
setTooltipsGUI<-function()
{ 
  if (length(get("controlWin", mvcEnv)) > 0)
  {  
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    if (getNumberofViews())
    {
      # first create a gtk window that will show the tooltips options
      # need the window to be modal
      assign("notDone", TRUE, controlEnv)

      w<-gtkWindow(show=FALSE)
      vbox<-gtkVBoxNew()
      w$Add(vbox)
      lab<-gtkLabel("Tooltip Options")
      gtkBoxPackStart(vbox, lab)

      curModel<-model(curMVC)

      if (is(curModel, "graphModel"))
      {
        curData<-modelData(curModel)
        nodDat<-nodeData(curData)
        if (length(nodDat) > 0)
        {
          # for now assume that all nodes have the same attributes!
          allAttr<-unique(unlist(lAttr<-lapply(nodDat, names)))         
          extraTTs<-allAttr
        }
        else
          extraTTs<-c()
      }
      if (is(curModel, "dfModel"))
      {
        curData<-modelData(curModel)
        if (ncol(curData) > 0)
          extraTTs<-colnames(curData)
        else
          extraTTs<-c()
      }
      if (is(curModel, "exprModel"))
      {
        # haven't thought about this yet
        extraTTs<-c()
      }
      if (is(curModel, "gseModel"))
      {
        curData<-modelData(curModel)
        extraTTs<-c("gene set test statistics")
        # currently don't have p-values stored (put as a model variable)
        if (length(modelVar(curModel)) > 0)
          extraTTs<-c(extraTTs, names(modelVar(curModel)))
      }

      # now need to add radio buttons
      radButs<-list()
      if (is(curModel, "graphModel"))
        radBut<-gtkRadioButtonNewWithLabelFromWidget(label="Node Names")
      if (is(curModel, "dfModel"))
        radBut<-gtkRadioButtonNewWithLabelFromWidget(label="Row Names")
      if (is(curModel, "exprModel"))
        radBut<-gtkRadioButtonNewWithLabelFromWidget(label="Gene Names")
      if (is(curModel, "gseModel"))
        radBut<-gtkRadioButtonNewWithLabelFromWidget(label="Gene Set Names")
      radButs[[1]]<-radBut
      gtkBoxPackStart(vbox, radBut)
      if (length(extraTTs) > 0)
        for (i in 1:length(extraTTs))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                           label=extraTTs[i])
          gtkBoxPackStart(vbox, newradBut)
          radButs[[i+1]]<-newradBut
        }

      # now add a set button
      but<-gtkButton("Set Tooltips")
      gtkBoxPackStart(vbox, but)    

      gtkAddCallback(but, "clicked",
        function(obj, ev)
        {
          # find out which button is active
          for (i in 1:length(radButs))
          {      
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }

          if (is(curModel, "graphModel"))
          {
            # get the node names
            curNetwork<-modelData(curModel)
            nodeNames<-nodes(curNetwork)

            # then the tooltips will be the node names
            if (curIndex == 1)
            {
              # set the tooltips to the node name
              ttVec<-nodeNames
              names(ttVec)<-nodeNames   

              assign("ttToShow", ttVec, controlEnv)
            }
            else
            {
              attrName<-allAttr[(curIndex-1)]
              ttVec<-unlist(lapply(nodDat, function(x) {x[[attrName]]}))
              names(ttVec)<-nodeNames
              assign("ttToShow", ttVec, controlEnv)              
            }
          }
          if (is(curModel, "dfModel"))
          {
            # get the row names
            rNames<-rownames(curData)
            if (curIndex==1)
            {
              # set the tooltips to the row name
              ttVec<-rNames
              names(ttVec)<-rNames
              assign("ttToShow", ttVec, controlEnv)
            }
            else
            {
              cNames<-colnames(curData)
              ttVec<-curData[,(curIndex-1)]
              names(ttVec)<-rNames
              assign("ttToShow", ttVec, controlEnv)
            }
          }
          if (is(curModel, "gseModel"))
          {
            # get the gene set names
            gsNames<-rownames(incidMat(modelData(curModel)))
            # get the gene set statistics
            gsStats<-gsTestStat(modelData(curModel))
            if (curIndex==1)
            {
              ttVec<-gsNames
              names(ttVec)<-gsNames
              assign("ttToShow", ttVec, controlEnv)
            }
            else
            {
              if (curIndex==2)
              {
                # only want 2 numbers after the decimal
                ttVec<-round(gsStats, 2)
                names(ttVec)<-gsNames
                assign("ttToShow", ttVec, controlEnv)
              }
              else
              {
                # the tooltips should be a model variable
                modelVarNames<-names(modelVar(curModel))
                curMVName<-modelVarNames[curIndex-2]
                ttVec<-modelVar(curModel)[[curMVName]]
                names(ttVec)<-gsNames
                assign("ttToShow", ttVec, controlEnv)
              }
            }
          }
          w$Destroy()
          assign("notDone", FALSE, controlEnv)
          return(TRUE)
        }
      )
      gtkAddCallback(w, "delete_event",
        function(obj, ev)
        {
          assign("notDone", FALSE, controlEnv)
          w$Destroy()
          return(TRUE)
        }
      )
      w$Show()
      while (get("notDone", controlEnv))
        Sys.sleep(0.5)
    }
  }
  else
    print(paste("Please call the function, setTooltips,", 
            "to set the tooltip names.  The default tooltips are the",
            "row or node names from the data set, but the setTooltips", 
            "function lets the user set the tooltips to other values."))
}

#########
# created this function so tooltips could be called with a character string
# rather than a node object
#########
tooltipsNode<-function(curNode)
{
  if (is.null(curNode))
    nodeName<-NULL
  else
  {
#    nodeName<-name(curNode)
    nodeName<-curNode
  }

  tooltips(nodeName)
}

###########
# to show a tooltip over a node
###########
tooltips<-function(curName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # need to get the named vector which will be shown as tooltips
  # this must be set in a previous function
  ttToShow<-get("ttToShow", controlEnv)

  # if the tooltips have not been set then use the default of the row or node 
  # names
  if (length(ttToShow) == 0)
  {
    curModel<-model(curMVC)
    curData<-modelData(curModel)    
    if (is(curModel, "dfModel"))
      rNames<-rownames(curData)
    if (is(curModel, "graphModel"))
      rNames<-nodes(curData)
    if (is(curModel, "gseModel"))
      rNames<-rownames(incidMat(curData))

    # set the tooltips to the row or node name
    ttVec<-rNames
    names(ttVec)<-rNames
    ttToShow<-ttVec
  }

  if (!is.null(curName))
  {
    curIndex<-match(curName, names(ttToShow))

    if (any(is.na(curIndex)))
    {
      if (length(curName)==1)
        curToolTip<-curName
      else
        curToolTip<-paste(curName, collapse="\n")
    }
    else
    {
      if (length(curIndex)==1)
        curToolTip<-ttToShow[curIndex]
      else
        curToolTip<-paste(ttToShow[curIndex], collapse="\n")
    }
  }
  else
    curToolTip<-""

  # check if a tooltip is currently being shown
  oldName<-get("oldTTName", controlEnv)
  oldTTWin<-get("oldTTWin", controlEnv)
      
  if (is.null(curName))
  {          
    # no old name and no new name
    if (all(oldName==rep("", length(oldName))))
    {
      # nothing needs to be updated
    }
    else                            # old name was shown, but no new name
    {
      assign("oldTTName", "", controlEnv)
      if (length(oldTTWin) > 0)  # which it should be
      {
        oldTTWin$Destroy()
        assign("oldTTWin", c(), controlEnv)
      }
    }
  }
  else
  {
    # no old name, but a new name to show
    if (all(oldName==rep("", length(oldName))))
    {
      w<-gtkWindow(type=GtkWindowType[3], show=FALSE)
      lab<-gtkLabel(curToolTip)
      w$Add(lab)
      gtkWindowSetPosition(w, position=GtkWindowPosition[3])
      w$Show()

      assign("oldTTName", curName, controlEnv)
      assign("oldTTWin", w, controlEnv)          
    }
    else
    {
      if (all(oldName==curName))
      # old name and new name are the same
      {
        # nothing needs to be updated
      }
      else                          # old name is different from new name
      {
        # destroy old tooltip
        oldTTWin$Destroy()
            
        w<-gtkWindow(type=GtkWindowType[3], show=FALSE)
        lab<-gtkLabel(curToolTip)
        w$Add(lab)
        gtkWindowSetPosition(w, position=GtkWindowPosition[3])
        w$Show()

        assign("oldTTName", curName, controlEnv)
        assign("oldTTWin", w, controlEnv)                 
      }
    }
  }
}

##########
# 6/7/05 added
# highlight points as the cursor moves over them
##########
highPointByMotion<-function(rowName)
{
  # 5/31/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  mName<-modelName(model(curMVC))
  controlEnv<-controller(curMVC)
  model<-model(curMVC)
  virtualData<-virtualData(model)  

  oldMotionRow<-get("oldMotionRow", controlEnv)

  if (length(rowName) == 0)
    rowName<-""

  # note that if the 2 variables don't have the same length, then they
  # are not the same
  booSame<-TRUE
  if (length(rowName) != length(oldMotionRow))
    booSame<-FALSE
  else
    if (all(rowName != oldMotionRow))
      booSame<-FALSE

  if (!booSame)
  {
    colName<-"highlit"
    # remove old highlighting
    testOldEmpty<-rep("", length(oldMotionRow))
    testNewEmpty<-rep("", length(rowName))
    if (all(oldMotionRow != testOldEmpty))
    {
      # then need to toggle old highlighting
      highIndex<-match(colName, colnames(virtualData))
      rowIndex<-match(oldMotionRow, rownames(virtualData))
      highValue<-virtualData[rowIndex, highIndex]
      # toggle the highlighting
      newHValue<-!highValue

      mData<-as.list(newHValue)
      names(mData)<-oldMotionRow

      # update the data set
      dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData, 
                      dataName=modelName(model(curMVC)), from="")
      handleMessage(dfMessage)

      assign("oldMotionRow", "", controlEnv)
    }
    # add new highlighting
    if (all(rowName != testNewEmpty))
    {
      # then need to toggle old highlighting
      highIndex<-match(colName, colnames(virtualData))
      rowIndex<-match(rowName, rownames(virtualData))
      highValue<-virtualData[rowIndex, highIndex]
      # toggle the highlighting
      newHValue<-!highValue

      mData<-as.list(newHValue)
      names(mData)<-rowName

      # update the data set
      dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                      dataName=modelName(model(curMVC)), from="")
      handleMessage(dfMessage)

      assign("oldMotionRow", rowName, controlEnv)      
    }

    # now need to update the MVC
    # need to get the curMVC again because the model has changed!
    curMVC<-getMVC(mName)
    controller(curMVC)<-controlEnv

    MVCList<-get("MVCList", mvcEnv)

    # need to reassign the MVC
    allNames <- getModelNames(sort = FALSE)
    mvcIndex <- match(mName, allNames)
    MVCList[[mvcIndex]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)
  }  
}

############
# 6/21/06
# this function sets the name of the new child model - it is called within
# createChildHeatmap (needs to be called each time b/c will need a new name
# each time so it's not a preprocessing function)
############
getModelName<-function()
{
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # make the gtk window modal
  assign("notDone", TRUE, controlEnv)

  # need to get the name of the new model
  w<-gtkWindow(show=FALSE)
  hbox<-gtkHBox()
  w$Add(hbox)
  lbl<-gtkLabel("New Model Name")
  gtkBoxPackStart(hbox, lbl, padding=5)
  entName<-gtkEntry()

  gtkBoxPackStart(hbox, entName, padding=5)
  okBut<-gtkButton("  Ok  ")
  gtkBoxPackStart(hbox, okBut, padding=5)
  w$Show() 

  gtkAddCallback(okBut, "clicked",
    function(obj, ev)
    {
      curName<-gtkEntryGetText(entName)
      if (curName != "")
      {
        # check that this name has not been used
        allNames<-getModelNames()
        if (curName %in% allNames)
        {
          showLabel("That name is already in use.  Please use something different.")
        }
        else
        {
          assign("heatmapModName", curName, controlEnv)
          w$Destroy()
          assign("notDone", FALSE, controlEnv)
        }
      }
      return(TRUE)
    }
  )
  gtkAddCallback(w, "delete_event",
    function(obj, ev)
    {
      assign("notDone", FALSE, controlEnv)
      w$Destroy()
      return(TRUE)
    }
  )

  while (get("notDone", controlEnv))
    Sys.sleep(0.5) 
}

############
# 6/21/06 
# create a heatmap by interacting with a node
# will need to:
#  1. determine which genes are annotated at that node
#  2. ask parent (or ancestor) for exprSet data that corresponds with those 
#     genes
#  3. create a child MVC with these data 
#  4. create a heatmap plot for this new child
#  5. then reset the active MVC
############
createChildHeatmap<-function(curNode)
{
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # first need to determine which genes are annotated at this node
  nodeToRNEnv<-get("nodeToRNEnv", controlEnv)
  chipType<-get("chipType", controlEnv)
  rnToNodeEnv<-get("rnToNodeEnv", controlEnv)
  nodeParentsEnv<-get("nodeParentsEnv", controlEnv)

  # need to also realize that not all data from the parent (or ancestor) was 
  # used
  if ("subsetParentData" %in% ls(controlEnv))
    subsetParentData<-get("subsetParentData", controlEnv)
  else
    subsetParentData<-NULL

  if (all(curNode %in% ls(eval(as.name(nodeToRNEnv))))) 
  {
    allRN<-unlist(mget(curNode, eval(as.name(nodeToRNEnv))), 
                           use.names=FALSE)

    aMessage<-new("gAskAncestorMessage", type="exprModel",
                   mData=list("genes"=allRN), from=modelName(model(curMVC)))
    retValue<-handleMessage(aMessage)

    if (!is.null(retValue))
    {
      # need to check for subset
      if (!is.null(subsetParentData))
        retValue<-retValue[geneNames(retValue) %in% subsetParentData, ]
    }

    if (!is.null(retValue))
    {
      # now create the child model (MVC)
      # call the getModelName function to set the name of this new model
      if (length(get("controlWin", mvcEnv)) > 0)
      {
        getModelName()
      }
      if ("heatmapModName" %in% ls(controlEnv))
        newModelName<-get("heatmapModName", controlEnv)
      else
        newModelName<-""

      # only perform the following if the name is unique!
      if (!(newModelName  %in% getModelNames()) && newModelName != "")
      {
        # set up the virtual data slot
        oldVirData<-virtualData(model(curMVC))
        oldCol<-oldVirData[match(curNode, rownames(oldVirData)), 
                           match("fillcolor", colnames(oldVirData))]

        if (any(oldCol == "white"))
          oldCol[oldCol=="white"]<-"black"

        oldHighCol<-oldVirData[match(curNode, rownames(oldVirData)),
                             match("color", colnames(oldVirData))]
        if (oldHighCol == "black")
          oldHighlight<-FALSE
        else
          oldHighlight<-TRUE

        noGenes<-length(geneNames(retValue))

        newVirData<-data.frame(rep(oldCol, noGenes), 
                             row.names=geneNames(retValue))
        newVirData[,1]<-as.character(newVirData[,1])
        newVirData[,2]<-rep(1, noGenes)
        newVirData[,3]<-rep(oldHighlight, noGenes)
        newVirData[,4]<-rep(FALSE, noGenes)
        colnames(newVirData)<-c("color", "pch", "highlit", "hide")

        # need to get newModelName and newVirData!!!!!
        newAddChildMessage<-new("gAddChildMessage", dataName=newModelName,
                               type="exprSet", data=list(data=retValue,
                               virtualData=newVirData))

        handleMessage(newAddChildMessage, subsetParentData=curNode,
                          toParent=linkRowNameToNode,
                          fromParent=linkNodeToRowName)
        
        # need to set environment variables for linking
        chControlEnv<-controller(getMVC(newModelName))
        assign("chipType", chipType, chControlEnv)
        assign("nodeToRNEnv", nodeToRNEnv, chControlEnv)
        assign("rnToNodeEnv", rnToNodeEnv, chControlEnv)
        assign("nodeParentsEnv", nodeParentsEnv, chControlEnv)

        # then set this model as the active MVC and create a heatmap of it
        oldActiveMVC<-get("activeMVC", mvcEnv)
        setActiveMVC(newModelName)        
        createHeatmap()

        # then reset the active MVC
        setActiveMVC(oldActiveMVC)
      }
      else
      {
        if (length(get("controlWin", mvcEnv)) > 0)
        { 
          showLabel("Please use a unique name for the new model.")
        }
        else
        {
          print(paste("Please use a unique name for the new model. ", 
                      "This can be set by calling the setModelName function."))
        }
      }
    }  
  }
}

###########
# to hide a node
# what about connecting edges??
# will also need to hide any edges that connect to this node and that will
# require redrawing the whole graph because there is no drawAgEdge function
# (ask Jeff about this)
# hiding a node is a property that can be toggled
###########
hideNode<-function(curNode)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)
  virData<-virtualData(model(curMVC))

  # for now assume that hiding a node uses a white color
  hideColor<-"white"
  unHideColor<-"black"

  # need to get the color for the current node
#  curName<-name(curNode)
  curName<-curNode
  curHide<-virData[match(curName, rownames(virData)), 
                     match("hide", colnames(virData))]

  newData<-!curHide
#  if (curHide == TRUE)
#    newData<-FALSE
#  else
#    newData<-TRUE

#  names(newData)<-name(curNode)
  names(newData)<-curNode
  newData<-as.list(newData)

  # change the type so it has more info about what was done to the node
  newType<-c("node"="hide")
  newdfMessage<-new("gUpdateDataMessage", type=newType, mData=newData,
                    dataName=modelName(model(curMVC)), from="")
  handleMessage(newdfMessage)
}

###########
# to highlight a node
# would we want to toggle highlighting a node like is done in a scatterplot?
# pass in node name rather than an AgNode object
###########
highlightNode<-function(curNode)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)
  virData<-virtualData(model(curMVC))

  # for now assume that highlighting a node uses a red color
  highlightColor<-"red"
  unHighlightColor<-"black"
  
  # need to get the color for the current node
#  curName<-name(curNode)
  curName<-curNode
  curColor<-virData[match(curName, rownames(virData)), 
                     match("color", colnames(virData))]

  if (curColor != highlightColor)
    newData<-highlightColor
  else
    newData<-unHighlightColor

#  names(newData)<-name(curNode)
  names(newData)<-curNode
  newData<-as.list(newData)

  # the type is color because that is the attribute of the node that
  # we are changing
  newType<-c("node"="color")
  newdfMessage<-new("gUpdateDataMessage", type=newType, mData=newData,
                    dataName=modelName(model(curMVC)), from="")
  handleMessage(newdfMessage)
}

###########
# to color a node
###########
colorNode<-function(curNode)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # this is set in createColorBrowser, the preprocessing function
  currentColor<-get("currentColor", controlEnv)

  newData<-currentColor
#  names(newData)<-name(curNode)
  names(newData)<-curNode
  newData<-as.list(newData)

  # we are actually changing the fillcolor
  newType<-c("node"="fillcolor")
  newdfMessage<-new("gUpdateDataMessage", type=newType, mData=newData,
                    dataName=modelName(model(curMVC)), from="")
  handleMessage(newdfMessage)
}

##########
# a callback function for coloring a point in a scatterplot
##########
colorPoint<-function(rowName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # column name
  colName<-c("color", "pch")

  # need to change the virtualData slot in model object 
  # using messages
  curColor<-get("currentColor", controlEnv)
  curPch<-as.numeric(19)

  # changed 2/15/06 because pch is being converted into character by 
  # the old method!
  mData<-vector(mode="list", length=(length(rowName)*2))
  mData[1:length(rowName)]<-rep(curColor, length(rowName))
  mData[(length(rowName)+1):(length(rowName)*2)]<-rep(curPch, length(rowName))

  names(mData)<-rep(rowName, 2)

  # if a point was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# added 4/26/06
# a callback function to color a gene (for an exprModel)
##########
colorGene<-function(geneName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # column name
  colName<-c("color", "pch")

  # need to change the virtualData slot in model object 
  # using messages
  curColor<-get("currentColor", controlEnv)
  curPch<-as.numeric(19)

  # changed 2/15/06 because pch is being converted into character by 
  # the old method!
  mData<-vector(mode="list", length=(length(geneName)*2))
  mData[1:length(geneName)]<-rep(curColor, length(geneName))
  mData[(length(geneName)+1):(length(geneName)*2)]<-
             rep(curPch, length(geneName))
  names(mData)<-rep(geneName, 2)

  # if a gene was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# added 4/26/06
# a callback function to hide a gene (for an exprModel)
##########
hideGene<-function(geneName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  virtualData<-virtualData(model(curMVC))

  # column name
  colName<-"hide"

  # need to change the virtualData slot in model object 
  # using messages
  # need to toggle the hide value for this row
  # so need to get current value
  colIndex<-match(colName, colnames(virtualData))
  rowIndex<-match(geneName, rownames(virtualData))
  curHValue<-virtualData[rowIndex, colIndex]
  newHValue<-!curHValue

  mData<-as.list(newHValue)
  names(mData)<-geneName

  # if a gene was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# added 4/25/06
# a callback function to highlight a gene (for an exprModel)
##########
highlightGene<-function(geneName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  virtualData<-virtualData(model(curMVC))

  # column name
  colName<-"highlit"

  # need to change the virtualData slot in model object 
  # using messages
  # need to toggle the highlit value for this row
  # so need to get current value
  colIndex<-match(colName, colnames(virtualData))
  rowIndex<-match(geneName, rownames(virtualData))
  curHValue<-virtualData[rowIndex, colIndex]
  newHValue<-!curHValue

  mData<-as.list(newHValue)
  names(mData)<-geneName

  # if a gene was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# added 6/7/05 
# a callback function to highlighting a point
##########
highlightPoint<-function(rowName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  virtualData<-virtualData(model(curMVC))

  # column name
  colName<-"highlit"

  # need to change the virtualData slot in model object 
  # using messages
  # need to toggle the highlit value for this row
  # so need to get current value
  colIndex<-match(colName, colnames(virtualData))
  rowIndex<-match(rowName, rownames(virtualData))
  curHValue<-virtualData[rowIndex, colIndex]
  newHValue<-!curHValue

  mData<-as.list(newHValue)
  names(mData)<-rowName

  # if a point was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# added 6/7/05 
# a callback function to hide a point
##########
hidePoint<-function(rowName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/31/06 added so that users can interact with views that don't belong to
  # the active MVC
  curMVC<-getMVCFromWinNum()
  virtualData<-virtualData(model(curMVC))

  # column name
  colName<-"hide"

  # need to change the virtualData slot in model object 
  # using messages
  # need to toggle the hide value for this row
  # so need to get current value
  colIndex<-match(colName, colnames(virtualData))
  rowIndex<-match(rowName, rownames(virtualData))
  curHValue<-virtualData[rowIndex, colIndex]
  newHValue<-!curHValue

  mData<-as.list(newHValue)
  names(mData)<-rowName

  # if a point was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# createKeyPressEvent is to add pseudo-accelerators to any 
# gtk window
##########
createKeyPressEvent<-function(w)
{
  # add a key press event to the window so it can seem like the user can use
  # the accelerators from the control window on any active gtk window
  gtkAddCallback(w,"key-press-event",
    function(obj,ev)
    {
      curStr<-gdkEventKeyGetString(ev)
      curState<-gdkEventKeyGetState(ev)
      curKeyval<-gdkEventKeyGetKeyval(ev)

      setKeyValToAction(curStr, curState, curKeyval)
      return(TRUE)
    }
  )
}

##############
# setKeyValToAction uses the controlEnv variable, keyVals, to match
# a key press to the appropriate action
##############
setKeyValToAction<-function(curStr, curState, curKeyval)
{
  keyVals<-get("keyVals", mvcEnv)

  assignedKeyVals<-unlist(lapply(keyVals, function(x){x$keyval}))
  assignedCurActions<-unlist(lapply(keyVals, function(x){x$action}))
  
  # what if the case is upper vs. lower?
  # note: having the Caps lock down changes the state
  # curState=4 when Ctrl has been pressed
  # curState=6 when Caps Lock is on and Ctrl has been pressed
  if (curState == 4 || curState == 6)
  {
    index<-which(assignedKeyVals %in% curKeyval)
    
    # first test if we just have the wrong capitalization
    if (length(index)==0)
    {
      # 32 is the difference between lower and upper caps on the same letter
      index<-which(assignedKeyVals %in% (curKeyval+32))
    }

    if (length(index) > 0)
    {
      if (length(index) > 1)
      {
        # need to find the correct index; the one where curAction != ""
        index<-index[assignedCurActions[index] != ""]
      }

      curAction<-assignedCurActions[index]

      # check if there are any extra parameters
      # each keyVals item should have 3 elements: keyval, action, and
      # accelChar so anything after the 3 is an extra parameter 
      if (length(keyVals[[index]]) > 3)
      {
        totalLen<-length(keyVals[[index]])
        updateCurAction(curAction, unlist(keyVals[[index]][4:totalLen]))
      }
      else
      {
        updateCurAction(curAction)
      }
    }
  }
}

##########
# sets the focus on a window
# is used in addEventsforViews in iSPlotfuns.R
##########
activateWindow<-function(winNum)
{
  if (winNum==0)
  {
    controlWin<-get("controlWin", mvcEnv)
    if (length(controlWin) > 0)
    {
#      gtkWindowActivateFocus(controlWin)
      gtkObjectSignalEmit(controlWin, "hide")
      gtkObjectSignalEmit(controlWin, "show")
    }
  }
  else
  {
    MVCList<-get("MVCList", mvcEnv)
    # need to get the window from MVCList
    if (length(MVCList) > 0)
    {
      for (i in 1:length(MVCList))
      {
        curviewList<-viewList(MVCList[[i]])
        curWinNums<-unlist(lapply(curviewList, function(x) {winNum(x)}))
        curWins<-lapply(curviewList, function(x) {win(x)})
        index<-match(winNum, curWinNums)
        if (!is.na(index))
        {
          gtkObjectSignalEmit(curWins[[index]], "hide")
          gtkObjectSignalEmit(curWins[[index]], "show")
        }
      }
    }
    if (length(get("modelWin", mvcEnv)) > 0)
    {
      index<-match(winNum, get("modelWinNum", mvcEnv))
      if (!is.na(index))
      {
        gtkObjectSignalEmit(get("modelWin", mvcEnv), "hide")
        gtkObjectSignalEmit(get("modelWin", mvcEnv), "show")
      }
    }
  }
}

##########
# added 6/8/05
# when a new callback function is linked to an event want to remove
# any selections on a spreadsheet view
##########
removeSpreadSelect<-function()
{
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)

  viewList<-viewList(curMVC)
  controlEnv<-controller(curMVC)

  if (length(viewList) > 0)
  {
    booSpread<-any(unlist(lapply(viewList, function(x) {is(x,
                                           "spreadView")})))
    if (booSpread)
    {
      # don't want to cause a click event to occur for the spreadsheet
      # so set contLoop variable to TRUE before unselecting rows
      assign("contLoop", TRUE, controlEnv)
      controller(curMVC)<-controlEnv
      MVCList<-get("MVCList", mvcEnv)
      allNames <- getModelNames(sort = FALSE)
      index <- match(activeMVC, allNames)
      MVCList[[index]]<-curMVC
      assign("MVCList", MVCList, mvcEnv)
      
      spreadIndex<-which(unlist(lapply(viewList, function(x) {is(x, 
                                           "spreadView")})))
      curSpread<-viewList[[spreadIndex]]
      curCList<-clist(curSpread)
      gtkCListUnselectAll(curCList)

      # now need to reset contLoop variable to FALSE
      curMVC<-getMVC(activeMVC)
      curVList<-viewList(curMVC)
      controlEnv<-controller(curMVC)

      assign("contLoop", FALSE, controlEnv)
      controller(curMVC)<-controlEnv
      MVCList<-get("MVCList", mvcEnv)
      allNames <- getModelNames(sort = FALSE)
      index <- match(activeMVC, allNames)
      MVCList[[index]]<-curMVC
      assign("MVCList", MVCList, mvcEnv)
    }
  }
}

##########
# different from the iSPlot version
# check all of the loaded models and return only those that are of type dfModel
##########
getLoadedDF<-function()
{
  mNames<-NULL
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    modelList<-lapply(MVCList, model)
    # need to check the class of model
    booDF<-unlist(lapply(modelList, testDfClass))
    if (any(booDF))
    {
      dfModels<-modelList[booDF]
      mNames<-sort(unlist(lapply(dfModels, modelName)))
    }
  }
  return(mNames)
}

########
# check if x is of class dfModel
########
testDfClass<-function(x)
{
  is(x, "dfModel")
}

################
# getPlotDev makes a vector of the device element in the plotList
################
getPlotDev<-function(plotList)
{
  dev.ele<-lapply(plotList, function(x)
                           {
                             if (is(x, "plotView"))
                               return(plotDevice(x))
                             else
                               return(0)
                           })
  dev.ele<-unlist(dev.ele)
  return(dev.ele)
}


