
mvcEnv<-new.env(parent=.GlobalEnv)

##########
# these are stored in the mvcEnv
# the global variables all deal with the control window appearance that
# is applicable to all MVCs
##########
setGlobalEnvVariables<-function()
{
  # controlWin is the gtkWindow
  assign("controlWin", c(), mvcEnv)
  # vBox is the vertical box that contains the main menu and frame, frMVC
  assign("vBox", c(), mvcEnv)
  # frMVC will hold the table
  assign("frMVC", c(), mvcEnv)  
  # for adding menu items to the main menu
  # ag is the accelerator group
  assign("ag", c(), mvcEnv)

  # the current action for the control window view
  assign("curAction", "", mvcEnv)
  # this is the last action that changed the control window appearance
  assign("lastCWAction", "", mvcEnv)

  # the default control window dimensions
  assign("winDefaultHeight", 500, mvcEnv)
  assign("winDefaultWidth", 300, mvcEnv)

  # menuBar is the main menu bar (all menus are added to the menu bar)
  assign("menuBar", c(), mvcEnv)
  # menus is the list of menus (such as file, edit, display, ...)
  assign("menus", list(), mvcEnv)

  # to store the window menu item - needed to update the menu when
  # sub menu items are added or removed from the window menu
  assign("windowItem", c(), mvcEnv)
  # to count the number of windows that have been opened
  assign("winCounter", 0, mvcEnv)

  # need to store this to determine which views can be created depending
  # on the model class that is stored in the active MVC
  assign("displayItem", c(), mvcEnv)

  # these are the global key values that all windows will share
  # need to store the key values for the menu item accelerators
  assign("keyVals", list(), mvcEnv)

  # the possible events that the user can match a callback function to;
  # each event will have its own list in controlEnv that contains
  # all the gEventFun objects that can be linked to this event
  assign("possEvents", c("mouseOver", "leftButtonClick", 
    "middleButtonClick", "rightButtonClick"), mvcEnv)

  # store the model name of the active MVC
  assign("activeMVC", "", mvcEnv)
  # store the active view's winNum - so when a view is interacted with, it can
  # be determined which MVC to update
  assign("activeWinNum", c(), mvcEnv)
  # the list of created MVCs
  assign("MVCList", list(), mvcEnv)
  # store the old active MVC so its key vals can be removed
  assign("oldActiveMVC", "", mvcEnv)  

  assign("defaultPlotData", list(color="black", pch=1, highlit=FALSE, 
        hide=FALSE), mvcEnv)

  # 3/9/05 assign type of model 
  assign("typeModel", "", mvcEnv)

  # 12/2/05 this list will include new callback functions that the user
  # has added from the command line (the function, createDefaultCallEvents, 
  # will check this variable when a new MVC object is loaded)
  assign("newCallFuns", list(), mvcEnv)

  # 3/3/06 this list stores any new methods to create child MVCs 
  assign("newChildMethods", list(), mvcEnv)

  # store the graph of models
  assign("modelGraph", c(), mvcEnv)
  # store the window that holds the plot of the model graph
  assign("modelWin", c(), mvcEnv)
  # store the device that holds the plot of the model graph
  assign("modelDevice", c(), mvcEnv)
  # store the window number
  assign("modelWinNum", 0, mvcEnv)

  # want to reassign these variables as well
  createMenuItemVariables()
}

##########
# set the defaults for the control environment variables
##########
setControllerDefaults<-function(newMVC)
{
  controlEnv<-controller(newMVC)

  assign("currentColor", "black", controlEnv)

  # to make a window modal
  assign("notDone", TRUE, controlEnv)

  # need to store the key values for the menu item accelerators
  assign("localKeyVals", list(), controlEnv)

  # either the name of the new file or the contents of the file
  assign("loadedFile", NULL, controlEnv)
  # the name of the new loaded file
  assign("curName", "", controlEnv)

  # for the old node name that was shown in a tooltip
  assign("oldTTName", "", controlEnv)
  # for the old tooltip gtk window
  assign("oldTTWin", c(), controlEnv)
  # for the named vector of tooltips
  assign("ttToShow", c(), controlEnv)

  # a list for each event that can link to gEventFun objects
  # each list element should be a list with 2 elements
  # the first element is the gEventFun object and the second
  # element is whether this function is currently linked to the event
  assign("mouseOver", list(), controlEnv)
  assign("leftButtonClick", list(), controlEnv)
  assign("middleButtonClick", list(), controlEnv)
  assign("rightButtonClick", list(), controlEnv)
  # to record which events the user wants to add the function to
  assign("selEvents", c(), controlEnv)

  # need an active plot window as well as an active device
  # this is needed for tooltips so that you don't end up with a 
  # open tooltip window that should have been closed
  assign("activePlotWindow", c(), controlEnv)

  # have an active view for each MVC
  assign("activeView", c(), controlEnv)

  # 7/12/05 the name of the method to create a child model
  assign("childModel", "", controlEnv)
  # the name of the child model
  assign("childName", "", controlEnv)

  # 7/14/05 boolean variables to determine whether parent/children should
  # be updated when a data change occurs
  assign("sendToParent", TRUE, controlEnv)
  assign("sendToChild", TRUE, controlEnv)

  # 8/27/05 this is used when creating a child model
  assign("curVariable", "", controlEnv)
 
  # 3/14/05 store the chip type to link affy ids to locus link ids 
  assign("chipType", "", controlEnv)

  # need this variable when selecting a row in spreadsheet so it doesn't
  # end up in an infinite loop (any model can have a spreadsheet)
  assign("contLoop", FALSE, controlEnv)

  # extra controller variables that are unique to a particular type of model
  if (is(model(newMVC), "dfModel"))
  {
    # this is stored for motion notify event
    assign("oldMotionRow", "", controlEnv)
    # want a named vector
    # the names of the vector are the functions to call
#    assign("createChild", c(createSubsetGUI=c("Create Subset Model"),
#           createMDSCluster=c("Perform Multidimensional Scaling (MDS)"),
#           createGoGraphGUI=c("Create GO Graph")),
#           controlEnv)
    assign("createChild", c(createSubsetGUI=c("Create Subset Model"),
           createMDSCluster=c("Perform Multidimensional Scaling (MDS)")),
           controlEnv)
    # create a variable to store functions to create a child model without 
    # using the GUI (these are command line functions)
#    assign("createChildNoGUI", c(createSubset=c("Create Subset Model"),
#           performMDS=c("Perform Multidimensional Scaling (MDS)"),
#           createGoGraph=c("Create GO Graph")),
#           controlEnv)
    assign("createChildNoGUI", c(createSubset=c("Create Subset Model"),
           performMDS=c("Perform Multidimensional Scaling (MDS)")),
           controlEnv)

    # need to check if there are extra methods to add
    newChildMethods<-get("newChildMethods", mvcEnv)
    if (length(newChildMethods) > 0)
    {
      allTypes<-unlist(lapply(newChildMethods, function(x) {x$typeModel}))
      if ("dfModel" %in% allTypes)
      {
        dfIndex<-which(allTypes %in% "dfModel")
        for (i in 1:length(dfIndex))
        {
          curcrChild<-get("createChildNoGUI", controlEnv)
          curLength<-length(curcrChild)
          curcrChild[(curLength+1)]<-newChildMethods[[dfIndex[i]]]$shortName
          names(curcrChild)[(curLength+1)]<-
                                  newChildMethods[[dfIndex[i]]]$methodName
          assign("createChildNoGUI", curcrChild, controlEnv)
        }
      }
    }
  } 
  if (is(model(newMVC), "graphModel"))
  {
    # currently don't have methods to create child of a graph model
    assign("createChild", c(), controlEnv)
    # create a variable to store functions to create a child model without 
    # using the GUI (these are command line functions)
    assign("createChildNoGUI", c(), controlEnv)

    # need to check if there are extra methods to add
    newChildMethods<-get("newChildMethods", mvcEnv)
    if (length(newChildMethods) > 0)
    {
      allTypes<-unlist(lapply(newChildMethods, function(x) {x$typeModel}))
      if ("graphModel" %in% allTypes)
      {
        grIndex<-which(allTypes %in% "graphModel")
        for (i in 1:length(grIndex))
        {
          curcrChild<-get("createChildNoGUI", controlEnv)
          curLength<-length(curcrChild)
          curcrChild[(curLength+1)]<-newChildMethods[[grIndex[i]]]$shortName
          names(curcrChild)[(curLength+1)]<-
                                  newChildMethods[[grIndex[i]]]$methodName
          assign("createChildNoGUI", curcrChild, controlEnv)
        }
      }
    }
  }
  if (is(model(newMVC), "gseModel"))
  {
    # this is stored for motion notify event
    assign("oldMotionRow", "", controlEnv)
    # currently no methods to create a child!
  }
  if (is(model(newMVC), "exprModel"))
  {
    # 4/30/06 to store the names of the genes in the dendrogram
    assign("dendNames", c(), controlEnv)
#    assign("createChild", 
#           c(createMDSCluster=c("Perform Multidimensional Scaling (MDS)"),
#           createGoGraphGUI=c("Create GO Graph"),  
#           createKEGGGSEGUI=c("Perform Gene Set Enrichment using KEGG"),
#         createChrLocGSEGUI=
#                         c("Perform Gene Set Enrichment using Chromosome")),
#                         controlEnv)
    assign("createChild", 
           c(createGoGraphGUI=c("Create GO Graph"),  
           createKEGGGSEGUI=c("Perform Gene Set Enrichment using KEGG"),
         createChrLocGSEGUI=c("Perform Gene Set Enrichment using Chromosome")),
           controlEnv)
    # create a variable to store functions to create a child model without 
    # using the GUI (these are command line functions)
#    assign("createChildNoGUI", 
#           c(performMDS=c("Perform Multidimensional Scaling (MDS)"),
#           createGoGraph=c("Create GO Graph"), 
#           createKEGGGSE=c("Perform Gene Set Enrichment using KEGG"),
#           createChrLocGSE=c("Perform Gene Set Enrichment using Chromosome")),
#           controlEnv)
    assign("createChildNoGUI", 
           c(createGoGraph=c("Create GO Graph"), 
           createKEGGGSE=c("Perform Gene Set Enrichment using KEGG"),
           createChrLocGSE=c("Perform Gene Set Enrichment using Chromosome")),
           controlEnv)

    # need to check if there are extra methods to add
    newChildMethods<-get("newChildMethods", mvcEnv)
    if (length(newChildMethods) > 0)
    {
      allTypes<-unlist(lapply(newChildMethods, function(x) {x$typeModel}))
      if ("exprModel" %in% allTypes)
      {
        exIndex<-which(allTypes %in% "exprModel")
        for (i in 1:length(exIndex))
        {
          curcrChild<-get("createChildNoGUI", controlEnv)
          curLength<-length(curcrChild)
          curcrChild[(curLength+1)]<-newChildMethods[[exIndex[i]]]$shortName
          names(curcrChild)[(curLength+1)]<-
                                  newChildMethods[[exIndex[i]]]$methodName
          assign("createChildNoGUI", curcrChild, controlEnv)
        }
      }
    }
  }
}

#############
# createControlWindow creates a gtk window that controls
# what happens when interacting with a plot or when linking
# plots
#############
createControlWindow<-function()
{
  # check to ensure that a control window is not already open
  if (length(ls(env=mvcEnv)) > 0)
  {
    oldWin<-get("controlWin", mvcEnv)
  }
  else
  {
    setGlobalEnvVariables()
    oldWin<-get("controlWin", mvcEnv)
  }

  if (length(oldWin)==0)
  {
    w<-gtkWindow(show=FALSE)

    # the vertical box will never be removed from the window, but
    # the table will be removed depending on which menu item is
    # chosen (table will be added later)
    vBox<-gtkVBoxNew()
    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    assign("vBox", vBox, mvcEnv)
    assign("frMVC", frMVC, mvcEnv)
    assign("controlWin", w, mvcEnv)

    menuBar<-createMainMenu()

    # need to create boxes or frames
    gtkBoxPackStart(vBox, menuBar, expand=FALSE, fill=FALSE)
    gtkBoxPackStart(vBox, frMVC, expand=TRUE, fill=TRUE)

    gtkContainerAdd(w, vBox)

    gtkObjectSetArgs(w, "allow_shrink"=TRUE)
    gtkObjectSetArgs(w, "allow_grow"=TRUE)
    gtkObjectSetArgs(w, "auto_shrink"=TRUE)
    gtkContainerSetResizeMode(w, GtkResizeMode[1])

    w$SetUsize(get("winDefaultHeight", mvcEnv),
               get("winDefaultWidth", mvcEnv))

    w$Show()

    # need a delete event callback in case the user closes
    # the window without using the Quit menu item
    gtkAddCallback(w, "delete_event",
      function(obj, ev)
      {
        # only reset the global env variables if there are no loaded MVCs
        if (length(get("MVCList", mvcEnv)) == 0)
          setGlobalEnvVariables()
        else
          assign("controlWin", c(), mvcEnv)
        w$Destroy()
        return(TRUE)
      }
    )
  }

  invisible(NULL)
}

############
# create the list of gEventFun objects that are loaded initially
############
createDefaultCallEvents<-function(newMVC)
{
  # callEvents for the graph model must have a callback function that takes
  # a node as the only parameter
  if (is(model(newMVC), "graphModel"))
  { 
    # add gEventFun objects for tooltips and coloring nodes
    addCallEvent(tooltipsNode, "show tooltips over nodes", 
               list(setTooltipsGUI), "mouseOver", newMVC)
    # add an event for coloring a node
    addCallEvent(colorNode, "color a node", list(createColorBrowser),
               c("leftButtonClick", "middleButtonClick", "rightButtonClick"),
               newMVC) 
    # add an event for hiding a node
    addCallEvent(hideNode, "hide a node", NULL,
               c("leftButtonClick", "middleButtonClick", "rightButtonClick"),
               newMVC)
    # add an event for highlighting a node
    addCallEvent(highlightNode, "highlight a node", NULL,
               c("leftButtonClick", "middleButtonClick", "rightButtonClick"),
               newMVC)
    addCallEvent(createChildHeatmap, "create a heatmap", NULL,
               c("leftButtonClick", "middleButtonClick", "rightButtonClick"),
               newMVC)

    # check to see if the user has added any callback functions through
    # the command line
    newCallFuns<-get("newCallFuns", mvcEnv)
    if (length(newCallFuns) > 0)
    {
      for (i in 1:length(newCallFuns))
      {
        curCallFun<-newCallFuns[[i]]
        # now the user will also need to give the modelType when adding new
        # callback functions through the command line (for example colorNode
        # as a callback function only works with a graphModel)
        if ("graphModel" %in% curCallFun$modelType)
          addCallEvent(curCallFun$callFunction, curCallFun$shortName, 
                     curCallFun$preprocessingFun, curCallFun$eventLists, 
                     newMVC)
      }
    }
  }
  # callEvents for the df model must have a callback function that takes
  # a row name as the only parameter
  if (is(model(newMVC), "dfModel"))
  {
    addCallEvent(tooltips, "show tooltips over points", list(setTooltipsGUI), 
                 "mouseOver", newMVC)
    addCallEvent(highPointByMotion, 
                 "highlight points as cursor moves over them", NULL,
                 "mouseOver", newMVC)
    addCallEvent(colorPoint, "color a point", list(createColorBrowser),
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    # there is no preprocessing function for hiding a point
    addCallEvent(hidePoint, "hide a point", NULL, 
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    addCallEvent(highlightPoint, "highlight a point", NULL,
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)

    # check to see if the user has added any callback functions through
    # the command line
    newCallFuns<-get("newCallFuns", mvcEnv)
    if (length(newCallFuns) > 0)
    {
      for (i in 1:length(newCallFuns))
      {
        curCallFun<-newCallFuns[[i]]
        # now the user will also need to give the modelType when adding new
        # callback functions through the command line (for example colorPoint
        # as a callback function only works with a dfModel)
        if ("dfModel" %in% curCallFun$modelType)
          addCallEvent(curCallFun$callFunction, curCallFun$shortName, 
                     curCallFun$preprocessingFun, curCallFun$eventLists, 
                     newMVC)
      }
    }
  }

  if (is(model(newMVC), "gseModel"))
  {
    addCallEvent(tooltips, "show tooltips over points", list(setTooltipsGUI), 
                 "mouseOver", newMVC)
    addCallEvent(highPointByMotion, 
                 "highlight points as cursor moves over them", NULL,
                 "mouseOver", newMVC)
    addCallEvent(colorPoint, "color a point", list(createColorBrowser),
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    # there is no preprocessing function for hiding a point
    addCallEvent(hidePoint, "hide a point", NULL, 
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    addCallEvent(highlightPoint, "highlight a point", NULL,
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)

    # check to see if the user has added any callback functions through
    # the command line
    newCallFuns<-get("newCallFuns", mvcEnv)
    if (length(newCallFuns) > 0)
    {
      for (i in 1:length(newCallFuns))
      {
        curCallFun<-newCallFuns[[i]]
        # now the user will also need to give the modelType when adding new
        # callback functions through the command line (for example colorPoint
        # as a callback function only works with a dfModel)
        if ("dfModel" %in% curCallFun$modelType)
          addCallEvent(curCallFun$callFunction, curCallFun$shortName, 
                     curCallFun$preprocessingFun, curCallFun$eventLists, 
                     newMVC)
      }
    }
  }

  # 4/22/06 temporarily add a callback function for a heatmap view
  # will need to think about this more carefully!!!!
  if (is(model(newMVC), "exprModel"))
  {
    addCallEvent(highlightGene, "highlight a gene", NULL,
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    addCallEvent(colorGene, "color a gene", list(createColorBrowser),
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    addCallEvent(hideGene, "hide a gene", NULL, 
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)   

    # check to see if the user has added any callback functions through
    # the command line
    newCallFuns<-get("newCallFuns", mvcEnv)
    if (length(newCallFuns) > 0)
    {
      for (i in 1:length(newCallFuns))
      {
        curCallFun<-newCallFuns[[i]]
        # now the user will also need to give the modelType when adding new
        # callback functions through the command line (for example colorPoint
        # as a callback function only works with a dfModel)
        if ("exprModel" %in% curCallFun$modelType)
          addCallEvent(curCallFun$callFunction, curCallFun$shortName, 
                     curCallFun$preprocessingFun, curCallFun$eventLists, 
                     newMVC)
      }
    }
  }
}

#############
# to add a gEventFun object to one of the event lists in controlEnv
#  callFunction is callFun in gEventFun
#  shortName is shortName in gEventFun
#  preprocessingFun is preprocessFun in gEventFun
#  eventLists is a vector of the events that this gEventFun object can 
#   be linked to (each of these should be an environment variable in 
#   controlEnv and should be an element in possEvents in controlEnv)
#############
addCallEvent<-function(callFunction, shortName, preprocessingFun, eventLists, newMVC)
{
  controlEnv<-controller(newMVC)
  if (all(eventLists %in% get("possEvents", mvcEnv)))
  {
    # create new gEventFun object
    newEventFun<-new("gEventFun", callFun=callFunction, shortName=shortName,
                     preprocessFun=preprocessingFun)

    for (i in 1:length(eventLists))
    {
      curEventList<-get(eventLists[i], controlEnv)
    
      # then add to list and reassign
      eventLen<-length(curEventList)
      # when adding a new element to the event list, assume the event is not
      # currently linked to this callback function - that will be assigned
      # later
      curEventList[[eventLen+1]]<-list(eventFun=newEventFun, assigned=FALSE)

      assign(eventLists[i], curEventList, controlEnv)
    }
  }
}

#############
# createMainMenu creates a top level menu for
# the control window
#
# note: don't need to pass the window because it can 
# be accessed through the environment variable, controlWin
#############
createMainMenu<-function()
{
  # list of menus to store in controlEnv
  menus<-list()

  ag<-gtkAccelGroup()
  win<-get("controlWin", mvcEnv)
  gtkAccelGroupAttach(ag, win)
  assign("ag", ag, mvcEnv)

  #following the code in gtk.org Menu example
  # want to see if callbacks work
  fileMenu<-gtkMenuNew()
  editMenu<-gtkMenuNew()  
  eventMenu<-gtkMenuNew()

  # added the control part to the label because it wasn't automatically
  # doing it using the GtkAccelFlags like it's supposed to (I think)
  fileItems<-get("fileItems", mvcEnv)
  for (i in 1:length(fileItems))
  {
    newItem<-createMenuItem(fileItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], fileItems[[i]]$action)
    gtkMenuAppend(fileMenu, newItem)
  }  

  editItems<-get("editItems", mvcEnv)
  for (i in 1:length(editItems))
  {
    newItem<-createMenuItem(editItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], editItems[[i]]$action)
    gtkMenuAppend(editMenu, newItem)
  }

  eventItems<-get("eventItems", mvcEnv)
  for (i in 1:length(eventItems))
  {
    newItem<-createMenuItem(eventItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], eventItems[[i]]$action)
    gtkMenuAppend(eventMenu, newItem)
  }

  menuBar<-gtkMenuBarNew()
  gtkWidgetShow(menuBar)
  
  # GdkModifierType[4] is alt
  fileItem<-createMenuItem("_File", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  editItem<-createMenuItem("_Edit", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  displayItem<-createMenuItem("_Display", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])  

  eventItem<-createMenuItem("E_vent", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2]) 

  windowItem<-createMenuItem("_Window", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  assign("windowItem", windowItem, mvcEnv)
  assign("displayItem", displayItem, mvcEnv)

  gtkMenuItemSetSubmenu(fileItem, fileMenu)
  gtkMenuItemSetSubmenu(editItem, editMenu)
  gtkMenuItemSetSubmenu(eventItem, eventMenu)

  gtkMenuBarAppend(menuBar, fileItem)
  gtkMenuBarAppend(menuBar, editItem)
  gtkMenuBarAppend(menuBar, displayItem)
  gtkMenuBarAppend(menuBar, eventItem)
  gtkMenuBarAppend(menuBar, windowItem)

  menus[["file"]]<-fileMenu
  menus[["edit"]]<-editMenu
  menus[["event"]]<-eventMenu
  assign("menus", menus, mvcEnv)

  assign("menuBar", menuBar, mvcEnv)

  addWindowMenu()

  if (length(getActiveMVC()) > 0)
  {
    setDisplayMenu(getActiveMVC())
  }

  return(menuBar)
}

############
# create one menu item with an accelerator and possibly, a callback
############
createMenuItem<-function(labelText, accelGroup, method, modType, 
                         accelFlag, curAction="", ...)
{
  newItem<-gtkMenuItemNew()
  newLabel<-gtkAccelLabelNew(labelText)
  newKey<-gtkLabelParseUline(newLabel, labelText)

  gtkWidgetAddAccelerator(newItem, method, accelGroup, newKey, modType,
                          accelFlag)
  gtkContainerAdd(newItem, newLabel)
  gtkWidgetShow(newItem)

  if (method=="activate")
  {
    gtkAddCallback(newItem,"activate",
      function(obj)
      {
        updateCurAction(curAction, ...)
        return(TRUE)
      }
    )
  }  

  addToKeyVals(labelText, curAction, ...)

  return(newItem)
}

#############
# addMenuItem adds a menu item to the main menu after the main menu
# has already been created
# meant to be used if the user wants to add a menu item from the 
# command line
#
# menuName is the name the menu will take in the list menus, which
# is stored in controlEnv (so the menu can be accessed again)
#############
addMenuItem<-function(menuName, labelText, ag=get("ag", mvcEnv),
                      method="activate-item", modType=GdkModifierType[4],
                      accelFlag=GtkAccelFlags[2])
{
  newItem<-createMenuItem(labelText, ag, method, modType, accelFlag)
  menuBar<-get("menuBar", mvcEnv)
  gtkMenuBarAppend(menuBar, newItem)
  gtkWidgetShow(menuBar)

  # make a sub menu for this new menu item (so things can be added to 
  # it later)
  newMenu<-gtkMenuNew()
  gtkMenuItemSetSubmenu(newItem, newMenu)

  # need to also add the new menu item to menus list
  menus<-get("menus", mvcEnv)
  menus[[menuName]]<-newMenu
  assign("menus", menus, mvcEnv)
}


##############
# addSubMenuItem is to add a sub menu item to an already existing menu
# first add the menu item using addMenuItem and then add sub menu items
# using this function (add the menu item if you don't want to use file,
# edit, display or viewMode)
#
# menuName is the name of the menu that this sub menu item will be added to
# this must match the name of one of the list items in menus in controlEnv 
# action should be the function to call when this item is activated
# note that this function cannot have any parameters (see updateControlWindow)
##############
addSubMenuItem<-function(menuName, labelText, action, ag=get("ag", mvcEnv),
                       method="activate", modType=GdkModifierType[3], 
                       accelFlag=GtkAccelFlags[2], ...)
{
  # should check that the accelerator is not already in use!!
  
  newItem<-createMenuItem(labelText=labelText, accelGroup=ag, method=method, 
           modType=modType, accelFlag=accelFlag, curAction=action, ...)

  menus<-get("menus", mvcEnv)
  curMenu<-menus[[menuName]]
  if (length(curMenu)>0)
  {
    gtkMenuAppend(curMenu, newItem)
    return(newItem)
  }
  else
  { 
    nam<-names(menus)
    cat("Wrong menu name.  The name must be one of the following:\n ", 
        paste(nam,collapse=" "),"\n")
    return(list())
  }
}

###########
# update the current action environment variable and call update control 
# window to reflect the change in current action
###########
updateCurAction<-function(curAct, ...)
{
  assign("curAction", curAct, mvcEnv)
  updateControlWindow(...)
}

##############
# updateControlWindow updates the control window
# after a different menu item was selected
##############
updateControlWindow<-function(...)
{
  curAction<-get("curAction", mvcEnv)

  # decide how to update based on curAction
  switch(curAction,
    "loadM"=setLoadModelView(),
    "loadMV"=setLoadModelVarView(),
    "createCh"=createChildModelView(),
    "plotGr"=setPlotGrView(),
    "viewData"=setViewDataView(),
    "plotD"=setPlotDView(),
    "plotQQ"=setPlotQQView(),
    "plotHm"=setPlotHmView(),
    "quit"=askToQuit(), 
    "empty"=setEmptyView(),
    "addEventFun"=addEventFunction(),
    "setMouseOver"=setCallFunEvent("mouseOver"), 
    "setLeftBC"=setCallFunEvent("leftButtonClick"),
    "setMiddleBC"=setCallFunEvent("middleButtonClick"),
    "setRightBC"=setCallFunEvent("rightButtonClick"),
    "setActiveMVCGUI"=setActiveMVCGUI(),
    "deleteMVCGUI"=deleteMVCGUI(),
    "showHide"=showHiddenView(),
    do.call(curAction,list(...))
  )
}

###############
# askToQuit opens a widget that asks the user
# if they really want to quit when the user has
# chosen quit on the main menu
#
# this function should close all windows if the
# user chooses yes!!!
###############
askToQuit<-function()
{
  win<-gtkWindow(show=FALSE)
  win$SetUsize(250,60)
  vBox<-gtkVBoxNew()
  lab<-gtkLabel("Are you sure you want to quit?")
  hBox<-gtkHBoxNew()
  gtkBoxPackStart(vBox,lab,padding=5)
  gtkBoxPackStart(vBox,hBox)
  yesBut<-gtkButton("Yes")
  yesBut$SetUsize(40,15)
  noBut<-gtkButton("No")
  noBut$SetUsize(40,15)
  gtkBoxPackEnd(hBox,yesBut,expand=FALSE,fill=FALSE,padding=5)
  gtkBoxPackEnd(hBox,noBut,expand=FALSE,fill=FALSE,padding=5)
  win$Add(vBox)
  gtkAddCallback(yesBut,"clicked",
    function(obj)
    {
      # call a function to destroy all windows
      clearAllWindows()
      win$Destroy()
      return(TRUE)
    }
  )
  gtkAddCallback(noBut,"clicked",
    function(obj)
    {
      win$Destroy()
      return(TRUE)
    }
  )
  win$Show()
}

#############
# clearAllWindows destroys all windows stored in mvcEnv
# called when the user decides to quit
#############
clearAllWindows<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
    for (i in 1:length(MVCList))
    {
      curViewList<-viewList(MVCList[[i]])
      if (length(curViewList) > 0)
        for (j in 1:length(curViewList))
        {
          setDeleteEvents(curViewList[[j]])
        }
    }

  # need to remove the control window
  cWin<-get("controlWin", mvcEnv)
  if (length(cWin) > 0)
  {
    cWin$Destroy()
    # reset the control window to empty
    assign("controlWin", c(), mvcEnv)
  }

  # need to remove the model graph window if it is open
  modelWin<-get("modelWin", mvcEnv)
  if (length(modelWin) > 0)
  {
    modelWin$Destroy()
    # turn off device
    modelDevice<-get("modelDevice", mvcEnv)
    dev.off(modelDevice)
    # reset values
    assign("modelDevice", c(), mvcEnv)
    assign("modelWin", c(), mvcEnv)
    assign("modelGraph", c(), mvcEnv)
  }

  setGlobalEnvVariables()
}


###########
# resetWinSize resets the gtkWindow to its default size 
# (having trouble with this)
###########
resetWinSize<-function()
{
  w<-get("controlWin", mvcEnv)
  w$SetUsize(get("winDefaultHeight", mvcEnv), 
             get("winDefaultWidth", mvcEnv))
}

#############
# setEmptyView sets the view on the control window to
# only have the main menu
#############
setEmptyView<-function()
{ 
  # so we'll know what the last action was that changed the 
  # control window appearance
  assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)

  resetWinSize()
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)
  
  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)
    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  # put in a new frame
  frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)
}

###########
# setActiveMVCGUI creates the frame where the active MVC can be chosen from 
# all loaded MVCs
###########
setActiveMVCGUI<-function()
{
  w<-gtkWindow(show=FALSE)
  frMVC<-gtkFrame(label="Set Active MVC")
  w$Add(frMVC)
  allvbox<-gtkVBox()
  frMVC$Add(allvbox)

  modNames<-getModelNames()
  sc<-gtkScrolledWindow()
  sc$SetPolicy("automatic", "automatic")

  MVCvbox<-gtkVBox()
  if (length(modNames) > 0)
  {
    radButs<-list()
    radBut<-gtkRadioButtonNewWithLabelFromWidget(label=modNames[1])
    gtkBoxPackStart(MVCvbox, radBut)
    radButs[[1]]<-radBut
    if (length(modNames) > 1)
      for (i in 2:length(modNames))
      {
        newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label=modNames[i])
        gtkBoxPackStart(MVCvbox, newradBut)
        radButs[[i]]<-newradBut
      }     
  } 
  sc$AddWithViewport(MVCvbox)  
  gtkBoxPackStart(allvbox, sc)

  hbox<-gtkHBox()
  gtkBoxPackStart(allvbox, hbox)

  setBut<-gtkButton("Set")
  canBut<-gtkButton("Cancel")

  gtkBoxPackStart(hbox, setBut)
  gtkBoxPackStart(hbox, canBut)

  w$Show()  

  # need a callback for the button press event
  gtkAddCallback(canBut, "clicked",
    function(obj, ev)
    {
      # destroy the window without doing anything
      w$Destroy()
      return(TRUE)
    }
  )
  gtkAddCallback(setBut, "clicked",
    function(obj, ev)
    {
      # first find out which name is checked and then set the 
      # environment variable
      if (length(modNames) > 0)
      {
        for (i in 1:length(radButs))
        {
          if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
            curIndex<-i
        }      
        activeMVC<-modNames[curIndex]
        assign("oldActiveMVC", get("activeMVC", mvcEnv), mvcEnv)
        assign("activeMVC", activeMVC, mvcEnv)

        # need to set the display menu when the active MVC changes
        setDisplayMenu(activeMVC)

        # set the control window to empty because the old action may be
        # for a different model type
        if (length(get("controlWin", mvcEnv)) > 0)
          setEmptyView()

        # need to refresh the model graph if there is one
        if (length(get("modelWin", mvcEnv)) > 0)
        {
          Sys.sleep(0.5)
          plotModelGraph()
        }
      }

      # destroy the window
      w$Destroy()
      return(TRUE)
    }
  )
}

#########
# need to change the display menu based on the class of the model
# object in the active MVC object
#########
setDisplayMenu<-function(activeModelName)
{
  displayMenu<-gtkMenuNew()
  displayItem<-get("displayItem", mvcEnv)
  # remove old submenu before adding new
  gtkMenuItemRemoveSubmenu(displayItem)

  # problem: also need to remove old key vals from keyVals in mvcEnv if 
  # we are resetting the active mvc - store old active MVC in mvcEnv
  # otherwise, there could be multiple actions set to one key event
  oldaMVC<-get("oldActiveMVC", mvcEnv)
  # now need to remove old key vals
  if (oldaMVC != "")
  {
    oldMVC<-getMVC(oldaMVC)
    oldModel<-model(oldMVC)
    if (is(oldModel, "graphModel"))
    {
      removeKeyVals(accelChar="P", action="plotGr")
      # now graphs can have a spreadsheet view
      removeKeyVals(accelChar="D", action="viewData")
    }
    if (is(oldModel, "dfModel"))
    {
      removeKeyVals(accelChar="D", action="viewData")
      removeKeyVals(accelChar="P", action="plotD")
    }
    if (is(oldModel, "exprModel"))
    {
      removeKeyVals(accelChar="P", action="plotHm")
    }
    if (is(oldModel, "gseModel"))
    {
      removeKeyVals(accelChar="P", action="plotQQ")
      removeKeyVals(accelChar="D", action="viewData")
    }
  }

  menus<-get("menus", mvcEnv)
  menus[["display"]]<-displayMenu
  assign("menus", menus, mvcEnv)

  activeMVC<-getMVC(activeModelName)
  actModel<-model(activeMVC)

  if (is(actModel, "graphModel")) 
  {
    plotGItem<-addSubMenuItem(labelText="_Plot Graph   Ctrl+P", 
        action="plotGr", menuName="display")
    viewDataItem<-addSubMenuItem(labelText="View Node _Data   Ctrl+D", 
        action="viewData", menuName="display")
  }
  if (is(actModel, "dfModel"))
  {
    plotItem<-addSubMenuItem(labelText="_Plot Data     Ctrl+P", 
        action="plotD",	menuName="display")
    viewDataItem<-addSubMenuItem(labelText="View _Data      Ctrl+D",
        action="viewData", menuName="display")
  }
  if (is(actModel, "exprModel"))
  {
    # need an item to make a heatmap of the exprSet data
    heatMItem<-addSubMenuItem(labelText="_Plot Heatmap   Ctrl+P", 
        action="plotHm", menuName="display")
  }
  if (is(actModel, "gseModel"))
  {
    plotItem<-addSubMenuItem(labelText="Create QQ _Plot   Ctrl+P", 
        action="plotQQ", menuName="display")
    viewDataItem<-addSubMenuItem(labelText="View _Data      Ctrl+D",
        action="viewData", menuName="display")
  }

  gtkMenuItemSetSubmenu(displayItem, displayMenu)

  menus<-get("menus", mvcEnv)
  menus[["display"]]<-displayMenu
  assign("menus", menus, mvcEnv)
}

##########
# assign a callback function to an event
# will be called whenever the user wants to set a function to a mouse over or
# a button click event
# eventList is a character string of the event name that has a 
#  list in controlEnv
##########
setCallFunEvent<-function(eventList)
{
  # first make sure there are MVCs
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    w<-get("controlWin", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()
    else  
    {
      # make sure old action gets set when the first window is loaded
      assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)  
    }
 
    # 5 rows and 6 columns
    tab<-gtkTableNew(5, 6, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }

    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)

    frAvail<-gtkFrame(paste("Available Functions for", eventList))
    frCurrent<-gtkFrame(paste("Current Function set for", eventList))
    gtkTableAttach(tab, frAvail, 0, 3, 0, 4, xpadding=5, ypadding=5)
    gtkTableAttach(tab, frCurrent, 3, 6, 0, 4, xpadding=5, ypadding=5) 

    # now need to populate these frames
    availFunctions<-get(eventList, controlEnv)
    shortDesc<-unlist(lapply(availFunctions, function(y)
                      {shortName(y$eventFun)})) 

    # need to show these short descriptions in frAvail   
    # make a vector of radio buttons
    if (length(shortDesc) > 0)
    {
      # may possibly need this to be scrolled
      vbox<-gtkVBox()
      # make a vector of radio buttons   
      radButs<-list()
      radBut<-gtkRadioButtonNewWithLabelFromWidget(label=shortDesc[1])
      radButs[[1]]<-radBut
      gtkBoxPackStart(vbox, radBut)
      if (length(shortDesc) > 1)
        for (i in 2:length(shortDesc))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label=shortDesc[i])
          gtkBoxPackStart(vbox, newradBut)
          radButs[[i]]<-newradBut
        } 
      frAvail$Add(vbox)
 
      # then add one more radio button for no action (function)
      newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label="no action")
      gtkBoxPackStart(vbox, newradBut)
      radButs[[length(shortDesc)+1]]<-newradBut
    }
    # find out if any function is assigned
    activeFun<-unlist(lapply(availFunctions, function(y)
                            {y$assigned==TRUE}))        
    if (any(activeFun))
    {
      curActiveIndex<-which(activeFun)
      if (length(curActiveIndex) == 1)
      {
        activeLab<-gtkLabel(shortName(availFunctions[[curActiveIndex]][["eventFun"]]))
      }
    }
    else
      activeLab<-gtkLabel(paste("There is currently no action\n set to", 
                              eventList))
    frCurrent$Add(activeLab)

    setBut<-gtkButton("Set")
    gtkTableAttach(tab, setBut, 2, 4, 4, 5, xpadding=10, ypadding=5)

    # need a callback function for the set button
    gtkAddCallback(setBut, "clicked",
      function(obj, ev) 
      {
        # if there are any spreadsheets in activeMVC remove selection
        removeSpreadSelect()
        # find out which button is active
        if (length(shortDesc) > 0)      
        {
          for (i in 1:length(radButs))
          {
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }
          # then no action (function) was chosen
          if (curIndex > length(shortDesc))
          {
            # need to make sure that assigned is set to FALSE for all functions
            if (any(activeFun))
            {
              availFunctions[[curActiveIndex]][["assigned"]]<-FALSE
              assign(eventList, availFunctions, controlEnv)
              # now recall this function to reset the view
              setCallFunEvent(eventList)
            }
          }
          # then one of the functions was chosen
          else
          {
            # first set the old function assigned to FALSE
            if (any(activeFun))
              availFunctions[[curActiveIndex]][["assigned"]]<-FALSE
            # now set the new function assigned to TRUE
            availFunctions[[curIndex]][["assigned"]]<-TRUE
            assign(eventList, availFunctions, controlEnv)
 
            # need to call the preprocessing function
            preprocess<-preprocessFun(availFunctions[[curIndex]][["eventFun"]])
            if (length(preprocess) > 0)
              for (i in 1:length(preprocess))
              {
                do.call(preprocess[[i]], list())
              }

            # now recall this function to reset the view
            setCallFunEvent(eventList)
          }
        }
      return(TRUE)
      }
    )
  }
  else
    if (length(get("controlWin", mvcEnv)) > 0)
      setEmptyView()  
}

#############
# add a callback function to one of the event lists in controlEnv
#
# this option is currently not available through the menu on the control
# window, but the functions have been created (see Add Event Function in 
# createMainMenu)
#############
addEventFunction<-function()
{
  # first make sure there are MVCs
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    # reset the events the user wants to add the function to
    assign("selEvents", c(), controlEnv)

    w<-get("controlWin", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()
    else
      # make sure old action gets set when the first window is loaded
      assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
 
    # 10 rows and 9 columns
    tab<-gtkTableNew(10, 9, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }

    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)

    callName<-gtkLabel("Name of Callback Function")
    shortName<-gtkLabel("Short Description of Function")
    preName<-gtkLabel("Name(s) of Preprocessing Function(s)")
    whichName<-gtkLabel("Which Events to Link to")

    gtkTableAttach(tab, callName, 1, 5, 0, 1, xpadding=5, ypadding=5)
    gtkTableAttach(tab, shortName, 1, 5, 1, 2, xpadding=5, ypadding=5)
    gtkTableAttach(tab, preName, 1, 5, 2, 3, xpadding=5, ypadding=5)
    gtkTableAttach(tab, whichName, 1, 5, 5, 6, xpadding=5, ypadding=5)

    # text is for multiline text and entry is for single line text
    callEntry<-gtkEntry()
    shortEntry<-gtkEntry()
    preText<-gtkText()
    gtkTextSetEditable(preText, TRUE)

    gtkTableAttach(tab, callEntry, 5, 8, 0, 1, xpadding=5, ypadding=5)
    gtkTableAttach(tab, shortEntry, 5, 8, 1, 2, xpadding=5, ypadding=5)
    gtkTableAttach(tab, preText, 5, 8, 2, 5, xpadding=5, ypadding=5)   

    # get a list of all the possible events
    possEvents<-get("possEvents", mvcEnv)

    sc<-gtkScrolledWindow()
    sc$SetPolicy("automatic", "automatic")
    pEventsList<-gtkList()
    if (length(possEvents)>0)
      for (i in 1:length(possEvents))
      {
        pEventsListItem<-gtkListItemNewWithLabel(possEvents[i])
        pEventsList$Add(pEventsListItem)
      }
 
    # allow multiple selections
    gtkListSetSelectionMode(pEventsList, GtkSelectionMode[3])
    sc$AddWithViewport(pEventsList)  
    gtkTableAttach(tab, sc, 5, 8, 5, 8, xpadding=5, ypadding=5)

    # couldn't figure out another way to figure out which 
    # list items were selected - would prefer to determine which list
    # items are selected by looking at state or something like that
    chosenEvents<-gtkAddCallback(pEventsList, "select-child",
      function(obj, objchild)
      {
        # get the active element in the list
        selItem<-gtkChildren(objchild)[[1]][["label"]]
        selEvents<-get("selEvents", controlEnv)
        selEvents[length(selEvents)+1]<-selItem
        assign("selEvents", selEvents, controlEnv)
        return(TRUE)
      }
    )  

    gtkAddCallback(pEventsList, "unselect-child",
      function(obj, objchild)
      {
        selItem<-gtkChildren(objchild)[[1]][["label"]]
        selEvents<-get("selEvents", controlEnv)
        curIndex<-match(selItem, selEvents)
        if (!is.na(curIndex))
        {
          selEvents<-selEvents[-curIndex]
          assign("selEvents", selEvents, controlEnv)
        }
        return(TRUE)
      }
    )

    setBut<-gtkButton("Set Event Function")
    gtkTableAttach(tab, setBut, 3, 6, 8, 10, xpadding=5, ypadding=10)

    gtkAddCallback(setBut,"clicked",
      function(obj,ev) 
      {
        # need to get all the text entries
        curCallText<-gtkEntryGetText(callEntry)
        curShortText<-gtkEntryGetText(shortEntry)
        curPreText<-gtkTextGetText(preText)
      
        # need to split up the text into the different function names
        if (length(grep("\n", curPreText)) > 0)
        {
          curPreText<-unlist(strsplit(curPreText, "\n"))
        }

        # need to get the selected list values
        selEvents<-get("selEvents", controlEnv)

        # now need to check the values - may want more checking here
        # may want to check if this function has already been added
        validValues<-TRUE
        if (curCallText == "")
          validValues<-FALSE
        if (curShortText == "")
          validValues<-FALSE
        # preprocessing functions can be zero or more functions
        if (length(selEvents) == 0)
          validValues<-FALSE

        if (validValues)
        {
          addCallEvent(callFunction=curCallText, shortName=curShortText, 
             preprocessingFun=curPreText, eventLists=selEvents)
          # then something should happen to let the user know that an
          # addition was successful
        }
        return(TRUE)
      }
    )
  }
  else
    if (length(get("controlWin", mvcEnv)) > 0)
      setEmptyView() 
}

###########
# 7/12/05 create a child model from the active model
###########
createChildModelView<-function()
{
  # can only make a child model if there is an active model
  activeMVC<-get("activeMVC", mvcEnv)
  if (activeMVC != "")
  {
    w<-get("controlWin", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()
    else  
    {
      # make sure old action gets set when the first window is loaded
      assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
    }

    # 5 rows and 5 columns
    tab<-gtkTableNew(5, 5, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }

    frMVC<-gtkFrameNew(label=activeMVC)
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)
  
    # get the possible methods to create a child model for the active model
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC) 
    createChild<-get("createChild", controlEnv)

    if (length(createChild) > 0)
    {
      screateChild<-sort(createChild)
      lblType<-gtkLabel("Method To Create Child Model")
      gtkTableAttach(tab, lblType, 1, 4, 0, 1)
      combType<-gtkCombo()
      gtkComboSetPopdownStrings(combType, screateChild)
      
      gtkComboSetValueInList(combType, TRUE, FALSE)
      gtkTableAttach(tab, combType, 1, 4, 1, 2)  

      gtkAddCallback(gtkComboGetList(combType), "select-child",
        function(obj, objchild)
        {
          curValue<-gtkChildren(objchild)[[1]][["label"]]
          assign("childModel", curValue, controlEnv)
          return(TRUE)
        }
      )
      # get the list so the first item can be selected
      lsType<-gtkComboGetList(combType)
      gtkListSelectItem(lsType, 0)

      # now add the name entry
      lblName<-gtkLabel("Name of the New Child Model")
      gtkTableAttach(tab, lblName, 1, 4, 2, 3)
      nameEntry<-gtkEntry()
      gtkTableAttach(tab, nameEntry, 1, 4, 3, 4)

      modelBut<-gtkButton("Create Child Model")
      gtkTableAttach(tab, modelBut, 1, 4, 4, 5)

      gtkAddCallback(modelBut, "clicked",
        function(obj, ev)
        {
          # get the entry text
          curNameText<-gtkEntryGetText(nameEntry)      
          childModel<-get("childModel", controlEnv)

          if (childModel != "" & curNameText != "")
          {
            assign("childName", curNameText, controlEnv)
            # find this in the named vector
            index<-match(childModel, screateChild)
            curFun<-names(screateChild)[index]
            do.call(curFun, list())
          }
          return(TRUE)
        }
      )
    }
  }
}

###########
# 7/12/05 create a subset model from a data frame model
###########
createSubsetGUI<-function()
{
  w<-gtkWindow(show=FALSE)
  w$SetUsize(500, 200)
 
  # 6 rows and 7 columns
  tab<-gtkTableNew(6, 7, TRUE)
  vbox<-gtkVBoxNew()

  w$Add(vbox)
  gtkBoxPackStart(vbox, tab)

  # need to get the active MVC
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  # need the data
  curData<-modelData(model(curMVC))
  variables<-colnames(curData)
  svariables<-sort(variables)
  controlEnv<-controller(curMVC)
  curDf<-modelData(model(curMVC))
  virDf<-virtualData(model(curMVC)) 

  relationships<-c("greater than", "greater than or equal to", "less than",
                   "less than or equal to", "equal to", "between", 
                   "excluding values between")

  instrLab<-gtkLabel("Subset the model using the relationships below")
  gtkTableAttach(tab, instrLab, 0, 7, 0, 1)

  varLab<-gtkLabel("Variables")
  relLab<-gtkLabel("Relationships")
  valLab<-gtkLabel("Value(s)")
    
  gtkTableAttach(tab, varLab, 0, 2, 2, 3, xpadding=5)
  gtkTableAttach(tab, relLab, 2, 4, 2, 3, xpadding=5)
  gtkTableAttach(tab, valLab, 4, 5, 2, 3, xpadding=5)

  combVar<-gtkCombo()
  gtkComboSetPopdownStrings(combVar, svariables)

  gtkComboSetValueInList(combVar, TRUE, FALSE)
  gtkTableAttach(tab, combVar, 0, 2, 3, 4, xpadding=5)  

  gtkAddCallback(gtkComboGetList(combVar), "select-child",
    function(obj, objchild)
    {
      curValue<-gtkChildren(objchild)[[1]][["label"]]
      assign("curVariable", curValue, controlEnv)
      return(TRUE)
    }
  )

  # get the list so the first item can be selected
  lsVar<-gtkComboGetList(combVar)
  gtkListSelectItem(lsVar, 0)

  combRel<-gtkCombo()
  gtkComboSetPopdownStrings(combRel, relationships)

  gtkComboSetValueInList(combRel, TRUE, FALSE)
  gtkTableAttach(tab, combRel, 2, 4, 3, 4, xpadding=5)

  firstEntry<-gtkEntry()
  secondEntry<-gtkEntry()
  andLab<-gtkLabel("And")
  gtkTableAttach(tab, firstEntry, 4, 5, 3, 4)
  gtkTableAttach(tab, andLab, 5, 6, 3, 4)
  gtkTableAttach(tab, secondEntry, 6, 7, 3, 4)
  gtkWidgetHide(andLab)
  gtkWidgetHide(secondEntry)

  gtkAddCallback(gtkComboGetList(combRel), "select-child",
    function(obj, objchild)
    {
      curValue<-gtkChildren(objchild)[[1]][["label"]]
      assign("curRelationship", curValue, controlEnv)
      if (curValue == "between" || curValue == "excluding values between")
      {
        gtkWidgetShow(andLab)
        gtkWidgetShow(secondEntry)
      }
      else
      {
        gtkWidgetHide(andLab)
        gtkWidgetHide(secondEntry)
      }
      return(TRUE)
    }
  )

  # get the list so the first item can be selected
  lsRel<-gtkComboGetList(combRel)
  gtkListSelectItem(lsRel, 0)

  contBut<-gtkButton("Continue")
  doneBut<-gtkButton("Done")

  # hide the continue button for now
  gtkWidgetHide(contBut)

  gtkTableAttach(tab, contBut, 1, 3, 5, 6)
  gtkTableAttach(tab, doneBut, 4, 6, 5, 6)
 
  gtkAddCallback(contBut, "clicked",
    function(obj, ev)
    {
      # need 2 more windows to appear
      # one window allowing the user to choose AND or OR
      # retValue will be either AND or OR
#      retValue<-createChoiceWindow()     

      # second window showing what the user has chosen to create the subset
      # so this second window will basically be a label
#      return(TRUE)
    }
  )

  gtkAddCallback(doneBut, "clicked",
    function(obj, ev)
    {
      # need to create a message to create the child model
      # need to create gAddChildMessage
      # will need 3 things: dataName, mData, type
      # type = "data.frame"
      dataName<-get("childName", controlEnv)
      curRelationship<-get("curRelationship", controlEnv)
      curVariable<-get("curVariable", controlEnv)

      if (curRelationship == "greater than")
        subRowNames<-rownames(curDf)[curDf[, curVariable] > 
                       as.numeric(gtkEntryGetText(firstEntry))]
      if (curRelationship == "greater than or equal to")
        subRowNames<-rownames(curDf)[curDf[, curVariable] >=
                       as.numeric(gtkEntryGetText(firstEntry))]
      if (curRelationship == "less than")
        subRowNames<-rownames(curDf)[curDf[, curVariable] <
                       as.numeric(gtkEntryGetText(firstEntry))]
      if (curRelationship == "less than or equal to")
        subRowNames<-rownames(curDf)[curDf[, curVariable] <=
                       as.numeric(gtkEntryGetText(firstEntry))]
      if (curRelationship == "equal to")
        subRowNames<-rownames(curDf)[curDf[, curVariable] ==
                       as.numeric(gtkEntryGetText(firstEntry))]
      if (curRelationship == "between")
        subRowNames<-rownames(curDf)[(curDf[, curVariable] >=
                       as.numeric(gtkEntryGetText(firstEntry))) &&
                       (curDf[, curVariable] <= 
                       as.numeric(gtkEntryGetText(secondEntry)))]
      if (curRelationship == "excluding values between")
        subRowNames<-rownames(curDf)[(curDf[, curVariable] <
                       as.numeric(gtkEntryGetText(firstEntry))) &&
                       (curDf[, curVariable] > 
                       as.numeric(gtkEntryGetText(secondEntry)))]

      subData<-curDf[match(subRowNames, rownames(curDf)),]     
      subVirData<-virDf[match(subRowNames, rownames(virDf)),]
    
      newAddChildMessage<-new("gAddChildMessage", dataName=dataName,
                             type="data.frame", data=list(data=subData, 
                             virtualData=subVirData))

      handleMessage(newAddChildMessage, toParent=linkToParentByRowName, 
                    fromParent=linkToChildByRowName, 
                    subsetParentData=subRowNames)     

      assign("childName", "", controlEnv)
      # then destroy the window
      w$Destroy()
      return(TRUE)
    }
  )

  w$Show()  
}

###########
# 9/10/05 test if the right packages are available to create the 
# GO graph (also check that the parent data are affy ids)
###########
checkIfOkToCreateGoGraph<-function()
{
  # either the active MVC or a parent of the active MVC must be an exprModel
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  curModel<-model(curMVC)  
  booExpr<-FALSE
  controlEnv<-controller(curMVC) 

  # not sure if I should require that some parent of the model (or the model
  # itself) is of type exprModel - may not be necessary
  # really just need exprModel to determine the chip type to convert affy ids
  # to locus link ids
  if (!is(curModel, "exprModel"))  
  {
    # need to loop through parent models to see if any are exprModels
    curParent<-parentMVC(curMVC)
    while (curParent != "")
    {
      newMVC<-getMVC(curParent)
      newModel<-model(newMVC)
      if (is(newModel, "exprModel"))
      {
        booExpr<-TRUE
        parExprModel<-modelName(newModel)
        # get the annotation slot
        annLib<-annotation(modelData(newModel))
      }
      curParent<-parentMVC(newMVC)
    }
  }
  else
  {
    booExpr<-TRUE
    # get the annotation slot
    annLib<-annotation(modelData(curModel))
  }

  booCont<-FALSE
  if (booExpr)
  {
    if (annLib =="" || is.null(annLib))
      getChipType(annLib)
    else
      assign("chipType", annLib, controlEnv)

    # now get the return value from getChipType (which creates a modal
    # GTK window)
    curChipType<-get("chipType", controlEnv)
    require(curChipType, character.only=TRUE) || stop(paste("The", 
             curChipType, "package is required to continue")) 
    
    booCont<-TRUE
  }
  else
  {
    # show a label saying that there is no parent exprModel so it is not
    # possible to create a child GO Graph
    showLabel("The active MVC does not have an exprSet model and none of the
  parents of the active MVC do either so it is not possible to create a child
  GO Graph.")
  }
  return(booCont)
}

##########
# 7/10/06 perform gene set enrichment using chromosome location (from MAP) as
# categories; the t-test is used to calculate the gene test statistics and the
# gene set statistics are standardized by dividing by the square root of the
# incidence matrix row sums
##########
createChrLocGSEGUI<-function()
{
  # assume that the (parent) model creating this new gseModel is of type
  # exprModel
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  curModel<-model(curMVC)  
  controlEnv<-controller(curMVC) 
  curModVar<-modelVar(curModel)

  annot<-annotation(modelData(curModel))
  if (annot== "")
    annot<-get("chipType", controlEnv)

  if (annot != "")
  {
    # make sure the user has the correct libraries installed
    require("genefilter") || 
        stop("The genefilter package is required to continue")
    require("Category") || stop("The Category package is required to continue")

    # check for a factor variable
    curpData<-pData(modelData(curModel))
    # couldn't get apply to work!
    booFac<-rep(FALSE, ncol(curpData))
    for (i in 1:ncol(curpData))
    {
      # want a factor with only 2 levels for the t-test
      if (is(curpData[,i], "factor") & length(unique(curpData[,i]))==2)
        booFac[i]<-TRUE
    }
    # must have a factor with 2 levels for the t-test
    if (any(booFac))
    {
      possFac<-colnames(curpData)[booFac]

      w<-gtkWindow(show=FALSE)
      w$SetUsize(500, 350)

      # table with 8 rows and 6 columns
      tab<-gtkTableNew(8, 6, TRUE)
      vbox<-gtkVBoxNew()
 
      w$Add(vbox)
      gtkBoxPackStart(vbox, tab)

      # now add to the table to get the affy ids and the ontology type
      affyids<-c()
      if (is(curModel, "exprModel"))
        affyids<-geneNames(modelData(curModel))

      frAvail<-gtkFrame("Available Affy Ids")
      boxForAvail<-gtkVBox(TRUE, 2)
      sc<-gtkScrolledWindow()
      sc$SetPolicy("automatic", "automatic")
      # one column in the new list
      availList<-gtkCListNew(columns=1)
      gtkCListFreeze(availList)
      if (length(affyids)>0)
        for (i in 1:length(affyids))
        {
          gtkCListAppend(availList, affyids[i])
        }
      gtkCListThaw(availList)

      gtkCListSetAutoSort(availList, TRUE)
      # supposedly this will allow the user to select multiple rows
      # (though I've had problems with it)
      gtkCListSetSelectionMode(availList, GtkSelectionMode[3])
      sc$AddWithViewport(availList)
      boxForAvail$PackStart(sc, expand=TRUE)
      frAvail$Add(boxForAvail)
      gtkTableAttach(tab, frAvail, 0, 2, 0, 4, xpadding=5, ypadding=5)

      selBut<-gtkButton("Select")
      selAllBut<-gtkButton("Select All")
      unsBut<-gtkButton("Unselect")
      unsAllBut<-gtkButton("Unselect All")

      gtkAddCallback(selAllBut, "clicked",
        function(obj, ev)
        {
          numRows<-gtkCListGetRows(availList)
          if (numRows > 0)
          {
            gtkCListFreeze(availList)
            gtkCListFreeze(selList)
            for (i in 1:numRows)
            {
              selValue<-gtkCListGetText(availList, row=0, cols=0)
              # first remove the item from availList
              gtkCListRemove(availList, row=0)

              # then add to selList
              gtkCListAppend(selList, selValue)
            }
            gtkCListThaw(availList)
            gtkCListThaw(selList)
          }
          return(TRUE)
        }
      )
      gtkAddCallback(selBut, "clicked",
        function(obj, ev)
        {
          # should I do any checking here (to make sure the list has items?)
          numRows<-gtkCListGetRows(availList)
      
          if (numRows > 0)
          {
            # need to determine which elements are highlit in availList
            rowInt<-gtkCListGetFocusRow(availList)
            chosenAffy<-gtkCListGetText(availList, row=rowInt, cols=0)  
        
            # next remove this item from availList
            gtkCListRemove(availList, row=rowInt)
 
            # next add the text to selList
            gtkCListAppend(selList, chosenAffy)
          }
          return(TRUE)
        }
      )
      gtkAddCallback(unsBut, "clicked",
        function(obj, ev)
        {
          numRows<-gtkCListGetRows(selList)

          if (numRows > 0)
          {
            # need to determine which elements are highlit in selList
            # then remove them from selList and add to availList
            rowInt<-gtkCListGetFocusRow(selList)
            chosenAffy<-gtkCListGetText(selList, row=rowInt, cols=0)
      
            # next remove the item from selList
            gtkCListRemove(selList, row=rowInt)
   
            # next add the text to availList
            gtkCListAppend(availList, chosenAffy)
          }
          return(TRUE)
        }
      )
      gtkAddCallback(unsAllBut, "clicked",
        function(obj, ev)
        {
          # need to remove everything in selList
          numRows<-gtkCListGetRows(selList)
          if (numRows > 0)
          {
            gtkCListFreeze(availList)
            gtkCListFreeze(selList)
            for (i in 1:numRows)
            {
              selValue<-gtkCListGetText(selList, row=0, cols=0)
              # first remove the item from selList
              gtkCListRemove(selList, row=0)

              # then add to availList
              gtkCListAppend(availList, selValue)
            }
            gtkCListThaw(availList)
            gtkCListThaw(selList)
          }
          return(TRUE)
        }
      )

      gtkTableAttach(tab, selAllBut, 2, 4, 0, 1, xpadding=5, ypadding=8)
      gtkTableAttach(tab, selBut, 2, 4, 1, 2, xpadding=5, ypadding=8)
      gtkTableAttach(tab, unsBut, 2, 4, 2, 3, xpadding=5, ypadding=8)
      gtkTableAttach(tab, unsAllBut, 2, 4, 3, 4, xpadding=5, ypadding=8)

      frSel<-gtkFrame("Selected Affy Ids")
      boxForSel<-gtkVBox(TRUE, 2)
      scSel<-gtkScrolledWindow()
      scSel$SetPolicy("automatic", "automatic")
      # one column
      selList<-gtkCListNew(columns=1)
      # will be an empty list on load

      gtkCListSetAutoSort(selList, TRUE)
      # supposedly this will allow the user to select multiple items
      gtkCListSetSelectionMode(selList, GtkSelectionMode[3])
      scSel$AddWithViewport(selList)
      boxForSel$PackStart(scSel, expand=TRUE)
      frSel$Add(boxForSel)
      gtkTableAttach(tab, frSel, 4, 6, 0, 4, xpadding=5, ypadding=5)

      if (length(curModVar) > 0)
      {
        modLab<-gtkLabel("Subset by Model Variable")
        gtkTableAttach(tab, modLab, 1, 3, 4, 5, xpadding=5, ypadding=5)

        combMV<-gtkCombo()
        gtkComboSetPopdownStrings(combMV, c("", names(curModVar)))
        gtkComboSetValueInList(combMV, TRUE, FALSE)
        gtkTableAttach(tab, combMV, 3, 5, 4, 5, xpadding=5, ypadding=5)
 
        gtkAddCallback(gtkComboGetList(combMV), "select-child",
          function(obj, objchild)
          {
            # need to update the lists
            curValue<-gtkChildren(objchild)[[1]][["label"]]

            # need to store this somewhere and update if it's a new value!
            if ("MVName" %in% ls(controlEnv))
            {
              oldMVValue<-get("MVName", controlEnv)
              if (oldMVValue == curValue)
                booCont<-FALSE
              else
                booCont<-TRUE
            }
            else
              booCont<-TRUE

            assign("MVName", curValue, controlEnv)

            if (booCont)
            {
              if (curValue != "")
              {
                curMVVal<-curModVar[[curValue]]
                if (is(curMVVal, "logical"))
                {
                  newaffyids<-affyids[curMVVal]
  
                  # now need to put these in the avail list

                  # first remove old values
                  numRows<-gtkCListGetRows(availList)
                  if (numRows > 0)
                  {
                    gtkCListFreeze(availList)
                    for (i in 1:numRows)
                    {
                      # first remove the item from availList
                      gtkCListRemove(availList, row=0)
                    }
                    gtkCListThaw(availList)
                  }

                  # now add new values
                  if (length(newaffyids)>0)
                    for (i in 1:length(newaffyids))
                    {
                      gtkCListAppend(availList, newaffyids[i])
                    }

                  gtkCListSetAutoSort(availList, TRUE)             
                }
              }
              else
              {
                newaffyids<-affyids

                # first remove old values
                numRows<-gtkCListGetRows(availList)
                if (numRows > 0)
                {
                  gtkCListFreeze(availList)
                  for (i in 1:numRows)
                  {
                    # first remove the item from availList
                    gtkCListRemove(availList, row=0)
                  }
                  gtkCListThaw(availList)
                }

                # now add new values
                if (length(newaffyids)>0)
                  for (i in 1:length(newaffyids))
                  {
                    gtkCListAppend(availList, newaffyids[i])
                  }

                gtkCListSetAutoSort(availList, TRUE)   
              }
            }
            return(TRUE)
          }          
        )
      }

      facLab<-gtkLabel("Factor for t-test")
      gtkTableAttach(tab, facLab, 1, 3, 5, 6, xpadding=5, ypadding=5)

      # need a drop down box for factor
      combFac<-gtkCombo()
      gtkComboSetPopdownStrings(combFac, possFac)
      gtkComboSetValueInList(combFac, TRUE, FALSE)
      gtkTableAttach(tab, combFac, 3, 5, 5, 6, xpadding=5, ypadding=5)  

      # get the list so the first item can be selected
      lsFac<-gtkComboGetList(combFac)
      gtkListSelectItem(lsFac, 0)   

      noGenesLab<-gtkLabel("Number of Genes in Gene Set")
      gtkTableAttach(tab, noGenesLab, 0, 3, 6, 7, xpadding=5, ypadding=5)

      # then need a text box
      noGEnt<-gtkEntryNew()
      gtkEntrySetText(noGEnt, "5")
      gtkTableAttach(tab, noGEnt, 3, 5, 6, 7, xpadding=5, ypadding=5)

      doneBut<-gtkButton("Done")
      gtkTableAttach(tab, doneBut, 2, 4, 7, 8, xpadding=5, ypadding=5)

      # here the chromosome location GSE child model is created
      gtkAddCallback(doneBut, "clicked",
        function(obj, ev)
        {
          # need to add the function that will create the child model
          # first get the affy ids
          numRows<-gtkCListGetRows(selList)
          if (numRows > 0)
          {
            allaffys<-rep("", numRows)
            for (i in 1:numRows)
              allaffys[i]<-gtkCListGetText(selList, row=(i-1), cols=0)

            # need to get the factor, the new child model name, and noGenes
            childMName<-get("childName", controlEnv)
            noGenes<-gtkEntryGetText(noGEnt)
            noGenes<-as.numeric(noGenes)
            curFac<-gtkEntryGetText(gtkContainerGetChildren(combFac)[[1]])

            # now create the child model
            createChrLocGSE(childMName, curFac, annot=annot, affyids=allaffys,
                            noGenes=noGenes)

            w$Destroy()
            return(TRUE)
          }
        }
      )
      # need to show the window!!!
      w$Show()
    } 
    else
      showLabel("There were no phenotype factors to perform the t-tests")
  }
}

###########
# 6/30/06 perform gene set enrichment using KEGG pathways as categories; 
# the t-test is used to calculate the gene test statistics and the gene set
#  statistics are standardized by diving by the square root of the incidence
#  matrix row sums
###########
createKEGGGSEGUI<-function()
{
  # assume that the (parent) model creating this new gseModel is of type
  # exprModel
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  curModel<-model(curMVC)  
  controlEnv<-controller(curMVC) 
  curModVar<-modelVar(curModel)

  annot<-annotation(modelData(curModel))
  if (annot== "")
    annot<-get("chipType", controlEnv)

  if (annot != "")
  {
    # make sure the user has the correct libraries installed
    require(genefilter) || 
        stop("The genefilter package is required to continue")
    require(Category) || stop("The Category package is required to continue")

    # check for a factor variable
    curpData<-pData(modelData(curModel))
    # couldn't get apply to work!
    booFac<-rep(FALSE, ncol(curpData))
    for (i in 1:ncol(curpData))
    {
      # want a factor with only 2 levels for the t-test
      if (is(curpData[,i], "factor") & length(unique(curpData[,i]))==2)
        booFac[i]<-TRUE
    }
    # must have a factor with 2 levels for the t-test
    if (any(booFac))
    {
      possFac<-colnames(curpData)[booFac]

      w<-gtkWindow(show=FALSE)
      w$SetUsize(500, 350)

      # table with 8 rows and 6 columns
      tab<-gtkTableNew(8, 6, TRUE)
      vbox<-gtkVBoxNew()
 
      w$Add(vbox)
      gtkBoxPackStart(vbox, tab)

      # now add to the table to get the affy ids and the ontology type
      affyids<-c()
      if (is(curModel, "exprModel"))
        affyids<-geneNames(modelData(curModel))

      frAvail<-gtkFrame("Available Affy Ids")
      boxForAvail<-gtkVBox(TRUE, 2)
      sc<-gtkScrolledWindow()
      sc$SetPolicy("automatic", "automatic")
      # one column in the new list
      availList<-gtkCListNew(columns=1)
      gtkCListFreeze(availList)
      if (length(affyids)>0)
        for (i in 1:length(affyids))
        {
          gtkCListAppend(availList, affyids[i])
        }
      gtkCListThaw(availList)

      gtkCListSetAutoSort(availList, TRUE)
      # supposedly this will allow the user to select multiple rows
      # (though I've had problems with it)
      gtkCListSetSelectionMode(availList, GtkSelectionMode[3])
      sc$AddWithViewport(availList)
      boxForAvail$PackStart(sc, expand=TRUE)
      frAvail$Add(boxForAvail)
      gtkTableAttach(tab, frAvail, 0, 2, 0, 4, xpadding=5, ypadding=5)

      selBut<-gtkButton("Select")
      selAllBut<-gtkButton("Select All")
      unsBut<-gtkButton("Unselect")
      unsAllBut<-gtkButton("Unselect All")

      gtkAddCallback(selAllBut, "clicked",
        function(obj, ev)
        {
          numRows<-gtkCListGetRows(availList)
          if (numRows > 0)
          {
            gtkCListFreeze(availList)
            gtkCListFreeze(selList)
            for (i in 1:numRows)
            {
              selValue<-gtkCListGetText(availList, row=0, cols=0)
              # first remove the item from availList
              gtkCListRemove(availList, row=0)

              # then add to selList
              gtkCListAppend(selList, selValue)
            }
            gtkCListThaw(availList)
            gtkCListThaw(selList)
          }
          return(TRUE)
        }
      )
      gtkAddCallback(selBut, "clicked",
        function(obj, ev)
        {
          # should I do any checking here (to make sure the list has items?)
          numRows<-gtkCListGetRows(availList)
      
          if (numRows > 0)
          {
            # need to determine which elements are highlit in availList
            rowInt<-gtkCListGetFocusRow(availList)
            chosenAffy<-gtkCListGetText(availList, row=rowInt, cols=0)  
        
            # next remove this item from availList
            gtkCListRemove(availList, row=rowInt)
 
            # next add the text to selList
            gtkCListAppend(selList, chosenAffy)
          }
          return(TRUE)
        }
      )
      gtkAddCallback(unsBut, "clicked",
        function(obj, ev)
        {
          numRows<-gtkCListGetRows(selList)

          if (numRows > 0)
          {
            # need to determine which elements are highlit in selList
            # then remove them from selList and add to availList
            rowInt<-gtkCListGetFocusRow(selList)
            chosenAffy<-gtkCListGetText(selList, row=rowInt, cols=0)
      
            # next remove the item from selList
            gtkCListRemove(selList, row=rowInt)
   
            # next add the text to availList
            gtkCListAppend(availList, chosenAffy)
          }
          return(TRUE)
        }
      )
      gtkAddCallback(unsAllBut, "clicked",
        function(obj, ev)
        {
          # need to remove everything in selList
          numRows<-gtkCListGetRows(selList)
          if (numRows > 0)
          {
            gtkCListFreeze(availList)
            gtkCListFreeze(selList)
            for (i in 1:numRows)
            {
              selValue<-gtkCListGetText(selList, row=0, cols=0)
              # first remove the item from selList
              gtkCListRemove(selList, row=0)

              # then add to availList
              gtkCListAppend(availList, selValue)
            }
            gtkCListThaw(availList)
            gtkCListThaw(selList)
          }
          return(TRUE)
        }
      )

      gtkTableAttach(tab, selAllBut, 2, 4, 0, 1, xpadding=5, ypadding=8)
      gtkTableAttach(tab, selBut, 2, 4, 1, 2, xpadding=5, ypadding=8)
      gtkTableAttach(tab, unsBut, 2, 4, 2, 3, xpadding=5, ypadding=8)
      gtkTableAttach(tab, unsAllBut, 2, 4, 3, 4, xpadding=5, ypadding=8)

      frSel<-gtkFrame("Selected Affy Ids")
      boxForSel<-gtkVBox(TRUE, 2)
      scSel<-gtkScrolledWindow()
      scSel$SetPolicy("automatic", "automatic")
      # one column
      selList<-gtkCListNew(columns=1)
      # will be an empty list on load

      gtkCListSetAutoSort(selList, TRUE)
      # supposedly this will allow the user to select multiple items
      gtkCListSetSelectionMode(selList, GtkSelectionMode[3])
      scSel$AddWithViewport(selList)
      boxForSel$PackStart(scSel, expand=TRUE)
      frSel$Add(boxForSel)
      gtkTableAttach(tab, frSel, 4, 6, 0, 4, xpadding=5, ypadding=5)

      if (length(curModVar) > 0)
      {
        modLab<-gtkLabel("Subset by Model Variable")
        gtkTableAttach(tab, modLab, 1, 3, 4, 5, xpadding=5, ypadding=5)

        combMV<-gtkCombo()
        gtkComboSetPopdownStrings(combMV, c("", names(curModVar)))
        gtkComboSetValueInList(combMV, TRUE, FALSE)
        gtkTableAttach(tab, combMV, 3, 5, 4, 5, xpadding=5, ypadding=5)
 
        gtkAddCallback(gtkComboGetList(combMV), "select-child",
          function(obj, objchild)
          {
            # need to update the lists
            curValue<-gtkChildren(objchild)[[1]][["label"]]

            # need to store this somewhere and update if it's a new value!
            if ("MVName" %in% ls(controlEnv))
            {
              oldMVValue<-get("MVName", controlEnv)
              if (oldMVValue == curValue)
                booCont<-FALSE
              else
                booCont<-TRUE
            }
            else
              booCont<-TRUE

            assign("MVName", curValue, controlEnv)

            if (booCont)
            {
              if (curValue != "")
              {
                curMVVal<-curModVar[[curValue]]
                if (is(curMVVal, "logical"))
                {
                  newaffyids<-affyids[curMVVal]
  
                  # now need to put these in the avail list

                  # first remove old values
                  numRows<-gtkCListGetRows(availList)
                  if (numRows > 0)
                  {
                    gtkCListFreeze(availList)
                    for (i in 1:numRows)
                    {
                      # first remove the item from availList
                      gtkCListRemove(availList, row=0)
                    }
                    gtkCListThaw(availList)
                  }

                  # now add new values
                  if (length(newaffyids)>0)
                    for (i in 1:length(newaffyids))
                    {
                      gtkCListAppend(availList, newaffyids[i])
                    }

                  gtkCListSetAutoSort(availList, TRUE)             
                }
              }
              else
              {
                newaffyids<-affyids

                # first remove old values
                numRows<-gtkCListGetRows(availList)
                if (numRows > 0)
                {
                  gtkCListFreeze(availList)
                  for (i in 1:numRows)
                  {
                    # first remove the item from availList
                    gtkCListRemove(availList, row=0)
                  }
                  gtkCListThaw(availList)
                }

                # now add new values
                if (length(newaffyids)>0)
                  for (i in 1:length(newaffyids))
                  {
                    gtkCListAppend(availList, newaffyids[i])
                  }

                gtkCListSetAutoSort(availList, TRUE)   
              }
            }
            return(TRUE)
          }          
        )
      }

      facLab<-gtkLabel("Factor for t-test")
      gtkTableAttach(tab, facLab, 1, 3, 5, 6, xpadding=5, ypadding=5)

      # need a drop down box for factor
      combFac<-gtkCombo()
      gtkComboSetPopdownStrings(combFac, possFac)
      gtkComboSetValueInList(combFac, TRUE, FALSE)
      gtkTableAttach(tab, combFac, 3, 5, 5, 6, xpadding=5, ypadding=5)  

      # get the list so the first item can be selected
      lsFac<-gtkComboGetList(combFac)
      gtkListSelectItem(lsFac, 0)   

      noGenesLab<-gtkLabel("Number of Genes in Gene Set")
      gtkTableAttach(tab, noGenesLab, 0, 3, 6, 7, xpadding=5, ypadding=5)

      # then need a text box
      noGEnt<-gtkEntryNew()
      gtkEntrySetText(noGEnt, "5")
      gtkTableAttach(tab, noGEnt, 3, 5, 6, 7, xpadding=5, ypadding=5)

      doneBut<-gtkButton("Done")
      gtkTableAttach(tab, doneBut, 2, 4, 7, 8, xpadding=5, ypadding=5)

      # here the KEGG GSE child model is created
      gtkAddCallback(doneBut, "clicked",
        function(obj, ev)
        {
          # need to add the function that will create the child model
          # first get the affy ids
          numRows<-gtkCListGetRows(selList)
          if (numRows > 0)
          {
            allaffys<-rep("", numRows)
            for (i in 1:numRows)
              allaffys[i]<-gtkCListGetText(selList, row=(i-1), cols=0)

            # need to get the factor, the new child model name, and noGenes
            childMName<-get("childName", controlEnv)
            noGenes<-gtkEntryGetText(noGEnt)
            noGenes<-as.numeric(noGenes)
            curFac<-gtkEntryGetText(gtkContainerGetChildren(combFac)[[1]])

            # now create the child model
            createKEGGGSE(childMName, curFac, annot=annot, affyids=allaffys, 
                          noGenes=noGenes)

            w$Destroy()
            return(TRUE)
          }
        }
      )
      # need to show the window!!!
      w$Show()
    } 
    else
      showLabel("There were no phenotype factors to perform the t-tests")
  }
}

##########
# 9/5/05 create GO Graph from a set of locus link ids
# to create this model some parent (grandparent, etc.) must be an
# exprModel so that we have annotation somewhere
##########
createGoGraphGUI<-function()
{
  # first check that creating a GO graph makes sense
  booCont<-checkIfOkToCreateGoGraph()

  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  curModel<-model(curMVC)  
  controlEnv<-controller(curMVC) 
  curModVar<-modelVar(curModel)

  curChipType<-get("chipType", controlEnv)

  if (booCont)
  {
    # make sure the user has the correct libraries installed
    require("GOstats") || stop("The GOstats package is required to continue")

    # will need to determine which type of ontology graph to make
    # the choices are MF, BP, CC
    ontChoices<-c("MF", "BP", "CC")

    w<-gtkWindow(show=FALSE)
    w$SetUsize(500, 350)

    # table with 7 rows and 6 columns
    tab<-gtkTableNew(7, 6, TRUE)
    vbox<-gtkVBoxNew()
 
    w$Add(vbox)
    gtkBoxPackStart(vbox, tab)

    # now add to the table to get the affy ids and the ontology type
    affyids<-c()
    if (is(curModel, "exprModel"))
      affyids<-geneNames(modelData(curModel))
    if (is(curModel, "dfModel"))
      affyids<-rownames(modelData(curModel))
    if (is(curModel, "graphModel"))
      affyids<-nodes(modelData(curModel))

    frAvail<-gtkFrame("Available Affy Ids")
    boxForAvail<-gtkVBox(TRUE, 2)
    sc<-gtkScrolledWindow()
    sc$SetPolicy("automatic", "automatic")
    # one column in the new list
    availList<-gtkCListNew(columns=1)
    gtkCListFreeze(availList)
    if (length(affyids)>0)
      for (i in 1:length(affyids))
      {
        gtkCListAppend(availList, affyids[i])
      }
    gtkCListThaw(availList)

    gtkCListSetAutoSort(availList, TRUE)
    # supposedly this will allow the user to select multiple rows
    # (though I've had problems with it)
    gtkCListSetSelectionMode(availList, GtkSelectionMode[3])
    sc$AddWithViewport(availList)
    boxForAvail$PackStart(sc, expand=TRUE)
    frAvail$Add(boxForAvail)
    gtkTableAttach(tab, frAvail, 0, 2, 0, 4, xpadding=5, ypadding=5)

    selBut<-gtkButton("Select")
    selAllBut<-gtkButton("Select All")
    unsBut<-gtkButton("Unselect")
    unsAllBut<-gtkButton("Unselect All")
    
    gtkAddCallback(selAllBut, "clicked",
      function(obj, ev)
      {
        numRows<-gtkCListGetRows(availList)
        if (numRows > 0)
        {
          gtkCListFreeze(availList)
          gtkCListFreeze(selList)
          for (i in 1:numRows)
          {
            selValue<-gtkCListGetText(availList, row=0, cols=0)
            # first remove the item from availList
            gtkCListRemove(availList, row=0)

            # then add to selList
            gtkCListAppend(selList, selValue)
          }
          gtkCListThaw(availList)
          gtkCListThaw(selList)
        }
        return(TRUE)
      }
    )
    gtkAddCallback(selBut, "clicked",
      function(obj, ev)
      {
        # should I do any checking here (to make sure the list has items?)
        numRows<-gtkCListGetRows(availList)
      
        if (numRows > 0)
        {
          # need to determine which elements are highlit in availList
          rowInt<-gtkCListGetFocusRow(availList)
          chosenAffy<-gtkCListGetText(availList, row=rowInt, cols=0)  
        
          # next remove this item from availList
          gtkCListRemove(availList, row=rowInt)
 
          # next add the text to selList
          gtkCListAppend(selList, chosenAffy)
        }
        return(TRUE)
      }
    )
    gtkAddCallback(unsBut, "clicked",
      function(obj, ev)
      {
        numRows<-gtkCListGetRows(selList)

        if (numRows > 0)
        {
          # need to determine which elements are highlit in selList
          # then remove them from selList and add to availList
          rowInt<-gtkCListGetFocusRow(selList)
          chosenAffy<-gtkCListGetText(selList, row=rowInt, cols=0)
      
          # next remove the item from selList
          gtkCListRemove(selList, row=rowInt)
   
          # next add the text to availList
          gtkCListAppend(availList, chosenAffy)
        }
        return(TRUE)
      }
    )
    gtkAddCallback(unsAllBut, "clicked",
      function(obj, ev)
      {
        # need to remove everything in selList
        numRows<-gtkCListGetRows(selList)
        if (numRows > 0)
        {
          gtkCListFreeze(availList)
          gtkCListFreeze(selList)
          for (i in 1:numRows)
          {
            selValue<-gtkCListGetText(selList, row=0, cols=0)
            # first remove the item from selList
            gtkCListRemove(selList, row=0)

            # then add to availList
            gtkCListAppend(availList, selValue)
          }
          gtkCListThaw(availList)
          gtkCListThaw(selList)
        }
        return(TRUE)
      }
    )

    gtkTableAttach(tab, selAllBut, 2, 4, 0, 1, xpadding=5, ypadding=8)
    gtkTableAttach(tab, selBut, 2, 4, 1, 2, xpadding=5, ypadding=8)
    gtkTableAttach(tab, unsBut, 2, 4, 2, 3, xpadding=5, ypadding=8)
    gtkTableAttach(tab, unsAllBut, 2, 4, 3, 4, xpadding=5, ypadding=8)

    frSel<-gtkFrame("Selected Affy Ids")
    boxForSel<-gtkVBox(TRUE, 2)
    scSel<-gtkScrolledWindow()
    scSel$SetPolicy("automatic", "automatic")
    # one column
    selList<-gtkCListNew(columns=1)
    # will be an empty list on load

    gtkCListSetAutoSort(selList, TRUE)
    # supposedly this will allow the user to select multiple items
    gtkCListSetSelectionMode(selList, GtkSelectionMode[3])
    scSel$AddWithViewport(selList)
    boxForSel$PackStart(scSel, expand=TRUE)
    frSel$Add(boxForSel)
    gtkTableAttach(tab, frSel, 4, 6, 0, 4, xpadding=5, ypadding=5)

    if (length(curModVar) > 0)
    {
      modLab<-gtkLabel("Subset by Model Variable")
      gtkTableAttach(tab, modLab, 1, 3, 4, 5, xpadding=5, ypadding=5)

      combMV<-gtkCombo()
      gtkComboSetPopdownStrings(combMV, c("", names(curModVar)))
      gtkComboSetValueInList(combMV, TRUE, FALSE)
      gtkTableAttach(tab, combMV, 3, 5, 4, 5, xpadding=5, ypadding=5)
 
      gtkAddCallback(gtkComboGetList(combMV), "select-child",
        function(obj, objchild)
        {
          # need to update the lists
          curValue<-gtkChildren(objchild)[[1]][["label"]]

          # need to store this somewhere and update if it's a new value!
          if ("MVName" %in% ls(controlEnv))
          {
            oldMVValue<-get("MVName", controlEnv)
            if (oldMVValue == curValue)
              booCont<-FALSE
            else
              booCont<-TRUE
          }
          else
            booCont<-TRUE

          assign("MVName", curValue, controlEnv)

          if (booCont)
          {
            if (curValue != "")
            {
              curMVVal<-curModVar[[curValue]]
              if (is(curMVVal, "logical"))
              {
                newaffyids<-affyids[curMVVal]
  
                # now need to put these in the avail list

                # first remove old values
                numRows<-gtkCListGetRows(availList)

                gtkCListFreeze(availList)

                if (numRows > 0)
                {
#                  gtkCListFreeze(availList)
                  for (i in 1:numRows)
                  {
                    # first remove the item from availList
                    gtkCListRemove(availList, row=0)
                  }
#                  gtkCListThaw(availList)
                }

                # now add new values
                if (length(newaffyids)>0)
                  for (i in 1:length(newaffyids))
                  {
                    gtkCListAppend(availList, newaffyids[i])
                  }

                gtkCListSetAutoSort(availList, TRUE)             

                gtkCListThaw(availList)
                # I'm not sure why the scrolled window sometimes disappears!!
                gtkWidgetShow(sc)
              }
            }
            else
            {
              newaffyids<-affyids

              # first remove old values
              numRows<-gtkCListGetRows(availList)

              gtkCListFreeze(availList)

              if (numRows > 0)
              {
#                gtkCListFreeze(availList)
                for (i in 1:numRows)
                {
                  # first remove the item from availList
                  gtkCListRemove(availList, row=0)
                }
#                gtkCListThaw(availList)
              }

              # now add new values
              if (length(newaffyids)>0)
                for (i in 1:length(newaffyids))
                {
                  gtkCListAppend(availList, newaffyids[i])
                }

              gtkCListSetAutoSort(availList, TRUE)   

              gtkCListThaw(availList)
              # I'm not sure why the scrolled window sometimes disappears!!
              gtkWidgetShow(sc)
            }
          }
          return(TRUE)
        }          
      )
    }

    ontLab<-gtkLabel("Ontology Type")
    gtkTableAttach(tab, ontLab, 1, 3, 5, 6, xpadding=5, ypadding=5)

    # need a drop down box for ontology
    combType<-gtkCombo()
    gtkComboSetPopdownStrings(combType, ontChoices)
      
    gtkComboSetValueInList(combType, TRUE, FALSE)
    gtkTableAttach(tab, combType, 3, 5, 5, 6, xpadding=5, ypadding=5)  

    gtkAddCallback(gtkComboGetList(combType), "select-child",
      function(obj, objchild)
      {
        curValue<-gtkChildren(objchild)[[1]][["label"]]
        assign("ontType", curValue, controlEnv)
        return(TRUE)
      }
    )
    # get the list so the first item can be selected
    lsType<-gtkComboGetList(combType)
    gtkListSelectItem(lsType, 0)   

    doneBut<-gtkButton("Done")
    gtkTableAttach(tab, doneBut, 2, 4, 6, 7, xpadding=5, ypadding=5)

    # here the GO graph child model is created
    gtkAddCallback(doneBut, "clicked",
      function(obj, ev)
      {
        # need to add the function that will create the child model
        # first get the affy ids
        numRows<-gtkCListGetRows(selList)
        if (numRows > 0)
        {
          allaffys<-rep("", numRows)
          for (i in 1:numRows)
            allaffys[i]<-gtkCListGetText(selList, row=(i-1), cols=0)

          # then convert to locus link ids
          LLenv<-paste(curChipType, "LOCUSID", sep="") 
          LLids<-as.character(unlist(mget(allaffys, eval(as.name(LLenv))), 
                 use.names=FALSE))

          # then create the GO graph using the makeGOGraph function
          # need to get the ontology type
          curOnt<-get("ontType", controlEnv)

          # this step seems to take a long time - I'm not sure why??
          newGraph<-makeGOGraph(x=LLids, Ontology=curOnt)

          if (length(nodes(newGraph)) == 0)
          {
            print("There are no GO terms annotated with those Affy Ids.")
          }
          else
          {
            newGraph<-setGoGraphAttr(newGraph, curChipType, LLids, curOnt)

            # then create and handle the add child data message
            dataName<-get("childName", controlEnv)

            # create the virtualData slot
            newVirData<-createGoGraphVirData(curMVC, allaffys, newGraph, 
                          dataName)

            newAddChildMessage<-new("gAddChildMessage", dataName=dataName,
                             type="graph", data=list(data=newGraph,
                             virtualData=newVirData))

            handleMessage(newAddChildMessage, subsetParentData=allaffys,
                          toParent=linkNodeToRowName,
                          fromParent=linkRowNameToNode)

            # should set the chipType in the new child MVC's controller (right 
            # now it is just set in the parent controller)
            childMVC<-getMVC(dataName)
            childControlEnv<-controller(childMVC)
            assign("chipType", curChipType, childControlEnv)
            # need to also set variables that refer to environments to change
            # from GO term to affy id and vice versa
            assign("nodeToRNEnv", paste(curChipType, "GO2ALLPROBES", sep=""), 
                   childControlEnv)
            assign("rnToNodeEnv", paste(curChipType, "GO", sep=""), 
                   childControlEnv) 
            assign("nodeParentsEnv", paste("GO", curOnt, "ANCESTOR", sep=""),
                   childControlEnv)
          }
        }
        w$Destroy()
        return(TRUE)
      }
    )

    w$Show()
  }
}

###########
# added 6/29/06
###########
createKEGGgseVirData<-function(curMVC, allaffys, newGSE)
{
  # should set the virtualData slot so color, pch, highlit, and hide will
  # match the parent data
  # first create the default data frame
  defaultPlotData<-list(color="black", pch=1, highlit=FALSE, hide=FALSE)

  numRows<-length(gsTestStat(newGSE))

  if (length(defaultPlotData) > 0)
  {
    virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                        row.names=rownames(incidMat(newGSE)))
    virData[,1]<-as.character(virData[,1])
    colnames(virData)<-names(defaultPlotData)[1]
  }
  if (length(defaultPlotData) > 1)
  {
    for (i in 2:length(defaultPlotData))
    {
      virData[,i]<-rep(defaultPlotData[[i]], numRows)
    }
    colnames(virData)<-names(defaultPlotData)
  }

  curModel<-model(curMVC)
  parentVirData<-virtualData(curModel)
  subParVirData<-parentVirData[match(allaffys,
                            rownames(parentVirData)), ]

  # check if color is something other than black, if pch is not 1,
  # if highlight is TRUE or if hide is TRUE

  annot<-annotation(modelData(model(curMVC)))
  if (annot=="")
  {
    controlEnv<-controller(curMVC)
    annot<-get("chipType", controlEnv)
  }
  linkEnv<-paste(annot, "LOCUSID", sep="")

  incMat<-incidMat(newGSE)
  if (any(subParVirData$color!="black"))
  {
    # set the appropriate color
    affyColor<-rownames(subParVirData)[(subParVirData$color!="black")]
    # may have different colors so need to look at affy ids 
    # individually
    for (i in 1:length(affyColor))
    {
      # first need to find the LocusID
      LLid<-unlist(mget(affyColor[i], eval(as.name(linkEnv))))
      if (!all(is.na(LLid)))
      {
        LLid<-LLid[!is.na(LLid)]
        # now need to find out which gene sets this LLid is in
        # can use the incidence matrix for that
        colIndex<-match(LLid, colnames(incMat)) 

        # now get the rownames (i.e. the gene set names that correspond with 
        # those genes)
        curGS<-rownames(incMat)[which(incMat[, colIndex]==1)]

        virData[match(curGS, rownames(virData)), 
                match("color", colnames(virData))]<-
          subParVirData[match(affyColor[i], rownames(subParVirData)),
                        match("color", colnames(subParVirData))]
      }
    }        
  }
  if (any(subParVirData$pch!=1))
  {
    # set the appropriate pch
    affyPch<-rownames(subParVirData)[(subParVirData$pch!=1)]
    # may have different pch so need to look at pch 
    # individually
    for (i in 1:length(affyPch))
    {
      # first need to find the LocusID
      LLid<-unlist(mget(affyPch[i], eval(as.name(linkEnv))))
      if (!all(is.na(LLid)))
      {
        LLid<-LLid[!is.na(LLid)]
        # now need to find out which gene sets this LLid is in
        # can use the incidence matrix for that
        colIndex<-match(LLid, colnames(incMat)) 

        # now get the rownames (i.e. the gene set names that correspond with 
        # those genes)
        curGS<-rownames(incMat)[which(incMat[, colIndex]==1)]

        virData[match(curGS, rownames(virData)), 
                match("pch", colnames(virData))]<-
          subParVirData[match(affyPch[i], rownames(subParVirData)),
                        match("pch", colnames(subParVirData))]
      }
    }        
  }
  if (any(subParVirData$highlit==TRUE))
  {
    # set the appropriate highlight value
    affyHigh<-rownames(subParVirData)[(subParVirData$highlit==TRUE)]
    for (i in 1:length(affyHigh))
    {
      # first need to find the LocusID
      LLid<-unlist(mget(affyHigh[i], eval(as.name(linkEnv))))
      if (!all(is.na(LLid)))
      {
        LLid<-LLid[!is.na(LLid)]
        # now need to find out which gene sets this LLid is in
        # can use the incidence matrix for that
        colIndex<-match(LLid, colnames(incMat)) 

        # now get the rownames (i.e. the gene set names that correspond with 
        # those genes)
        curGS<-rownames(incMat)[which(incMat[, colIndex]==1)]

        virData[match(curGS, rownames(virData)), 
                match("highlit", colnames(virData))]<-
          subParVirData[match(affyHigh[i], rownames(subParVirData)),
                        match("highlit", colnames(subParVirData))]
      }
    }        
  }
  if (any(subParVirData$hide==TRUE))
  {
    # set the appropriate hide value
    affyHide<-rownames(subParVirData)[(subParVirData$hide==TRUE)]
    for (i in 1:length(affyHide))
    {
      # first need to find the LocusID
      LLid<-unlist(mget(affyHide[i], eval(as.name(linkEnv))))
      if (!all(is.na(LLid)))
      {
        LLid<-LLid[!is.na(LLid)]
        # now need to find out which gene sets this LLid is in
        # can use the incidence matrix for that
        colIndex<-match(LLid, colnames(incMat)) 

        # now get the rownames (i.e. the gene set names that correspond with 
        # those genes)
        curGS<-rownames(incMat)[which(incMat[, colIndex]==1)]

        virData[match(curGS, rownames(virData)), 
                match("hide", colnames(virData))]<-
          subParVirData[match(affyHide[i], rownames(subParVirData)),
                        match("hide", colnames(subParVirData))]
      }
    }        
  }
  return(virData)
}

################
# changed 3/1/06
# to create the virtualData slot for a new graph model created by using
# createGoGraph function
################
createGoGraphVirData<-function(curMVC, allaffys, newGraph, dataName)
{
  # should set the virtualData slot so fillcolor, color and hide will
  # match the parent data
  # virtual data slot needs to be a data frame
  # first create the default data frame
  defaultPlotData<-list(color="black", fillcolor="white", hide=FALSE)

  numRows<-length(nodes(newGraph))

  if (length(defaultPlotData) > 0)
  {
    virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=nodes(newGraph))
    virData[,1]<-as.character(virData[,1])
    colnames(virData)<-names(defaultPlotData)[1]
  }
  if (length(defaultPlotData) > 1)
  {
    for (i in 2:length(defaultPlotData))
    {
      virData[,i]<-rep(defaultPlotData[[i]], numRows)
    }
    colnames(virData)<-names(defaultPlotData)
  }

  # now change the values depending on what's in the parent model

#  newVirData<-NULL

  curModel<-model(curMVC)
  parentVirData<-virtualData(curModel)
  subParVirData<-parentVirData[match(allaffys,
                            rownames(parentVirData)), ]
  # check if color is something other than black, if highlight is 
  # TRUE or if hide is TRUE
#  newFillColors<-rep("transparent", length(nodes(newGraph)))
#  newColors<-rep("black", length(nodes(newGraph)))
#  newHide<-rep(FALSE, length(nodes(newGraph)))

  controlEnv<-controller(curMVC)
  curChipType<-get("chipType", controlEnv)
  linkEnv<-paste(curChipType, "GO", sep="")

  if (any(subParVirData$color!="black"))
  {
    # set the fillcolor to whatever the color is
    affyColor<-rownames(subParVirData)[(subParVirData$color!="black")]
    # may have different colors so need to look at affy ids 
    # individually
    for (i in 1:length(affyColor))
    {
      # now convert these affy ids to GO terms
      GOterms<-mget(affyColor[i], eval(as.name(linkEnv)))
      if (!all(is.na(GOterms)))
      {
        allGOterms<-unlist(lapply(GOterms, function(x) 
          {
            names(x)
          }
        ), use.names=FALSE)

        # now determine which GO terms are in the new model
        colorNodes<-nodes(newGraph)[nodes(newGraph) %in% allGOterms]
        # get the indeces of these nodes
        nodeIndex<-match(colorNodes, nodes(newGraph))

        virData[nodeIndex, match("fillcolor", colnames(virData))]<-
                                 subParVirData$color[match(affyColor[i],
                                 rownames(subParVirData))]
#        newFillColors[nodeIndex]<-subParVirData$color[match(affyColor[i], 
#                                  rownames(subParVirData))]
      }
    }            
  }          
  if (any(subParVirData$highlit==TRUE))
  {
    # set the color to "red"
    affyHigh<-rownames(subParVirData)[(subParVirData$highlit==TRUE)]
    for (i in 1:length(affyHigh))
    {
      # now convert these affy ids to GO terms
      GOterms<-mget(affyHigh[i], eval(as.name(linkEnv)))
      if (!all(is.na(GOterms)))
      {
        allGOterms<-unlist(lapply(GOterms, function(x) 
          {
            names(x)
          }
        ), use.names=FALSE)
        # now determine which GO terms are in the new model
        highNodes<-nodes(newGraph)[nodes(newGraph) %in% allGOterms]
        # get the indeces of these nodes
        highIndex<-match(highNodes, nodes(newGraph))
        virData[highIndex, match("color", colnames(virData))]<-"red"
#        newColors[highIndex]<-"red"
      }
    }
  }
  if (any(subParVirData$hide==TRUE))
  {
    # set the color and fillcolor to "white" for the nodes that match
    # this affy id
    affyHide<-rownames(subParVirData)[(subParVirData$hide==TRUE)]
    for (i in 1:length(affyHide))
    {
      # now convert these affy ids to GO terms
      GOterms<-mget(affyHide[i], eval(as.name(linkEnv)))
      if (!all(is.na(GOterms)))
      {
        allGOterms<-unlist(lapply(GOterms, function(x) 
          {
            names(x)
          }
        ), use.names=FALSE)
        # now determine which GO terms are in the new model
        hideNodes<-nodes(newGraph)[nodes(newGraph) %in% allGOterms]
        # get the indeces of these nodes
        hideIndex<-match(hideNodes, nodes(newGraph))
        virData[hideIndex, match("hide", colnames(virData))]<-TRUE
#        newColors[hideIndex]<-"white"    
      }
    }
  }

#  newVirData<-agopen(newGraph, name=dataName, 
#                      nodeAttrs=makeNodeAttrs(newGraph, label="", 
#                      fillcolor=newFillColors, color=newColors))
#  return(newVirData)
  return(virData)
}

###########
# perform clustering using MDS
# could be used for either data.frame or exprSet models
###########
createMDSCluster<-function()
{
  # will need to set k (default is 2)
  # may also want to set the method of calculating the distance
  # will need to decide whether to use a subset of the data
  #  for now assume that no subsetting needs to be done
  w<-gtkWindow(show=FALSE)
  w$SetUsize(400, 200)
 
  # 6 rows and 6 columns
  tab<-gtkTableNew(6, 6, TRUE)
  vbox<-gtkVBoxNew()

  w$Add(vbox)
  gtkBoxPackStart(vbox, tab)

  # need to get the active MVC
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  # need the data
  curData<-modelData(model(curMVC))
  isDf<-is(curData, "data.frame")
  # need a matrix (or data.frame) to perform the distance function on
  if (!isDf)
    clusterData<-exprs(curData)
  else
    clusterData<-curData

  # will need two labels (one may be hidden) and one text box and one drop
  # down box and one button
  labK<-gtkLabel("k (number of dimensions)")
  gtkTableAttach(tab, labK, 0, 3, 1, 2, xpadding=5)
  entryK<-gtkEntry()
  gtkTableAttach(tab, entryK, 3, 6, 1, 2, xpadding=20)
  gtkEntrySetText(entryK, 2)

  if (!isDf)
  {
    labClusterOn<-gtkLabel("Cluster On")
    gtkTableAttach(tab, labClusterOn, 0, 3, 3, 4, xpadding=5)

    combCluster<-gtkCombo()
    # for now only allow the user to cluster based on genes rather than 
    # samples - did this because it makes the link data functions easier to
    # create 
#    clusterPoss<-c("genes", "samples")
    clusterPoss<-c("genes")
    gtkComboSetPopdownStrings(combCluster, clusterPoss)

    gtkComboSetValueInList(combCluster, TRUE, FALSE)
    gtkTableAttach(tab, combCluster, 3, 6, 3, 4, xpadding=20)  

    gtkAddCallback(gtkComboGetList(combCluster), "select-child",
      function(obj, objchild)
      {
        curValue<-gtkChildren(objchild)[[1]][["label"]]
        assign("curVariable", curValue, controlEnv)
        return(TRUE)
      }
    )

    # get the list so the first item can be selected
    lsVar<-gtkComboGetList(combCluster)
    gtkListSelectItem(lsVar, 0)
  }

  doneBut<-gtkButton("Done")
  gtkTableAttach(tab, doneBut, 2, 4, 5, 6)
   
  gtkAddCallback(doneBut, "clicked",
    function(obj, ev)
    {
      # need to check if k is numeric
      curK<-gtkEntryGetText(entryK)
      if (is.na(as.numeric(curK)))
        showLabel("k must be a number")
      else
      {
        if (!isDf)
        {
          curVariable<-get("curVariable", controlEnv)
          if (curVariable == "samples")
            clusterData<-t(clusterData)        
        }
        # now need to create the child model
        newModelData<-cmdscale(dist(clusterData), k=curK)

        # will need to link this back to the parent model
        dataName<-get("childName", controlEnv)
        newAddChildMessage<-new("gAddChildMessage", dataName=dataName,
                             type="data.frame", data=list(data=newModelData))

        # need to add toParent, fromParent, and subsetParentData to the
        # handleMessage call
        # currently not subsetting the data so use all row names
        subRowNames<-rownames(clusterData)

        handleMessage(newAddChildMessage, subsetParentData=subRowNames,
                     toParent=linkToParentByRowName, 
                     fromParent=linkToChildByRowName)

        assign("childName", "", controlEnv)

        w$Destroy()
      }
      return(TRUE)
    }
  )

  w$Show()
}

##########
# 9/10/05 create a general function that converts an update data 
# message for a graph model to an update data message for a dfModel
# or an exprModel
#
# there are 3 steps:
# 1. determine the nodes that changed
# 2. convert the nodes to row names using an environment
# 3. create the new update data message
##########
linkNodeToRowName<-function(updateMessage)
{
  # first determine if this message is going to a parent or a child
  # need to look in the child MVC for the controller variables
  if (is(updateMessage, "gSendParentMessage"))
  {
    # means we are updating the parent MVC (a child was just updated)
    booUpdateChild<-FALSE

    curUpdateDataMessage<-childUpdateDataMessage(updateMessage)
    chName<-dataName(curUpdateDataMessage)
    chMVC<-getMVC(chName)
    paName<-parentMVC(chMVC)
    toName<-paName
    fromName<-chName
    chControlEnv<-controller(chMVC)

    # get the nodes that changed
    curNodes<-names(mData(curUpdateDataMessage))
  }
  else
  {
    # means we are updating a child MVC (a parent was just updated)
    booUpdateChild<-TRUE

    curUpdateDataMessage<-parentUpdateDataMessage(updateMessage)
    chName<-childName(updateMessage)
    chMVC<-getMVC(chName)
    paName<-dataName(curUpdateDataMessage)
    toName<-chName
    fromName<-paName
    chControlEnv<-controller(chMVC)

    subsetParentData<-get("subsetParentData", chControlEnv)
    # check that the nodes that were updated were actually used to create
    # the child model
    curNodes<-names(mData(curUpdateDataMessage))
    curNodes<-curNodes[(curNodes %in% subsetParentData)]
  }

  if (length(curNodes) > 0)
  {
    # next need to convert the nodes to row names 
    nodeToRNEnv<-get("nodeToRNEnv", chControlEnv)

    # get all of the row names that match according to the environment
    if (all(curNodes %in% ls(eval(as.name(nodeToRNEnv))))) 
      allRN<-unlist(mget(curNodes, eval(as.name(nodeToRNEnv))), 
                           use.names=FALSE)
    else
      allRN<-c()

    # next need to check if only a subset of the row names are needed
    if (booUpdateChild)
    {
      # do I need to check if the row names are in the child model?
      # might as well
      cModelData<-modelData(model(chMVC))
      if (is(cModelData, "exprSet"))
        cRN<-geneNames(cModelData)
      else
        cRN<-rownames(cModelData)
      updRN<-cRN[(cRN %in% allRN)]
    }
    else
    {
      # first check that the row names are part of the parent subset
      subsetParentData<-get("subsetParentData", chControlEnv)
      updRN<-subsetParentData[(subsetParentData %in% allRN)]
    }

    if (length(updRN) > 0)
    {
      # need to create an update data message for these row names
      # going from an update data message for a graph model to an update
      # data message for a df or expr model
      curType<-type(curUpdateDataMessage)

      # remove the name on curType (because it will be node or edge)
      names(curType)<-rep(c(), length(curType))

      # need to determine how the mData slot will be set
      curData<-c()
      # currently curType can be one of 3 possibilities: 
      # 1. color  2. highlit  3. hide
      # fillcolor means that the color of the node is changing
      if (curType=="fillcolor")  
      {
        curColor<-unique(unlist(mData(curUpdateDataMessage), use.names=FALSE))

        # need to change both color and pch when the model to be updated is
        # either dfModel or exprModel (where there are row names and the 
        # virtualData slot is filled with a data frame)
        
        curType<-c("color", "pch")
        curPch<-19

        curData<-vector(mode="list", length=(length(updRN)*2))
        curData[1:length(updRN)]<-rep(curColor, length(updRN))
        curData[(length(updRN)+1):(length(updRN)*2)]<-
                                  rep(curPch, length(updRN))

        names(curData)<-rep(updRN, 2)
      }
      else
      {
        # color means highlight is being changed
        if (curType=="color")
        {
          curType<-"highlit"
          # want to be able to toggle highlit so need to check color
          curmData<-mData(curUpdateDataMessage)
          curColors<-unlist(curmData, use.names=FALSE)
          if (all(curColors == "red"))
          {
            # this means all the nodes became highlit
            curData<-as.list(rep(TRUE, length(updRN)))
            names(curData)<-updRN
          }
          else
          {
            # this means the nodes were un-highlit
            curData<-as.list(rep(FALSE, length(updRN)))
            names(curData)<-updRN
          }
        }
        if (curType=="hide")
        {
          # 6/12/06 only hide row names if all the nodes they are annotated 
          # at are hidden
          # for the updRN we need to get all the nodes they are annotated at
          # and determine if all those nodes are hidden - this requires 2
          # environments
          rnToNodeEnv<-get("rnToNodeEnv", chControlEnv)
          nodeParentsEnv<-get("nodeParentsEnv", chControlEnv)

          allNodes<-mget(updRN, eval(as.name(rnToNodeEnv)))

          rowsToUpdate<-rep(FALSE, length(allNodes))
          fromDat<-getData(fromName)
          fromVd<-virtualData(model(getMVC(fromName)))

          curmData<-mData(curUpdateDataMessage)
          # problem: sometimes mData is longer than 1
          curValue<-unique(unlist(curmData, use.names=FALSE))

          for (i in 1:length(allNodes))
          {
            curNodes<-unique(names(allNodes[[i]]))

            # now need to see if all these nodes are hidden
            curNodes<-curNodes[curNodes %in% nodes(fromDat)]

            # now need to get the ancestors of these specific nodes
            acurNodes<-unlist(mget(curNodes, eval(as.name(nodeParentsEnv))),
                         use.names=FALSE)
            curNodes<-unique(c(curNodes, acurNodes))
            curNodes<-curNodes[(curNodes %in% nodes(fromDat))]

            # now look at the virtual data slot for these nodes
            hidVals<-fromVd[match(curNodes, rownames(fromVd)), 
			     match("hide", colnames(fromVd))]

            if (all(hidVals==TRUE) && curValue==TRUE)
              rowsToUpdate[i]<-TRUE

            # also need to think of a way to reshow rows!!!!!!!!
            # redraw if curValue==FALSE and any hidValus and the value
            # for that row is currently TRUE - would need to look
            if (curValue==FALSE)
            {
              # need to check if the corresponding row name is hidden
              currentRN<-updRN[i]
              toVd<-virtualData(model(getMVC(toName))) 
              curHidVal<-toVd[match(currentRN, rownames(toVd)),
                               match("hide", colnames(toVd))]
              if (curHidVal==TRUE)
                rowsToUpdate[i]<-TRUE
            }
          }

          if (any(rowsToUpdate))
          {
            updRN<-updRN[rowsToUpdate]
            curData<-as.list(rep(curValue, length(updRN)))
            names(curData)<-updRN
          }
          else
          {
            curData<-c()
          }
        } 
      }

      # now create the new update data message
      if (length(curData) > 0)   
      {
        updDMessage<-new("gUpdateDataMessage", type=curType, mData=curData, 
                     dataName=toName, from=fromName)
        return(updDMessage)
      }
      else
      {
        # there is nothing to update
        return(NULL)
      }
    }
    else
    {
      # there is nothing to update
      return(NULL)
    }
  }
  else
  {
    # there is nothing to update because the nodes that changed were not
    # used to create the child model
    return(NULL)
  }
}

#########
# 9/10/05 create a general function that converts an update data 
# message for a df or expr model to an update data message for a graphModel
#
# there are 3 steps:
# 1. determine the row names that changed
# 2. convert the row names to nodes using an environment (note: this can be 
#     tricky because of the many-to-many relationships between row names and
#     nodes)
# 3. create the new update data message
#########
linkRowNameToNode<-function(updateMessage)
{
  # first determine if this message is going to a parent or a child
  # need to look in the child MVC for the controller variables
  if (is(updateMessage, "gSendParentMessage"))
  {
    # means we are updating the parent MVC (a child was just updated)
    booUpdateChild<-FALSE

    curUpdateDataMessage<-childUpdateDataMessage(updateMessage)
    chName<-dataName(curUpdateDataMessage)
    chMVC<-getMVC(chName)
    paName<-parentMVC(chMVC)
    toName<-paName
    fromName<-chName
    graphModel<-model(getMVC(paName))
    chControlEnv<-controller(chMVC)

    # get the row names that changed
    curRN<-names(mData(curUpdateDataMessage))
  }
  else
  {
    # means we are updating a child MVC (a parent was just updated)
    booUpdateChild<-TRUE

    curUpdateDataMessage<-parentUpdateDataMessage(updateMessage)
    chName<-childName(updateMessage)
    chMVC<-getMVC(chName)
    paName<-dataName(curUpdateDataMessage)
    toName<-chName
    fromName<-paName
    graphModel<-model(chMVC)
    chControlEnv<-controller(chMVC)

    subsetParentData<-get("subsetParentData", chControlEnv)
    # check that the nodes that were updated were actually used to create
    # the child model
    curRN<-names(mData(curUpdateDataMessage))
    curRN<-curRN[(curRN %in% subsetParentData)]
  }
  curRN<-unique(curRN)

  if (length(curRN) > 0)
  {
    # next need to convert the row names to nodes 
    rnToNodeEnv<-get("rnToNodeEnv", chControlEnv)

    # get all of the nodes that match according to the environment
    allNodes<-mget(curRN, eval(as.name(rnToNodeEnv)))
    allNodes<-unique(unlist(lapply(allNodes, names), use.names=FALSE))

    # next need to check if only a subset of the nodes are needed
    if (booUpdateChild)
    {
      # do I need to check if the nodes are in the child model?
      # might as well
      cModelData<-modelData(model(chMVC))
      cNodes<-nodes(cModelData)
      updNodes<-cNodes[(cNodes %in% allNodes)]      

      # 6/9/06 next need to check if we need to get the parent nodes
      nodeParentsEnv<-get("nodeParentsEnv", chControlEnv)
      if (!is.null(nodeParentsEnv))
      {
        # this returns the ancestors
        aupdNodes<-unlist(mget(updNodes, eval(as.name(nodeParentsEnv))),
                       use.names=FALSE)
        # need to add the ancestors to the specific nodes
        updNodes<-unique(c(updNodes, aupdNodes))
        updNodes<-cNodes[(cNodes %in% updNodes)]
      }      
    }
    else
    {
      # check that the nodes are part of the parent subset
      subsetParentData<-get("subsetParentData", chControlEnv)
      updNodes<-subsetParentData[(subsetParentData %in% allNodes)]

      # 6/9/06 next need to check if we need to get the parent nodes
      nodeParentsEnv<-get("nodeParentsEnv", chControlEnv)
      if (!is.null(nodeParentsEnv))
      {
        aupdNodes<-unlist(mget(updNodes, eval(as.name(nodeParentsEnv))),
                       use.names=FALSE)
        updNodes<-unique(c(updNodes, aupdNodes))
        updNodes<-subsetParentData[(subsetParentData %in% updNodes)]
      }     
    }

    if (length(updNodes) > 0)
    {
      # need to create an update data message for these nodes
      # going from an update data message for a df or expr model to an update
      # data message for a graph model
      curType<-type(curUpdateDataMessage)

      # need to synch the types

      # add name of "node" on curType so the graph will know to update
      # the nodes
      lenType<-length(curType)

      # need to determine how the mData slot will be set
      curData<-c()
      # currently curType can be one of 3 possibilities: 
      # 1. color  2. highlit  3. hide
      if (any(curType=="color"))  
      {
        # curType will include both color and pch!!
        # need to change the color value for an AgNode object
        # how does the mData slot get filled when creating an update data
        # message for a graph model????
        # first need to get the current color 
        curColor<-unique(unlist(mData(curUpdateDataMessage), use.names=FALSE))
        if (length(curColor) > 1)
          curColor<-curColor[1]

        curType<-c("node"="fillcolor")

        curData<-rep(curColor, length(updNodes))
        curData<-as.list(curData)
        names(curData)<-updNodes
      }
      else
      {
        if (curType=="highlit")
        {
          # need to check if highlight is on or off
          curValue<-unique(unlist(mData(curUpdateDataMessage), 
                    use.names=FALSE))

          curType<-c("node"="color")

          # for now assume that highlighting a node uses a red color
          highlightColor<-"red"
          unHighlightColor<-"black"

          if (curValue == TRUE)
            curData<-rep(highlightColor, length(updNodes))
          else
            curData<-rep(unHighlightColor, length(updNodes))
          names(curData)<-updNodes
          curData<-as.list(curData)
        }
        if (curType=="hide")
        {
          # 6/9/06 here we need to check if only all genes annotated at each 
          # node are hidden
          nodeToRnEnv<-get("nodeToRNEnv", chControlEnv)

          # now get the genes annotated at updNodes
          nodeAnnot<-mget(updNodes, eval(as.name(nodeToRnEnv)))
          booNodes<-rep(FALSE, length(updNodes))
          fromMVC<-getMVC(fromName)
          fromModel<-model(fromMVC)
          fromVd<-virtualData(fromModel)
          fromMd<-modelData(fromModel)
          allGN<-geneNames(fromMd)

          for (i in 1:length(updNodes))
          {
            genesAnnot<-nodeAnnot[[i]]
            # need to find out which of these genes are in the linked data set
            if (any(genesAnnot %in% allGN))
            {
              genesAnnot<-genesAnnot[genesAnnot %in% allGN]
              # now look if they are all hidden
              hidValues<-fromVd[match(genesAnnot, rownames(fromVd)), 
                                match("hide", colnames(fromVd))]

              curValue<-unique(unlist(mData(curUpdateDataMessage), 
                    use.names=FALSE))
              # to hide the proper nodes
              if (all(hidValues==TRUE) && curValue==TRUE)
                booNodes[i]<-TRUE
              # need to reshow the nodes - this will reshow more than just
              # the nodes that were originally hidden!!!!
              if (any(hidValues==FALSE) && curValue==FALSE)
                booNodes[i]<-TRUE
            }   
          }
          if (any(booNodes))
          {
            updNodes<-updNodes[booNodes]

            # need to check if highlight is on or off
            curValue<-unique(unlist(mData(curUpdateDataMessage), 
                    use.names=FALSE))

            curType<-c("node"="hide")

            if (curValue == TRUE)
              curData<-rep(TRUE, length(updNodes))
            else
              curData<-rep(FALSE, length(updNodes))
            names(curData)<-updNodes
            curData<-as.list(curData)
          }
          else
            curData<-c()
        } 
      }

      # now create the new update data message
      if (length(curData) > 0)   
      {
        updDMessage<-new("gUpdateDataMessage", type=curType, mData=curData, 
                     dataName=toName, from=fromName)
        return(updDMessage)
      }
      else
      {
        # there is nothing to update
        return(NULL)
      }
    }
    else
    {
      # there is nothing to update
      return(NULL)
    }
  }
  else
  {
    # there is nothing to update because the nodes that changed were not
    # used to create the child model
    return(NULL)
  }
}

###########
# added 6/29/06
# link gene to a gene set
# changed 7/3/06: only hide a gene set if all genes are hidden
###########
linkGeneToGeneSets<-function(updateMessage)
{
  # first determine if this message is going to a parent or a child
  # need to look in the child MVC for the controller variables
  if (is(updateMessage, "gSendParentMessage"))
  {
    # means we are updating the parent MVC (a child was just updated)
    booUpdateChild<-FALSE

    curUpdateDataMessage<-childUpdateDataMessage(updateMessage)
    chName<-dataName(curUpdateDataMessage)
    chMVC<-getMVC(chName)
    paName<-parentMVC(chMVC)
    toName<-paName
    fromName<-chName
    toMVC<-getMVC(toName)
    fromMVC<-getMVC(fromName)

    # get the genes that changed
    curGenes<-names(mData(curUpdateDataMessage))
  }
  else
  {
    # means we are updating a child MVC (a parent was just updated)
    booUpdateChild<-TRUE

    curUpdateDataMessage<-parentUpdateDataMessage(updateMessage)
    chName<-childName(updateMessage)
    chMVC<-getMVC(chName)
    paName<-dataName(curUpdateDataMessage)
    toName<-chName
    fromName<-paName
    toMVC<-getMVC(toName)
    fromMVC<-getMVC(fromName)

    chControlEnv<-controller(chMVC)

    subsetParentData<-get("subsetParentData", chControlEnv)
    # check that the nodes that were updated were actually used to create
    # the child model
    curGenes<-names(mData(curUpdateDataMessage))
    curGenes<-curGenes[(curGenes %in% subsetParentData)]
  }

  if (length(curGenes) > 0)
  {
    # check if mData has more than one value
    curVals<-unique(mData(curUpdateDataMessage)
                   [names(mData(curUpdateDataMessage)) %in% curGenes])

    # not sure how to update if there is more than one type!!
    # assume type can be one of 3 options: highlit, hide, or color
    curType<-type(curUpdateDataMessage)

    # color sometimes comes as color and pch
    if (all(curType == c("color", "pch")))
    {
      curType<-"color"
      # need to get the current color values only (remove pch values)
      curVals<-curVals[unlist(lapply(curVals, is.character))]
    }

    if (length(curType)==1)
    {
      if (length(curVals) == 1)
      {
        curGenes<-unique(curGenes)

        # now convert the genes into gene sets

        # first need to convert Affymetrix identifiers to locus link ids
        annot<-annotation(modelData(model(fromMVC)))
        if (annot == "")
          annot<-get("chipType", controller(fromMVC))

        linkEnv<-paste(annot, "LOCUSID", sep="")

        LLid<-unlist(mget(curGenes, eval(as.name(linkEnv))))
        LLid<-unique(LLid)

        # now get the gene sets using the incidence matrix
        incMat<-incidMat(modelData(model(toMVC)))
        colIndex<-match(LLid, colnames(incMat)) 

        # now get the rownames (i.e. the gene set names that correspond with 
        # those genes) - these are gene sets to update

        if (length(colIndex) > 1)
        {
          # get a matrix of TRUE/FALSE to determine if a gene is linked to a 
          # gene set
          booMat<-apply(incMat[, colIndex], 2, function(x) {x==1})
          # see if any of the values are TRUE by row - if any are true, then
          # we'll update that gene set
          booVec<-apply(booMat, 1, function(x) any(x))
        }
        else
        {
          booVec<-which(incMat[,colIndex]==1)
        }
        curGS<-rownames(incMat)[booVec]

        if (length(curGS) > 0)
        {
          curGS<-unique(curGS)

          if (curType != "color")
          {
            # if we are hiding a gene (i.e. curType is "hide" and curVals is 
            # TRUE), then we need to check if all genes in a gene set are 
            # hidden
            if (curType=="hide" && curVals==TRUE)
            {
              curmData<-list()
              # need to see if all genes in each gene set are hidden -
              # only then do we send an update data message to hide the 
              # gene set
              for (i in 1:length(curGS))
              {
                # get all the genes in the gene set
                rowIndex<-match(curGS[i], rownames(incMat))
                llToCheck<-colnames(incMat)[which(incMat[rowIndex, ]==1)]
                # need to convert to affy ids
                linkEnvList<-as.list(eval(as.name(linkEnv)))
                affyToCheck<-list()
                for (j in 1:length(llToCheck))
                {
                  affyToCheck[[j]]<-names(linkEnvList)[which(linkEnvList==llToCheck[j])]
                }
                affyToCheck<-unique(unlist(affyToCheck))
                # then get the subset that were used to create this model
                if (booUpdateChild)
                {
                  affyToCheck<-affyToCheck[affyToCheck %in% subsetParentData]
                }

                # determine if all those genes are hidden
                fromVD<-virtualData(model(fromMVC))

                # if all are hidden, then add to curmData list
                if (all(fromVD$hide[match(affyToCheck, rownames(fromVD))]))
                {
                  origLen<-length(curmData)
                  curmData[origLen+1]<-curVals
                  names(curmData)[origLen+1]<-curGS[i]
                }
              }
            } 
            else
            {
              curmData<-as.list(rep(curVals, length(curGS)))
              names(curmData)<-curGS
            }
          }
          else
          {
            # if color, then also need to change pch
            curType<-c("color", "pch")
            curColor<-curVals
            curPch<-as.numeric(19)

            curmData<-vector(mode="list", length=(length(curGS)*2))
            curmData[1:length(curGS)]<-rep(curColor, length(curGS))
            curmData[(length(curGS)+1):(length(curGS)*2)]<-
                                     rep(curPch, length(curGS))
            names(curmData)<-rep(curGS, 2)        
          }

          if (length(curmData) > 0)
          {
            # now need to make the update data message for the gene sets
            updDMessage<-new("gUpdateDataMessage", type=curType, 
                           mData=curmData, dataName=toName, from=fromName)
            return(updDMessage)
          }
          else
            return(NULL)
        }
        else
        {
          # there is nothing to update
          return(NULL)
        }
      }
      else
      {
        # have more than one value

        curmData<-list()

        # first need to convert Affymetrix identifiers to locus link ids
        annot<-annotation(modelData(model(fromMVC)))
        if (annot == "")
          annot<-get("chipType", controller(fromMVC))

        linkEnv<-paste(annot, "LOCUSID", sep="") 

        incMat<-incidMat(modelData(model(toMVC)))       

        # need to loop through the different values
        for (i in 1:length(curVals))
        {
          # get the genes associated with each value
          genesWithVal<-names(mData(curUpdateDataMessage))[mData(curUpdateDataMessage)==curVals[[i]]]
          newcurGenes<-unique(genesWithVal[genesWithVal %in% curGenes])

          LLid<-unlist(mget(newcurGenes, eval(as.name(linkEnv))))
          LLid<-unique(LLid)

          # now get the gene sets using the incidence matrix
          colIndex<-match(LLid, colnames(incMat)) 

          # now get the rownames (i.e. the gene set names that correspond with 
          # those genes) - these are gene sets to update

          if (length(colIndex) > 1)
          {
            # get a matrix of TRUE/FALSE to determine if a gene is linked to a 
            # gene set
            booMat<-apply(incMat[, colIndex], 2, function(x) {x==1})
            # see if any of the values are TRUE by row - if any are true, then
            # we'll update that gene set
            booVec<-apply(booMat, 1, function(x) any(x))
          }
          else
          {
            booVec<-which(incMat[,colIndex]==1)
          }

          curGS<-rownames(incMat)[booVec]

          if (length(curGS) > 0)
          {
            curGS<-unique(curGS)

            if (curType != "color")
            {
              if (length(curmData) == 0)
              {
                curmData<-as.list(rep(curVals[i], length(curGS)))
                names(curmData)<-curGS
              }
              else
              {
                origLen<-length(curmData)
                curmData[(origLen+1):(origLen+length(curGS))]<-
                                rep(curVals[i], length(curGS))
                names(curmData)[(origLen+1):(origLen+length(curGS))]<-curGS
              }
            }
            else
            {
              # if color, then also need to change pch
              curType<-c("color", "pch")
              curColor<-curVals[i]
              curPch<-as.numeric(19)

              if (length(curmData)==0)
              {
                curmData<-vector(mode="list", length=(length(curGS)*2))
                curmData[1:length(curGS)]<-rep(curColor, length(curGS))
                curmData[(length(curGS)+1):(length(curGS)*2)]<-
                                         rep(curPch, length(curGS))
                names(curmData)<-rep(curGS, 2)        
              }
              else
              {
                origLen<-length(curmData)
                curmData[(origLen+1):(origLen+length(curGS))]<-
                                rep(curColor, length(curGS))
                curmData[(origLen+1+length(curGS)):(origLen+2*length(curGS))]<-
                                rep(curPch, length(curGS))
                names(curmData)[(origLen+1):(origLen+2*length(curGS))]<-
                                rep(curGS, 2)                      
              }
            }
          }
        }

        # then send the update data message
        if (length(curmData) > 0)
        {
          # should pare down curmData because may have repeats!
          if (length(unique(names(curmData))) !=  length(names(curmData)))
          {
            # then there are repeats in curmData
            # if the values are boolean, then take the ANY of the values
            # if the values aren't boolean, just take the first value
            newmData<-list()
            for (i in 1:length(unique(names(curmData))))
            {
              curN<-unique(names(curmData))[i]
              # find all the values with this name
              allCVals<-unlist(curmData[names(curmData)==curN])
              if (is(allCVals, "logical"))
              {
                newmData[[i]]<-any(allCVals)
              }
              else
                newmData[[i]]<-allCVals[1]              
            }
            names(newmData)<-unique(names(curmData))
            curmData<-newmData
          }
          
          # now need to make the update data message for the gene sets
          updDMessage<-new("gUpdateDataMessage", type=curType, mData=curmData,
                         dataName=toName, from=fromName)
          return(updDMessage)
        }
        else
        {
          return(NULL)
        }
      }
    }
    else
    {
      # have more than one type and not sure how to deal with that now 
      # currently, there is no way to end up with more than one type - if it's
      # not color and pch as the 2 types
      print("The type must be of length 1")
      return(NULL)
    }
  }
  else
  {
    # there is nothing to update
    return(NULL)
  }
}

##########
# 7/2/06
# link gene set to its genes
##########
linkGeneSetToGenes<-function(updateMessage)
{
  # first determine if this message is going to a parent or a child
  # need to look in the child MVC for the controller variables
  if (is(updateMessage, "gSendParentMessage"))
  {
    # means we are updating the parent MVC (a child was just updated)
    booUpdateChild<-FALSE

    curUpdateDataMessage<-childUpdateDataMessage(updateMessage)
    chName<-dataName(curUpdateDataMessage)
    chMVC<-getMVC(chName)
    paName<-parentMVC(chMVC)
    toName<-paName
    fromName<-chName
    toMVC<-getMVC(toName)
    fromMVC<-getMVC(fromName)

    # get the genes that changed
    curGeneSets<-names(mData(curUpdateDataMessage))

    # get the subset of parent data that made this model
    subsetParentData<-get("subsetParentData", controller(fromMVC))
  }
  else 
  {
    # means we are updating a child MVC (a parent was just updated)
    booUpdateChild<-TRUE

    curUpdateDataMessage<-parentUpdateDataMessage(updateMessage)
    chName<-childName(updateMessage)
    chMVC<-getMVC(chName)
    paName<-dataName(curUpdateDataMessage)
    toName<-chName
    fromName<-paName
    toMVC<-getMVC(toName)
    fromMVC<-getMVC(fromName)

    chControlEnv<-controller(chMVC)
    subsetParentData<-get("subsetParentData", chControlEnv)
    # check that the nodes that were updated were actually used to create
    # the child model
    curGeneSets<-names(mData(curUpdateDataMessage))
    curGeneSets<-curGeneSets[(curGeneSets %in% subsetParentData)]
  }

  if (length(curGeneSets) > 0)
  {
    # check if mData has more than one value
    curVals<-unique(mData(curUpdateDataMessage)
                   [names(mData(curUpdateDataMessage)) %in% curGeneSets])

    # not sure how to update if there is more than one type!!
    # assume type can be one of 3 options: highlit, hide, or color
    curType<-type(curUpdateDataMessage)

    # color sometimes comes as color and pch
    if (all(curType == c("color", "pch")))
    {
      curType<-"color"
      # need to get the current color values only (remove pch values)
      curVals<-curVals[unlist(lapply(curVals, is.character))]
    }

    if (length(curType)==1)
    {
      if (length(curVals) == 1)
      {
        curGeneSets<-unique(curGeneSets)

        # now convert the gene sets into genes

        # first get the genes in the gene sets using the incidence matrix
        incMat<-incidMat(modelData(model(fromMVC)))
        GSindex<-match(curGeneSets, rownames(incMat))
        if (length(GSindex) > 1)
        {
          sumVec<-apply(incMat[GSindex, ], 2, sum)
          curLL<-colnames(incMat)[which(sumVec > 0)]
        }
        else
        {
          curLL<-colnames(incMat)[which(incMat[GSindex, ]==1)]
        }

        # now need to convert LL ids to affy ids - use the link environment
        annot<-annotation(modelData(model(toMVC)))
        if (annot == "")
          annot<-get("chipType", controller(toMVC))

        linkEnv<-paste(annot, "LOCUSID", sep="")

        # now get the affy ids

        # need to convert to affy ids
        linkEnvList<-as.list(eval(as.name(linkEnv)))
        curAffy<-list()
        for (j in 1:length(curLL))
        {
          curAffy[[j]]<-names(linkEnvList)[which(linkEnvList==curLL[j])]
        }
        curAffy<-unique(unlist(curAffy))

        # then get the subset that were used to create this model
        if (!booUpdateChild)
        {
          curAffy<-curAffy[curAffy %in% subsetParentData]
        }

        # then we need to update these genes
        if (length(curAffy) > 0)
        {
          if (curType != "color")
          {
            # if we are hiding a gene set (i.e. curType is "hide" and curVals
            # is TRUE), then we need to check that all gene sets that a gene
            # is in are hidden
            if (curType=="hide" && curVals==TRUE)
            {
              curmData<-list()
              fromVD<-virtualData(model(fromMVC))
              # need to see if all gene sets that a gene is in are hidden - 
              # only then do we send an update data message to hide the gene
              for (i in 1:length(curAffy))
              {
                # get the LL id
                curLLid<-get(curAffy[i], eval(as.name(linkEnv)))

                # then find its gene sets using the incidMat
                colIndex<-match(curLLid, colnames(incMat))
                
                curGS<-rownames(incMat)[which(incMat[, colIndex]==1)]

                if (length(curGS) > 0)
                {
                  # check the virtual data to see if they are all hidden
                  if (all(fromVD$hide[match(curGS, rownames(fromVD))]))
                  {
                    origLen<-length(curmData)
                    curmData[origLen+1]<-curVals
                    names(curmData)[origLen+1]<-curAffy[i]
                  }
                }
              }
            } 
            else
            {
              curmData<-as.list(rep(curVals, length(curAffy)))
              names(curmData)<-curAffy
            }
          }
          else
          {
            # if color, then also need to change pch
            curType<-c("color", "pch")
            curColor<-curVals
            curPch<-as.numeric(19)

            curmData<-vector(mode="list", length=(length(curAffy)*2))
            curmData[1:length(curAffy)]<-rep(curColor, length(curAffy))
            curmData[(length(curAffy)+1):(length(curAffy)*2)]<-
                                     rep(curPch, length(curAffy))
            names(curmData)<-rep(curAffy, 2)        
          }

          if (length(curmData) > 0)
          {
            # now need to make the update data message for the gene sets
            updDMessage<-new("gUpdateDataMessage", type=curType, 
                           mData=curmData, dataName=toName, from=fromName)
            return(updDMessage)
          }
          else
            return(NULL)
        }
        else
        {
          # there is nothing to update
          return(NULL)
        }
      }
      else
      {
        # have more than one value - for example if highlighting some gene
        #  sets and removing highlighting on other gene sets

        curmData<-list()

        # first need to convert Affymetrix identifiers to locus link ids
        annot<-annotation(modelData(model(toMVC)))
        if (annot == "")
          annot<-get("chipType", controller(toMVC))

        linkEnv<-paste(annot, "LOCUSID", sep="") 

        incMat<-incidMat(modelData(model(fromMVC)))       

        # need to loop through the different values - still just have one
        #  type, though 
        for (i in 1:length(curVals))
        {
          # get the gene sets associated with each value
          gsWithVal<-names(mData(curUpdateDataMessage))[mData(curUpdateDataMessage)==curVals[[i]]]
          newcurGS<-unique(gsWithVal[gsWithVal %in% curGeneSets])
       
          # now get the genes associated with these gene sets

          # first get the genes in the gene sets using the incidence matrix
          GSindex<-match(curGeneSets, rownames(incMat))
          if (length(GSindex) > 1)
          {
            sumVec<-apply(incMat[GSindex, ], 2, sum)
            curLL<-colnames(incMat)[which(sumVec > 0)]
          }
          else
          {
            curLL<-colnames(incMat)[which(incMat[GSindex, ]==1)]
          }

          # now need to convert LL ids to affy ids - use the link environment
          # now get the affy ids
          # need to convert to affy ids
          linkEnvList<-as.list(eval(as.name(linkEnv)))
          curAffy<-list()
          for (j in 1:length(curLL))
          {
            curAffy[[j]]<-names(linkEnvList)[which(linkEnvList==curLL[j])]
          }
          curAffy<-unique(unlist(curAffy))

          # then get the subset that were used to create this model
          if (!booUpdateChild)
          {
            curAffy<-curAffy[curAffy %in% subsetParentData]
          }

          # then we need to update these genes
          if (length(curAffy) > 0)
          {
            curAffy<-unique(curAffy)

            if (curType != "color")
            {
              # note: assume if curType is hide, then we have only one value
              # so we don't need to take curType == hide into account here
              if (length(curmData) == 0)
              {
                curmData<-as.list(rep(curVals[i], length(curAffy)))
                names(curmData)<-curAffy
              }
              else
              {
                origLen<-length(curmData)
                curmData[(origLen+1):(origLen+length(curAffy))]<-
                                rep(curVals[i], length(curAffy))
                names(curmData)[(origLen+1):(origLen+length(curAffy))]<-curAffy
              }
            }
            else
            {
              # if color, then also need to change pch
              curType<-c("color", "pch")
              curColor<-curVals[i]
              curPch<-as.numeric(19)

              if (length(curmData)==0)
              {
                curmData<-vector(mode="list", length=(length(curAffy)*2))
                curmData[1:length(curAffy)]<-rep(curColor, length(curAffy))
                curmData[(length(curAffy)+1):(length(curAffy)*2)]<-
                                         rep(curPch, length(curAffy))
                names(curmData)<-rep(curAffy, 2)        
              }
              else
              {
                origLen<-length(curmData)
                curmData[(origLen+1):(origLen+length(curAffy))]<-
                                rep(curColor, length(curAffy))
                curmData[(origLen+1+length(curAffy)):
                         (origLen+2*length(curAffy))]<-
                                rep(curPch, length(curAffy))
                names(curmData)[(origLen+1):(origLen+2*length(curAffy))]<-
                                rep(curAffy, 2)
              }
            }
          }
        }
        # end of for loop

        # then send the update data message
        if (length(curmData) > 0)
        {
          # should pare down curmData because may have repeats!
          if (length(unique(names(curmData))) !=  length(names(curmData)))
          {
            # then there are repeats in curmData
            # if the values are boolean, then take the ANY of the values
            # if the values aren't boolean, just take the first value
            newmData<-list()
            for (i in 1:length(unique(names(curmData))))
            {
              curN<-unique(names(curmData))[i]
              # find all the values with this name
              allCVals<-unlist(curmData[names(curmData)==curN])
              if (is(allCVals, "logical"))
              {
                newmData[[i]]<-any(allCVals)
              }
              else
                newmData[[i]]<-allCVals[1]              
            }
            names(newmData)<-unique(names(curmData))
            curmData<-newmData
          }
          
          # now need to make the update data message for the gene sets
          updDMessage<-new("gUpdateDataMessage", type=curType, mData=curmData,
                         dataName=toName, from=fromName)
          return(updDMessage)
        }
        else
        {
          return(NULL)
        }
      }
    }
    else
    {
      # have more than one type and not sure how to deal with that now 
      # currently, there is no way to end up with more than one type - if it's
      # not color and pch as the 2 types
      print("The type must be of length 1")
      return(NULL)
    }
  }
  else
  {
    # there is nothing to update
    return(NULL)
  }
}

##########
# 8/31/05 the toParent function must take an update data message
# that comes from the child and create an update data message for the
# parent
#
# this function works when the child model is linked to the parent model by
# row names (it would also work if the linking is by column names)
##########
linkToParentByRowName<-function(updateParentMessage)
{
  curchildUpdateDataMessage<-childUpdateDataMessage(updateParentMessage)

  childName<-dataName(curchildUpdateDataMessage)
  childMVC<-getMVC(childName)
  parentName<-parentMVC(childMVC)
  curtype<-type(curchildUpdateDataMessage)
  data<-mData(curchildUpdateDataMessage)

  # now create the new message
  parentMessage<-new("gUpdateDataMessage", type=curtype, mData=data, 
                      dataName=parentName, from=childName) 
  return(parentMessage)
}

#########
# the fromParent function when the child model relates to the parent model
# by row names
#########
linkToChildByRowName<-function(updateChildMessage)
{
  # first check if the child has the data that is in the parent
  # note that the child model may not have all the row names that the 
  # parent model has
  curparentUpdateDataMessage<-parentUpdateDataMessage(updateChildMessage)
  parentName<-dataName(curparentUpdateDataMessage)
  curChildName<-childName(updateChildMessage)

  curChildMVC<-getMVC(curChildName)
  childModelData<-modelData(model(curChildMVC))
  childRowNames<-rownames(childModelData)

  curtype<-type(curparentUpdateDataMessage)
  data<-mData(curparentUpdateDataMessage)

  # the data that was changed in the parent must be in the child to 
  # update the child MVC
  changeRows<-names(data)

  # need to check that the child has the data that the parent has just updated
  if (any(changeRows %in% childRowNames))
  {
    if (all(changeRows %in% childRowNames))
      childMessage<-new("gUpdateDataMessage", type=curtype, mData=data,
                        dataName=curChildName, from=parentName)
    else
    {
      curIndex<-which(changeRows %in% childRowNames)
      childMessage<-new("gUpdateDataMessage", type=curtype, 
                        mData=data[curIndex], dataName=curChildName, 
                        from=parentName)
    }
    return(childMessage)
  }
  else
    return(NULL)
}

###########
# 6/26/06
# let the user load a model variable through the GUI
###########
setLoadModelVarView<-function()
{
  if (length(get("MVCList", mvcEnv)) > 0)
  {
    w<-get("controlWin", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()
    else
    {
      # make sure old action gets set when the first window is loaded
      assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
    }
  
    # 9 rows and 9 columns
    tab<-gtkTableNew(9, 9, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }

    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)

    lblMVC<-gtkLabel("Loaded MVCs (Models)")
    gtkTableAttach(tab, lblMVC, 1, 4, 1, 2)

    combMName<-gtkCombo()
    gtkComboSetPopdownStrings(combMName, getModelNames())
    gtkComboSetValueInList(combMName, TRUE, FALSE)
    gtkTableAttach(tab, combMName, 5, 8, 1, 2)  

    lblMVName<-gtkLabel("Model Variable Name")
    gtkTableAttach(tab, lblMVName, 1, 4, 3, 4)
    nameEntry<-gtkEntry()
    gtkTableAttach(tab, nameEntry, 5, 8, 3, 4)

    lblMVar<-gtkLabel("Model Variable")
    gtkTableAttach(tab, lblMVar, 1, 4, 5, 6)
    varEntry<-gtkEntry()
    gtkTableAttach(tab, varEntry, 5, 8, 5, 6)

    # next need the load button
    loadBut<-gtkButton("Load")
    gtkTableAttach(tab, loadBut, 3, 6, 7, 8, xpadding=20)

    gtkAddCallback(loadBut, "clicked",
      function(obj, ev)
      {
        # need to get the values of everything on the GUI

        # get the model name
        curMVCName<-gtkEntryGetText(gtkContainerGetChildren(combMName)[[1]])

        # get the entry text
        curNameText<-gtkEntryGetText(nameEntry)      
        curVarText<-gtkEntryGetText(varEntry)

        if (curNameText != "" && curVarText != "")
        {
          # should also check the name and the variable
          curMVC<-getMVC(curMVCName)

          namesMV<-names(modelVar(model(curMVC)))

          # only continue if the model variable name is not already used
          if (!(curNameText %in% namesMV))
          {
            # next check that the variable exists in .GlobalEnv
            if (curVarText %in% ls(.GlobalEnv))
            {
              loadModelVar(curMVCName, curNameText, eval(as.name(curVarText)))
              updateCurAction("loadMV")
              showLabel(paste(curNameText, "was loaded as a model variable."))
            }
            else
              showLabel("That variable is not loaded in .GlobalEnv.")
          }
          else
            showLabel("That model variable name is already in use.")
        }
        else
        {
          showLabel(paste("The model variable name and model variable must",
                          "contain a non-empty character string."))
        }
        return(TRUE)  
      }
    )
  }
}

#############
# setLoadModelView sets the control window so the
# user can choose which list to load as the top model
#############
setLoadModelView<-function()
{
  w<-get("controlWin", mvcEnv)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()
  else
  {
    # make sure old action gets set when the first window is loaded
    assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
  }

  # 10 rows and 5 columns
  tab<-gtkTableNew(10, 5, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
  frMVC$Add(tab)

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  possLists<-getPossLists(type="data.frame")
  assign("typeModel", "data.frame", mvcEnv)

  frPotential<-gtkFrame("Potential Models")
  boxForPotential<-gtkVBox(TRUE,2)
  sc<-gtkScrolledWindow()
  sc$SetPolicy("automatic", "automatic")
  potModelList<-gtkCListNew(columns=1)
  if (!is.null(possLists))
    for (i in 1:length(possLists))
      gtkCListAppend(potModelList, possLists[i])
 
  gtkCListSetSelectionMode(potModelList, GtkSelectionMode[1])
  sc$AddWithViewport(potModelList)
  boxForPotential$PackStart(sc, expand=TRUE)
  frPotential$Add(boxForPotential)
  gtkTableAttach(tab, frPotential, 0, 3, 0, 8, xpadding=20, ypadding=10)

  lblType<-gtkLabel("Type of Model")
  gtkTableAttach(tab, lblType, 3, 5, 1, 2)
  combType<-gtkCombo()
  gtkComboSetPopdownStrings(combType, sort(c("exprSet", "graph", "data.frame")))
  gtkComboSetValueInList(combType, TRUE, FALSE)
  gtkTableAttach(tab, combType, 3, 5, 2, 3, xpadding=10, ypadding=5)  

  lblName<-gtkLabel("Name of the Model")
  gtkTableAttach(tab, lblName, 3, 5, 4, 5)
  nameEntry<-gtkEntry()
  gtkTableAttach(tab, nameEntry, 3, 5, 5, 6, xpadding=10, ypadding=5)

  gtkAddCallback(gtkComboGetList(combType), "select-child",
    function(obj, objchild)
    {
      # remove everything from potModelList
      gtkCListClear(potModelList)

      # need to change the values in possLists
      curValue<-gtkChildren(objchild)[[1]][["label"]]
      possLists<-getPossLists(type=curValue)

      if (!is.null(possLists))
        for (i in 1:length(possLists))
          gtkCListAppend(potModelList, possLists[i])

      assign("typeModel", curValue, mvcEnv)
      return(TRUE)
    }
  )

  modelBut<-gtkButton("Set Model")
  gtkTableAttach(tab, modelBut, 1, 4, 8, 10, xpadding=20, ypadding=10)

  gtkAddCallback(modelBut, "clicked",
    function(obj, ev)
    {
      # get the chosen model
      # gives the row number starting with zero
      rowInt<-gtkCListGetFocusRow(potModelList)
      chosenModel<-gtkCListGetText(potModelList, row=rowInt, cols=0)  
      typeModel<-get("typeModel", mvcEnv)
 
      # get the entry text
      curNameText<-gtkEntryGetText(nameEntry)      

      if (chosenModel != "" && curNameText != "")
      {
        # 6/1/05 use messages to load models
        curData<-list(data=eval(as.name(chosenModel)), linkData=NULL)

        newMessage<-new("gAddDataMessage", data=curData, dataName=curNameText, 
                        type=typeModel)
        handleMessage(newMessage)

        # need to show that the model was loaded somehow
        updateCurAction("loadM")
      }
      return(TRUE)
    }
  )
}

#########
# return any objects in the global environment that have class list
# returns NULL if no objects have class list in global environment
# type is the class to look for
#########
getPossLists<-function(type="exprSet")
{
  curElements<-ls(.GlobalEnv)
  if (length(curElements) > 0)
  {
    testClass<-rep(FALSE,length(curElements))
    # now check if any of the elements have class list
    for (i in 1:length(curElements))
    {
      if (type != "data.frame")
        testClass[i]<- is(eval(as.name(curElements[i])), type)
      else
        testClass[i]<- is(eval(as.name(curElements[i])), type) ||
                       is(eval(as.name(curElements[i])), "matrix")      
    }
    if (any(testClass))
      return(curElements[testClass])
    else 
      return(NULL)
  }
  else
  return(NULL)
}

##########
# 9/1/05
# plot a heatmap for an exprSet model
##########
setPlotHmView<-function()
{
  # get the controller environment for the active MVC
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()  
  else
  {
    # make sure old action gets set when the first window is loaded
    assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
  }  

  tab<-gtkTableNew(4, 4, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=activeMVC)
  frMVC$Add(tab)  

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  # if there are model variables then let users subset based on them
  if (length(modelVar(model(curMVC))) > 0)
  {
    lblSub<-gtkLabel("Subset by Model Variable")
    gtkTableAttach(tab, lblSub, 0, 2, 0, 2, xpadding=20, ypadding=40)

    combMName<-gtkCombo()
    gtkComboSetPopdownStrings(combMName, c("", names(modelVar(model(curMVC)))))
    gtkComboSetValueInList(combMName, TRUE, FALSE)
    gtkTableAttach(tab, combMName, 2, 4, 0, 2, xpadding=20, ypadding=40)  

    # can only make heatmap of the active MVC
    plotLabel<-paste("Plot", activeMVC)
    plotButton<-gtkButton(plotLabel)
    gtkTableAttach(tab, plotButton, 1, 3, 2, 4, ypadding=45, xpadding=30)
  }
  else
  {
    # can only make heatmap of the active MVC
    plotLabel<-paste("Plot", activeMVC)
    plotButton<-gtkButton(plotLabel)
    gtkTableAttach(tab, plotButton, 1, 3, 1, 3, ypadding=50, xpadding=30)
  }

  gtkAddCallback(plotButton, "clicked",
    function(obj)
    {
      modData<-modelData(model(curMVC))
      if (length(geneNames(modData)) > 1)
      {
        if (length(modelVar(model(curMVC))) > 0)
        {
          # check if the user wants to subset the data
          curMVar<-gtkEntryGetText(gtkContainerGetChildren(combMName)[[1]])
          if (curMVar == "")
            curmData<-list()
          else
          {
            modelVarVal<-modelVar(model(curMVC))[[curMVar]]
            if (is(modelVarVal, "logical"))
              curmData<-list(rNames=
                           geneNames(modelData(model(curMVC)))[modelVarVal])
            else
              curmData<-list()
          }
        }
        else
          curmData<-list()
        vMessage<-new("gAddViewMessage", dataName=activeMVC,
                    type="heatmapView", mData=curmData)
        handleMessage(vMessage)
      }
      else
      {
        if (length(get("controlWin", mvcEnv)) > 0)
          showLabel(paste("This model has only one row and",
                         "the heatmap plot needs at least two rows."))
        else
          print(paste("This model has only one row and",
                         "the heatmap plot needs at least two rows."))
      }      
      return(TRUE)
    }
  )  
}

############
# setPlotGrView sets the control window so it
# shows the view of loaded graphs to
# choose which one to plot
############
setPlotGrView<-function()
{
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  w<-get("controlWin", mvcEnv)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()
  else
  {
    # make sure old action gets set when the first window is loaded
    assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
  }

  # need a list of loaded graphs and a plot button
  # 10 rows and 5 columns
  tab<-gtkTableNew(10, 5, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
  frMVC$Add(tab)

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  lblLayout<-gtkLabel("Layout Method")
  gtkTableAttach(tab, lblLayout, 1, 4, 1, 2)
  combLayout<-gtkCombo()
  gtkComboSetPopdownStrings(combLayout,c("dot", "neato", "twopi"))
  gtkComboSetValueInList(combLayout, TRUE, FALSE)
  gtkTableAttach(tab, combLayout, 1, 4, 2, 3, xpadding=40, ypadding=5)  

  lblNode<-gtkLabel("Node Shape")
  gtkTableAttach(tab, lblNode, 1, 4, 4, 5)
  combNode<-gtkCombo()
  gtkComboSetPopdownStrings(combNode, c("circle", "ellipse", "rectangle"))
  gtkComboSetValueInList(combNode, TRUE, FALSE)
  gtkTableAttach(tab, combNode, 1, 4, 5, 6, xpadding=40, ypadding=5)    

  plotLabel<-paste("Plot", activeMVC)
  plotBut<-gtkButton(plotLabel)
  gtkTableAttach(tab, plotBut, 1, 4, 8, 10, ypadding=10, xpadding=40)

  gtkAddCallback(plotBut, "clicked",
    function(obj, ev)
    {
      # need to get the chosen graph
      chosenGraph<-activeMVC
      layout<-gtkEntryGetText(gtkContainerGetChildren(combLayout)[[1]])
      nodeShape<-gtkEntryGetText(gtkContainerGetChildren(combNode)[[1]])

      if (chosenGraph!="")
      {
        # 5/24/05 new message method
        vMessage<-new("gAddViewMessage", dataName=chosenGraph, 
                      type="graphView", 
                      mData=list(glayout=layout, nShape=nodeShape))
        handleMessage(vMessage)
      }
      return(TRUE)
    }
  )
}

########
# check if x is of class graphModel
########
testGraphClass<-function(x)
{
  is(x, "graphModel")
}

###############
# createColorBrowser creates the color selection dialog so the user
# can pick a color
###############
createColorBrowser<-function()
{
  if (length(get("controlWin", mvcEnv)) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    assign("notDone", TRUE, controlEnv)
    w<-gtkColorSelectionDialog(title="Pick a Color", show=FALSE)
    gtkWindowSetModal(w, TRUE)
    # w gives the whole window - we want to get certain elements of the window
    okBut<-gtkColorSelectionDialogGetOkButton(w)
    canBut<-gtkColorSelectionDialogGetCancelButton(w)
    helpBut<-gtkColorSelectionDialogGetHelpButton(w)
    colorSel<-gtkColorSelectionDialogGetColorsel(w)
    # hide the Help button
    gtkWidgetHide(helpBut)
    w$Show()

    # need to write callbacks for these buttons
    gtkAddCallback(okBut, "clicked",
      function(obj)
      {
        # need to set the color and destroy w 
        curColor<-gtkColorSelectionGetColor(colorSel)
        curRed<-curColor[1]
        curGreen<-curColor[2]
        curBlue<-curColor[3]
        curColor<-rgb(curRed, curGreen, curBlue)
        # now assign the color to the environment
        assign("currentColor", curColor, controlEnv)
        assign("notDone", FALSE, controlEnv)

        w$Destroy()
        return(TRUE)
      }
    )
    gtkAddCallback(canBut,"clicked",
      function(obj)
      {
        assign("notDone", FALSE, controlEnv)
        w$Destroy()
        return(TRUE)
      }
    )
    gtkAddCallback(w, "delete_event",
      function(obj, ev)
      {
        assign("notDone", FALSE, controlEnv)
        w$Destroy()
        return(TRUE)
      }
    )

    # to make the color browser modal
    while (get("notDone", controlEnv))
      Sys.sleep(0.5)
  }
  else
    print(paste("Please call the function, setColor, to set the color. ",  
                "The default color is black."))
}

#########
# get the chip type to link affy ids and locus link ids
#
# annot will be the text in the annotation slot of the exprSet object
#########
getChipType<-function(annot="")
{
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  # make the gtk window modal
  assign("notDone", TRUE, controlEnv)

  # need to get the chip type to link affy ids to locus link ids
  w<-gtkWindow(show=FALSE)
  hbox<-gtkHBox()
  w$Add(hbox)
  lbl<-gtkLabel("Affymetrix Chip Type")
  gtkBoxPackStart(hbox, lbl, padding=5)
  entChip<-gtkEntry()
  # set the default chip type to be hgu95av2
  if (annot=="")
    gtkEntrySetText(entChip, "hgu95av2")
  else
    gtkEntrySetText(entChip, annot)
  gtkBoxPackStart(hbox, entChip, padding=5)
  okBut<-gtkButton("  Ok  ")
  gtkBoxPackStart(hbox, okBut, padding=5)
  w$Show()
  gtkAddCallback(okBut, "clicked",
    function(obj, ev)
    {
      curChipText<-gtkEntryGetText(entChip)
      if (curChipText != "")
      {
        assign("chipType", curChipText, controlEnv)
        w$Destroy()
        assign("notDone", FALSE, controlEnv)
      }
      return(TRUE)
    }
  )
  gtkAddCallback(w, "delete_event",
    function(obj, ev)
    {
      assign("notDone", FALSE, controlEnv)
      w$Destroy()
      return(TRUE)
    }
  )
  while (get("notDone", controlEnv))
    Sys.sleep(0.5)         
}

##########
# getLoadedGraphs gets the names of the loaded graphs in dataEnv
# this is called when the active MVC is has a graph model and the user
# wants to plot the graph
##########
getLoadedGraphs<-function()
{
  mNames<-NULL
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    modelList<-lapply(MVCList, model)
    # need to check the class of model
    booGraph<-unlist(lapply(modelList, testGraphClass))
    if (any(booGraph))
    {
      grModels<-modelList[booGraph]
      mNames<-sort(unlist(lapply(grModels, modelName)))
    }
  }
  return(mNames)
}

#############
# setViewDataView allows the user to view the data
# this should be put in winControl.R in iSNetwork
#############
setViewDataView<-function()
{
  # get the controller environment for the active MVC
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()
  else
  {
    # make sure old action gets set when the first window is loaded
    assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
  }    

  tab<-gtkTableNew(4, 4, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=activeMVC)
  frMVC$Add(tab)  

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  # 7/21/05
  # only allow users to make spreadsheet of the active MVC
  loadedDF<-activeMVC

  butLabel<-paste("Show", activeMVC)
  showButton<-gtkButton(butLabel)
  gtkTableAttach(tab, showButton, 1, 3, 1, 3, ypadding=50, xpadding=30)

  gtkAddCallback(showButton, "clicked",
    function(obj)
    {
      showDF<-get("activeMVC", mvcEnv)

      # need to also check that this data is not currently being shown in
      # a spreadsheet
      if (showDF != "")
      {
        curMVC<-getMVC(showDF)
        curData<-getData(showDF)
        curviewList<-viewList(curMVC)
        booSpreadView<-unlist(lapply(curviewList, function(x) {is(x, 
                                 "spreadView")}))
        # if there are any spread views then this data set is already shown
        # in a spreadsheet
        if (any(booSpreadView))
          booCreate<-FALSE
        else
          booCreate<-TRUE

        if (booCreate)
        {
          if (is(model(curMVC), "dfModel"))
          {
            sData<-cbind(rownames(curData))
            sData<-cbind(sData, curData)

            # create and handle a message to create a view
            vMessage<-new("gAddViewMessage", dataName=showDF, 
                        type="spreadView", mData=list(spreadData=sData, 
                                   colNames=c("Row.Names", colnames(curData))))
            handleMessage(vMessage)
          }
          if (is(model(curMVC), "graphModel"))
          {
            nattr<-getNodeAttr()
            # only build the spreadsheet if there are some node attributes
            if (!is.null(nattr))
            {
              # need to make the data frame to show
              nd<-nodeData(curData)
              sData<-cbind(nodes(curData))
              # now need to add all the node attributes
              for (i in 1:length(nattr))
              {
                curVec<-unlist(lapply(nd, function(x) {x[[i]]}))
                sData<-cbind(sData, curVec)
              }        
              cNames<-c("Node Names", nattr)

              vMessage<-new("gAddViewMessage", dataName=showDF, 
                        type="spreadView",
                        mData=list(spreadData=sData, colNames=cNames))
              handleMessage(vMessage)
            }
            else
            {
              showLabel("There are no node attributes to show in a spreadsheet.")
            }
          }
          if (is(model(curMVC), "gseModel"))
          {
            # decide what to put in the spreadsheet - gene set name (from the 
            # row names of the incidMat), gene set test stat (from the
            # gsTestStat GSE slot), and then all model variables
            geneSetNames<-rownames(incidMat(curData))
            geneSetTestStats<-round(gsTestStat(curData), 3)
            # get the number of genes per gene set
            numberGenes<-apply(incidMat(curData), 1, sum)
            sData<-cbind(geneSetNames, numberGenes, geneSetTestStats)

            # next look at model variables
            curMVar<-modelVar(model(curMVC))
            if (length(curMVar) > 0)
            {
              for (i in 1:length(curMVar))
              {
                sData<-cbind(sData, curMVar[[i]])
              }
              cNames<-c("Gene Set Names", "Number of Genes in Gene Set", 
                        "Gene Set Test Stats", names(curMVar))
            }
            else
              cNames<-c("Gene Set Names", "Number of Genes in Gene Set", 
                        "Gene Set Test Stats")

            vMessage<-new("gAddViewMessage", dataName=showDF, 
                        type="spreadView",
                        mData=list(spreadData=sData, colNames=cNames))
            handleMessage(vMessage)
          }
        }
      }
      return(TRUE)
    }
  )
}

##########
# 7/4/06
# setPlotQQView sets the control window so it the user can create a
# QQ-plot of the gene set test statistics from a gseModel
##########
setPlotQQView<-function()
{
  # get the controller environment for the active MVC
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()  
  else
  {
    # make sure old action gets set when the first window is loaded
    assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
  }  

  tab<-gtkTableNew(4, 4, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=activeMVC)
  frMVC$Add(tab)  

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  # for now can only make a QQ plot of the gene set test statistics from the
  # active MVC
  plotButton<-gtkButton("Create QQ-Plot")
  gtkTableAttach(tab, plotButton, 1, 3, 1, 3, ypadding=50, xpadding=30)

  gtkAddCallback(plotButton, "clicked",
    function(obj)
    {
      createQQplot()
      return(TRUE)
    }
  )
}

##############
# setPlotDView sets the control window so it
# shows the view of loaded dataframes to
# choose which one to plot
# should be put in winControl.R in iSNetwork
##############
setPlotDView<-function()
{
  # get the controller environment for the active MVC
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()
  else
  {
    # make sure old action gets set when the first window is loaded
    assign("lastCWAction", get("curAction", mvcEnv), mvcEnv)
  }

  tab<-gtkTableNew(8, 8, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=activeMVC)
  frMVC$Add(tab)  

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  # changed 7/21/05
  # only allow users to plot the active MVC
  loadedDF<-activeMVC

  frVar<-gtkFrame("Data Variables")
  gtkTableAttach(tab, frVar, 2, 6, 0, 6, xpadding=10)
  setVariables(activeMVC, frVar)

  plotBut<-gtkButton("Plot")
  gtkTableAttach(tab, plotBut, 3, 5, 6, 8, xpadding=20, ypadding=20)

  gtkAddCallback(plotBut, "clicked",
    function(obj, ev)
    {
      dataF<-activeMVC
      curMVC<-getMVC(dataF)
      controlEnv<-controller(curMVC)

      # need to get the two plot variables and call plotting function
      chosenX<-get("chosenX", controlEnv)
      chosenY<-get("chosenY", controlEnv)
      if (chosenX != "" && chosenY != "")
      {
        # then need to call plotting function
        curDF<-getData(modelName=dataF)
        DFrows<-rownames(curDF)

        # 5/24/05 new message method
        vMessage<-new("gAddViewMessage", dataName=dataF, type="sPlotView",
                      mData=list(dfRows=DFrows, xvar=chosenX, yvar=chosenY))
        handleMessage(vMessage)
      }
      return(TRUE)
    }
  )
}

##########
# setVariables sets the dataframe variables in a list box
#
# dataF is the dataframe name, frVar is the frame that
# will hold the dataframe variables
##########
setVariables<-function(modelName, frVar)
{
  # get the controller environment for the current model
  curMVC<-getMVC(modelName)
  controlEnv<-controller(curMVC)

  # need to reset the button boxes before adding anything
  resetVarButtonBoxes(frVar)

  # need to add the button boxes along with the variable labels
  varNames<-names(getData(modelName))

  boxForVar<-gtkVBox(TRUE, 2)
  sc<-gtkScrolledWindow()
  sc$SetPolicy("automatic", "automatic")

  XbuttonList<-list()
  YbuttonList<-list()

  if (length(varNames) > 0)
  {
    # length(varNames)=# of rows, 3 columns, homogenous=FALSE??
    tab<-gtkTable(length(varNames), 3, FALSE)

    for (i in 1:length(varNames))
    {
      XbuttonList[[i]]<-gtkToggleButton("X")
#      XbuttonList[[i]]$SetUsize(10, 10)
      YbuttonList[[i]]<-gtkToggleButton("Y")
#      YbuttonList[[i]]$SetUsize(10, 10)
      lab<-gtkLabel(varNames[i])
#      lab$SetUsize(10, 10)
      gtkTableAttach(tab, XbuttonList[[i]], 0, 1, i-1, i, xpadding=5)
      gtkTableAttach(tab, YbuttonList[[i]], 1, 2, i-1, i, xpadding=5)
      gtkTableAttach(tab, lab, 2, 3, i-1, i, xpadding=5)

      XbuttonList[[i]]$AddCallback("clicked",
        substitute(setToggleX(j, dataFr), list(j=i, dataFr=modelName)))
      YbuttonList[[i]]$AddCallback("clicked",
        substitute(setToggleY(j, dataFr),list(j=i, dataFr=modelName)))
    }
    sc$AddWithViewport(tab)
  }
  assign("toggleButtonXList", XbuttonList, controlEnv)
  assign("toggleButtonYList", YbuttonList, controlEnv)

  boxForVar$PackStart(sc, expand=FALSE)

  frVar$Add(boxForVar)
}

###########
# resetVarButtonBoxes removes all children from the frVar
#
# frVar is the frame containing the button boxes for the
# dataframe variables
###########
resetVarButtonBoxes<-function(frVar)
{
  # remove all items from the frame
  kids<-gtkContainerGetChildren(frVar)
  if (length(kids) > 0)
    for (i in 1:length(kids))
    {
      gtkWidgetDestroy(kids[[i]])
    }
}

############
# setToggleX sets the chosen X value
#
# j is the index of the button that just changed state
# dataFr is the current dataframe
############
setToggleX<-function(j, dataFr)
{
  setToggleInactive(j, "X", dataFr)
}

##############
# setToggleY sets the chosen Y value
#
# j is the index of the button that just changed state
# dataFr is the current dataframe
##############
setToggleY<-function(j, dataFr)
{
  setToggleInactive(j, "Y", dataFr)
}

##############
# setToggleInactive sets the toggle buttons so
# only one button can be active at a time and it
# also sets the chosenX/chosenY values for plotting
#
# j is the index of the button that just changed state
# type is X or Y
# dataFr is the current dataframe
##############
setToggleInactive<-function(j, type, dataFr)
{
  # get the controller environment for the current model
  curMVC<-getMVC(dataFr)
  controlEnv<-controller(curMVC)  

  # need to check if there are two buttons active
  XbuttonList<-get("toggleButtonXList", controlEnv)
  YbuttonList<-get("toggleButtonYList", controlEnv)

  indexActive<-0
  numActive<-0
  for (i in 1:length(XbuttonList))
  {
    if (type=="X")
    {
      if (gtkToggleButtonGetActive(XbuttonList[[i]])==TRUE)
      {
        numActive<-numActive+1
        indexActive<-i
      }
    }
    else
    {
      if (gtkToggleButtonGetActive(YbuttonList[[i]])==TRUE)
      {
        numActive<-numActive+1
        indexActive<-i
      }
    }
  }

  if (numActive > 1)
    for (i in 1:length(XbuttonList))
    {
      if (i != j)
      {
        if (type=="X")
        {
          gtkToggleButtonSetActive(XbuttonList[[i]], FALSE)
        }
        else
        {
          gtkToggleButtonSetActive(YbuttonList[[i]], FALSE)
        }
      }
    }

  # if there is only one active toggle button - set that to be chosen
  else
  {
    # get the variable names
    varNames<-names(getData(dataFr))

    if (type=="X")
      assign("chosenX", varNames[indexActive], controlEnv)
    else
      assign("chosenY", varNames[indexActive], controlEnv)
  }
}

#########
# 6/8/06 
# give users a way to show values that are hidden through the GUI
#########
showHiddenView<-function()
{
  hv<-getHiddenValues(printState=FALSE)
  if (!is.null(hv))
  {
    # use the same interface as used when choosing the affy ids to make the
    # child GO graph
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    curModel<-model(curMVC)  
    controlEnv<-controller(curMVC) 
  
    w<-gtkWindow(show=FALSE)
    w$SetUsize(400, 300)

    # table with 6 rows and 6 columns
    tab<-gtkTableNew(6, 6, TRUE)
    vbox<-gtkVBoxNew()
 
    w$Add(vbox)
    gtkBoxPackStart(vbox, tab)

    frAvail<-gtkFrame("Hidden Values")
    boxForAvail<-gtkVBox(TRUE, 2)
    sc<-gtkScrolledWindow()
    sc$SetPolicy("automatic", "automatic")
    # one column in the new list
    availList<-gtkCListNew(columns=1)
    if (length(hv)>0)
      for (i in 1:length(hv))
      {
        gtkCListAppend(availList, hv[i])
      }

    gtkCListSetAutoSort(availList, TRUE)
    # supposedly this will allow the user to select multiple rows
    # (though I've had problems with it)
    gtkCListSetSelectionMode(availList, GtkSelectionMode[3])
    sc$AddWithViewport(availList)
    boxForAvail$PackStart(sc, expand=TRUE)
    frAvail$Add(boxForAvail)
    gtkTableAttach(tab, frAvail, 0, 2, 0, 4, xpadding=5, ypadding=5)

    selBut<-gtkButton("Select")
    selAllBut<-gtkButton("Select All")
    unsBut<-gtkButton("Unselect")
    unsAllBut<-gtkButton("Unselect All")
    
    gtkAddCallback(selAllBut, "clicked",
      function(obj, ev)
      {
        numRows<-gtkCListGetRows(availList)
        if (numRows > 0)
        {
          for (i in 1:numRows)
          {
            selValue<-gtkCListGetText(availList, row=0, cols=0)
            # first remove the item from availList
            gtkCListRemove(availList, row=0)

            # then add to selList
            gtkCListAppend(selList, selValue)
          }
        }
        return(TRUE)
      }
    )
    gtkAddCallback(selBut, "clicked",
      function(obj, ev)
      {
        # should I do any checking here (to make sure the list has items?)
        numRows<-gtkCListGetRows(availList)
      
        if (numRows > 0)
        {
          # need to determine which elements are highlit in availList
          rowInt<-gtkCListGetFocusRow(availList)
          chosenHv<-gtkCListGetText(availList, row=rowInt, cols=0)  
        
          # next remove this item from availList
          gtkCListRemove(availList, row=rowInt)
 
          # next add the text to selList
          gtkCListAppend(selList, chosenHv)
        }
        return(TRUE)
      }
    )
    gtkAddCallback(unsBut, "clicked",
      function(obj, ev)
      {
        numRows<-gtkCListGetRows(selList)

        if (numRows > 0)
        {
          # need to determine which elements are highlit in selList
          # then remove them from selList and add to availList
          rowInt<-gtkCListGetFocusRow(selList)
          chosenHv<-gtkCListGetText(selList, row=rowInt, cols=0)
      
          # next remove the item from selList
          gtkCListRemove(selList, row=rowInt)
   
          # next add the text to availList
          gtkCListAppend(availList, chosenHv)
        }
        return(TRUE)
      }
    )
    gtkAddCallback(unsAllBut, "clicked",
      function(obj, ev)
      {
        # need to remove everything in selList
        numRows<-gtkCListGetRows(selList)
        if (numRows > 0)
        {
          for (i in 1:numRows)
          {
            selValue<-gtkCListGetText(selList, row=0, cols=0)
            # first remove the item from selList
            gtkCListRemove(selList, row=0)

            # then add to availList
            gtkCListAppend(availList, selValue)
          }
        }
        return(TRUE)
      }
    )

    gtkTableAttach(tab, selAllBut, 2, 4, 0, 1, xpadding=5, ypadding=8)
    gtkTableAttach(tab, selBut, 2, 4, 1, 2, xpadding=5, ypadding=8)
    gtkTableAttach(tab, unsBut, 2, 4, 2, 3, xpadding=5, ypadding=8)
    gtkTableAttach(tab, unsAllBut, 2, 4, 3, 4, xpadding=5, ypadding=8)

    frSel<-gtkFrame("Selected Values to Show")
    boxForSel<-gtkVBox(TRUE, 2)
    scSel<-gtkScrolledWindow()
    scSel$SetPolicy("automatic", "automatic")
    # one column
    selList<-gtkCListNew(columns=1)
    # will be an empty list on load

    gtkCListSetAutoSort(selList, TRUE)
    # supposedly this will allow the user to select multiple items
    gtkCListSetSelectionMode(selList, GtkSelectionMode[3])
    scSel$AddWithViewport(selList)
    boxForSel$PackStart(scSel, expand=TRUE)
    frSel$Add(boxForSel)
    gtkTableAttach(tab, frSel, 4, 6, 0, 4, xpadding=5, ypadding=5)

    doneBut<-gtkButton("Done")
    gtkTableAttach(tab, doneBut, 2, 4, 5, 6, xpadding=5, ypadding=5)

    # here the hidden values will be shown
    gtkAddCallback(doneBut, "clicked",
      function(obj, ev)
      {
        # need to add the function that will create the child model
        # first get the affy ids
        numRows<-gtkCListGetRows(selList)
        if (numRows > 0)
        {
          allshv<-rep("", numRows)
          for (i in 1:numRows)
            allshv[i]<-gtkCListGetText(selList, row=(i-1), cols=0)

          # call the appropriate callback function
          if (is(curModel, "graphModel"))
            hideNode(allshv)
          if (is(curModel, "dfModel"))
            hidePoint(allshv)
          if (is(curModel, "exprModel"))
            hideGene(allshv)
        }
        w$Destroy()
        return(TRUE)
      }
    )

    w$Show()
  }
}

##########
# let the user delete a MVC object through the GUI - do not let the user
# delete the active MVC, though
# note that if this MVC is a parent or a child MVC, deleting this MVC will 
# affect other MVC objects (it will remove the linking)
##########
deleteMVCGUI<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 1)
  {
    w<-gtkWindow(show=FALSE)
    frMVC<-gtkFrame(label="Delete a MVC")
    w$Add(frMVC)
    allvbox<-gtkVBox()
    frMVC$Add(allvbox)

    modNames<-getModelNames()
    activeMVC<-get("activeMVC", mvcEnv)
    # need to remove the active MVC from the possibilities of MVCs to delete
    remModNames<-modNames[!(modNames %in% activeMVC)]

    sc<-gtkScrolledWindow()
    sc$SetPolicy("automatic", "automatic")

    MVCvbox<-gtkVBox()
    if (length(remModNames) > 0)
    {
      radButs<-list()
      radBut<-gtkRadioButtonNewWithLabelFromWidget(label=remModNames[1])
      gtkBoxPackStart(MVCvbox, radBut)
      radButs[[1]]<-radBut
      if (length(remModNames) > 1)
        for (i in 2:length(remModNames))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label=remModNames[i])
          gtkBoxPackStart(MVCvbox, newradBut)
          radButs[[i]]<-newradBut
        }     
    } 
    sc$AddWithViewport(MVCvbox)  
    gtkBoxPackStart(allvbox, sc)

    hbox<-gtkHBox()
    gtkBoxPackStart(allvbox, hbox)

    delBut<-gtkButton("Delete")
    canBut<-gtkButton("Cancel")

    gtkBoxPackStart(hbox, delBut)
    gtkBoxPackStart(hbox, canBut)

    w$Show()  

    # need a callback for the button press event
    gtkAddCallback(canBut, "clicked",
      function(obj, ev)
      {
        # destroy the window without doing anything
        w$Destroy()
        return(TRUE)
      }
    )

    gtkAddCallback(delBut, "clicked",
      function(obj, ev)
      {
        # first find out which name is checked and then delete that MVC
        if (length(remModNames) > 0)
        {
          for (i in 1:length(radButs))
          {
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }      
          deleteMVC<-remModNames[curIndex]

          # now need to remove the views for this MVC 
          delMVC<-getMVC(deleteMVC)
          vList<-viewList(delMVC)
        
          if (length(vList) > 0)
            for (i in 1:length(vList))
              setDeleteEvents(vList[[i]])          

          # next need to check the parentMVC and childMVCList slots
          curParent<-parentMVC(delMVC)
          curChildren<-childMVCList(delMVC)

          if (curParent != "")
          {
            # then need to remove this MVC object from the parent's 
            # childMVCList slot
            unsortMod<-getModelNames(sort=FALSE)
            # there can only be one parent
            changeMVC<-getMVC(curParent)
            # get the parent's child list
            parentChild<-childMVCList(changeMVC)
            # find the child that matches the soon to be deleted MVC
            curChildIndex<-match(deleteMVC, parentChild)
            # reset the child list without the deleted MVC
            newChildList<-parentChild[-curChildIndex]
            childMVCList(changeMVC)<-newChildList

            # now need to reassign this MVC to the MVCList variable
            MVCList<-get("MVCList", mvcEnv)
            curIndex<-match(curParent, unsortMod)
            MVCList[[curIndex]]<-changeMVC
            assign("MVCList", MVCList, mvcEnv)           
          }
          if (length(curChildren) > 0)
          {
            # then need to remove this MVC object from the child's parentMVC 
            # slot
            for (i in 1:length(curChildren))
            {
              unsortMod<-getModelNames(sort=FALSE)
              mvcToChange<-curChildren[[i]]
              changeMVC<-getMVC(mvcToChange)
              parentMVC(changeMVC)<-""
              # now need to reassign this MVC to the MVCList variable
              MVCList<-get("MVCList", mvcEnv)
              curIndex<-match(mvcToChange, unsortMod)
              MVCList[[curIndex]]<-changeMVC
              assign("MVCList", MVCList, mvcEnv)
            }
          }

          # then remove the MVC from MVCList in mvcEnv
          # find the index of the item to remove    
          allNames<-getModelNames(sort=FALSE)
          index<-match(deleteMVC, allNames)

          newMVCList<-MVCList[-index]
          assign("MVCList", newMVCList, mvcEnv)

          # set the control window to empty because the old action may be
          # for a different model type
          if (length(get("controlWin", mvcEnv)) > 0)
            setEmptyView()

          # need to refresh the model graph if there is one
          if (length(get("modelWin", mvcEnv)) > 0)
          {
            Sys.sleep(0.5)
            plotModelGraph()
          }
        }

        # destroy the window
        w$Destroy()
        return(TRUE)
      }
    )
  }
  else
  {
    if (length(MVCList) == 0)
      print("There is no data loaded so there are no MVCs to delete.")
    else
      print(paste("The active MVC cannot be deleted", 
                  "and there is only one MVC loaded at this time.")) 
  }
}

