\name{addCBFunction}
\alias{addCBFunction}
\title{ Add a potential callback function for an event }
\description{
  addCBFunction adds a potential callback function for an event.  This
  callback function will then be available to link to an event through the
  Event menu on the GUI and through the command line function, setCallFunc.
}
\usage{
addCBFunction(callFunction, shortName, preprocessingFun, eventLists, typeModel)
}
\arguments{
  \item{callFunction}{ the callback function }
  \item{shortName}{ a short character description of what the callback
  function does }
  \item{preprocessingFun}{ a list of functions that must be called
  before the callback function; these preprocessing functions
  set values that the callback function will need; if there are no
  preprocessing functions, then this parameter is set to NULL }
  \item{eventLists}{ a vector of event names that this callback function can
  be linked to }
  \item{typeModel}{ the types of models that this callback function will work
  with - for now the options are "graphModel", "dfModel", "exprModel" and/or
  "gseModel" }
}
\details{
  The callback function that is given in the callFunction parameter must take
  only one argument.  That one argument must be one of the following: type
  character to represent a row name from the data frame if the callback
  function acts on a MVC that contains a dfModel, type character to
  represent a node name from the graph if the callback function acts on a MVC
  that contains a graphModel, type character to represent a gene name from the
  exprSet if the callback function acts on a MVC that contains an exprModel,
  or type character to represent a gene set name from the GSE if the callback
  function acts on a MVC that contains a gseModel.
  In other words, the one parameter of the callback function must 
  be of a type that makes sense for the data stored in the model.  The
  callback function must create and handle a gUpdateDataMessage so that the
  data is updated. 

  Not all callback functions will need preprocessing functions so this
  parameter can be set to NULL.  If the callback function needs some
  information besides the row name (which is the only parameter a callback
  function has), then the callback function must have one or more
  preprocessing functions.  For example, when the callback function is to
  color a point, there must be a preprocessing function that has the user set
  the color they want to use.

  Currently, the eventLists parameter can be a vector containing one or more
  of the following events: "mouseOver", "leftButtonClick", "middleButtonClick",
  "rightButtonClick". 

  Certain callback functions only make sense with views of certain types of
  models.  For example, if the callback function colors a node, then the
  only type of model this will work with is a graph so the typeModel parameter
  is set to "graphModel".
}
\author{ Elizabeth Whalen }
\seealso{  
  \code{\link{getEvents}},
  \code{\link{getDescForEvent}},
  \code{\link{setCallFunc}} 
}
\examples{
  if (interactive())
  {
    loadData(modelData=USArrests, dataName="USArrests",
             typeModel="data.frame")
    setActiveMVC("USArrests")
    getDescForEvent("leftButtonClick")

    changePch<-function(rowName)
    {
      curMVC<-getMVCFromWinNum()
      virtualData<-virtualData(model(curMVC))

      # column name
      colName<-"pch"

      # need to change the virtualData slot in model object 
      # using messages
      # for this example, just set the pch value to 2
      colIndex<-match(colName, colnames(virtualData))
      rowIndex<-match(rowName, rownames(virtualData))
      # determining the pch value should be set in a preprocessing function
      newPchValue<-2

      mData<-as.list(newPchValue)
      names(mData)<-rowName

      # if a point was clicked, then we want to create a 
      # gUpdateDataMessage
      dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
      handleMessage(dfMessage)
    }

    addCBFunction(changePch, "change the plotting character", NULL,
    c("leftButtonClick", "middleButtonClick", "rightButtonClick"), 
    typeModel="dfModel")

    getDescForEvent("leftButtonClick")
  }
}
\keyword{ data }
