\name{addNewChildMethod}
\alias{addNewChildMethod}
\title{ Stores the Names of new Methods to Create Child MVC }
\description{
  addNewChildMethod stores the name of a new method to create child MVC so it
  is available for users to see when they call the function,
  getMethodsToCreateChild.  The addNewChildMethod stores the method name in
  the controller slot of the appropriate MVCs.
}
\usage{
addNewChildMethod(methodName, shortName, typeModel)
}
\arguments{
  \item{methodName}{ the name of the function to create the child MVC }
  \item{shortName}{ a short description of what the function does }
  \item{typeModel}{ the type of models this function to create a child MVC
  works with (options are ``dfModel'', ``graphModel'', and ``exprModel'') }
}
\author{ Elizabeth Whalen }
\seealso{ 
  \code{\link{getMethodsToCreateChild}}
}
\examples{
  if (interactive())
  {
    # load a graph
    set.seed(123)
    V <- letters[1:10]
    M <- 1:4
    g1 <- randomGraph(V, M, .2)
    loadData(g1, "testGraph", "graph")   

    setActiveMVC("testGraph")
    getMethodsToCreateChild()

    # now add a method to create a child MVC (please see Vignette)

    # first need to create the link functions
    # this function must take an update data message from the child and convert
    # it into an update data message that works for the parent MVC
    linkToParentByNodeName<-function(updateParentMessage)
    {
      curchildUpdateDataMessage<-childUpdateDataMessage(updateParentMessage)

      childName<-dataName(curchildUpdateDataMessage)
      allMVCs<-getMVCList()
      allMVCNames<-unlist(lapply(allMVCs, function(x) {x@model@modelName}))
      mvcIndex<-match(childName, allMVCNames)
      childMVC<-allMVCs[[mvcIndex]]

      parentName<-parentMVC(childMVC)
      curtype<-type(curchildUpdateDataMessage)
      data<-mData(curchildUpdateDataMessage)

      # now create the new message
      parentMessage<-new("gUpdateDataMessage", type=curtype, mData=data, 
                      dataName=parentName, from=childName) 
      return(parentMessage)
    }

    # this function must take an update data message from the parent and 
    # convert it into an update data message that works for the child MVC
    linkToChildByNodeName<-function(updateChildMessage)
    {
      # first check if the child has the data that is in the parent
      # note that the child model may not have all the node names that the 
      # parent model has
      curparentUpdateDataMessage<-parentUpdateDataMessage(updateChildMessage)
      parentName<-dataName(curparentUpdateDataMessage)
      curChildName<-childName(updateChildMessage)

      allMVCs<-getMVCList()
      allMVCNames<-unlist(lapply(allMVCs, function(x) {x@model@modelName}))
      mvcIndex<-match(curChildName, allMVCNames)
      curChildMVC<-allMVCs[[mvcIndex]]

      childModelData<-modelData(model(curChildMVC))
      childNodeNames<-nodes(childModelData)

      curtype<-type(curparentUpdateDataMessage)
      data<-mData(curparentUpdateDataMessage)

      # the data that was changed in the parent must be in the child to 
      # update the child MVC
      changeNodes<-names(data)

      # need to check that the child has the data that the parent has just 
      # updated
      if (any(changeNodes \%in\% childNodeNames))
      {
        if (all(changeNodes \%in\% childNodeNames))
          childMessage<-new("gUpdateDataMessage", type=curtype, mData=data,
                        dataName=curChildName, from=parentName)
        else
        {
          curIndex<-which(changeNodes \%in\% childNodeNames)
          childMessage<-new("gUpdateDataMessage", type=curtype, 
                        mData=data[curIndex], dataName=curChildName, 
                        from=parentName)
        }
        return(childMessage)
      }
      else
        return(NULL)
    }

    # then create the method to create a child MVC
    createSubGraph<-function(newModelName, nodeNames)
    {
      aMVC<-getActiveMVC()
      if (aMVC != "") 
      {
        allMVCs<-getMVCList()
        allMVCNames<-unlist(lapply(allMVCs, function(x) {x@model@modelName}))
        mvcIndex<-match(aMVC, allMVCNames)
        curMVC<-allMVCs[[mvcIndex]]

        controlEnv <- controller(curMVC)
        # check to make sure this is a valid method
        curModelData<-modelData(model(curMVC))
        if (is(curModelData, "graph"))
        {
          curData <- getData(aMVC)
          if (any(nodeNames \%in\% nodes(curData))) 
          {
            curIndex <- match(nodeNames, nodes(curData))
            curIndex <- curIndex[!is.na(curIndex)]
            nodesToUse<-nodeNames[nodeNames \%in\% nodes(curData)]
            # make the subgraph
            subData<-subGraph(nodesToUse, curData)
            # make the virtual data slot
            subVirData <- virtualData(model(curMVC))[curIndex, ]
            newAddChildMessage <- new("gAddChildMessage",
                  dataName = newModelName, type = "graph",
                  data = list(data = subData, virtualData = subVirData))
            handleMessage(newAddChildMessage, toParent=linkToParentByNodeName,
                  fromParent = linkToChildByNodeName, 
                  subsetParentData = nodeNames)
            assign("childName", "", controlEnv)
            print(paste(newModelName, "data set has been loaded."))
          }
        }
      }                          
    }

    # now add this new method to create a child MVC for graph models
    addNewChildMethod("createSubGraph", "Create Sub Graph", "graphModel") 
    # now you will see this new method
    getMethodsToCreateChild()  
    # now create a child MVC
    createSubGraph("testGrSub", nodeNames=c("a","b","c","e","g","i"))
    setActiveMVC("testGrSub")
  }
}
\keyword{ data }

