\name{heatmapNI}
\alias{heatmapNI}
\title{ Draw a heat map }
\description{
  A heat map is a false color image (basically 'image(t(x))') with a
  dendrogram added to the left side and to the top.  Typically,
  reordering of the rows and columns according to some set of values
  (row or column means) within the restrictions imposed by the
  dendrogram is carried out.

  heatmapNI is slightly different from heatmap in that heatmapNI allows
  multiple variables shown in row and column side images, whereas heatmap only
  allows one variable for the row side image and one variable for the column
  side image. 
}
\usage{
heatmapNI(x, Rowv = NULL, Colv = if (symm) "Rowv" else NULL, distfun = dist, hclustfun = hclust, reorderfun = function(d, w) reorder(d, w), add.expr, symm = FALSE, revC = identical(Colv, "Rowv"), scale = c("row", "column", "none"), na.rm = TRUE, margins = c(5, 5), ColSideColors, RowSideColors, cexRow = 0.2 + 1/log10(nr), cexCol = 0.2 + 1/log10(nc), labRow = NULL, labCol = NULL, main = NULL, xlab = NULL, ylab = NULL, keep.dendro = FALSE, verbose = getOption("verbose"), ...)
}
\arguments{
  \item{x}{ numeric matrix of the values to be plotted. }
  \item{Rowv}{ determines if and how the row dendrogram should be computed
          and reordered.  Either a 'dendrogram' or a vector of values
          used to reorder the row dendrogram or 'NA' to suppress any
          row dendrogram (and reordering) or by default, 'NULL', see
          Details below. }
  \item{Colv}{ determines if and how the column dendrogram should be
          reordered.  Has the same options as the 'Rowv' argument above
          and additionally when 'x' is a square matrix, 'Colv =
          "Rowv"' means that columns should be treated identically to
          the rows. }
  \item{distfun}{ function used to compute the distance (dissimilarity) between
          both rows and columns.  Defaults to 'dist'. }
  \item{hclustfun}{ function used to compute the hierarchical clustering when
          'Rowv' or 'Colv' are not dendrograms.  Defaults to 'hclust'. }
  \item{reorderfun}{ function(d,w) of dendrogram and weights for reordering the
          row and column dendrograms.  The default uses
          'reorder.dendrogram'. }
  \item{add.expr}{ expression that will be evaluated after the call to 'image'.
          Can be used to add components to the plot. }
  \item{symm}{ logical indicating if 'x' should be treated *symm*etrically;
          can only be true when 'x' is a square matrix. }
  \item{revC}{ logical indicating if the column order should be 'rev'ersed
          for plotting, such that e.g., for the symmetric case, the
          symmetry axis is as usual. }
  \item{scale}{ character indicating if the values should be centered and
          scaled in either the row direction or the column direction,
          or none.  The default is '"row"' if 'symm' false, and
          '"none"' otherwise. }
  \item{na.rm}{  logical indicating whether 'NA''s should be removed. }
  \item{margins}{ numeric vector of length 2 containing the margins (see
          'par(mar= *)') for column and row names, respectively. }
  \item{ColSideColors}{ (optional) character vector of length 'ncol(x)'
          containing the color names for a horizontal side bar that may
          be used to annotate the columns of 'x' or a character matrix that is
          created by using cbind on the multiple character vectors of length
          ncol(x). }
  \item{RowSideColors}{ (optional) character vector of length 'nrow(x)'
          containing the color names for a vertical side bar that may
          be used to annotate the rows of 'x' or a character matrix that is
          created by using rbind on the multiple character vectors of length
          nrow(x). }
  \item{cexRow}{ positive number, used as 'cex.axis' in for the row
          axis labeling.  The default currently only uses number
          of rows. }
  \item{cexCol}{ positive number, used as 'cex.axis' in for the
          column axis labeling.  The default currently only uses number
          of columns. }
  \item{labRow}{ character vectors with row labels to use;
          these default to 'rownames(x)'. }
  \item{labCol}{ character vector with column labels to use;
          these default to 'colnames(x)'. }
  \item{main}{ main title; defaults to none. }  
  \item{xlab}{ x-axis title; defaults to none. }
  \item{ylab}{ y-axis title; defaults to none. }
  \item{keep.dendro}{ logical indicating if the dendrogram(s) should be kept as
          part of the result (when 'Rowv' and/or 'Colv' are not NA). }
  \item{verbose}{ logical indicating if information should be printed. }
  \item{\dots}{ additional arguments passed on to 'image', e.g., 'col'
          specifying the colors. }
}
\details{
     If either 'Rowv' or 'Colv' are dendrograms they are honored (and
     not reordered).  Otherwise, dendrograms are computed as 'dd <-
     as.dendrogram(hclustfun(distfun(X)))' where 'X' is either 'x' or
     't(x)'.

     If either is a vector (of "weights") then the appropriate
     dendrogram is reordered according to the supplied values subject
     to the constraints imposed by the dendrogram, by 'reorder(dd,
     Rowv)', in the row case. If either is missing, as by default, then
     the ordering of the corresponding dendrogram is by the mean value
     of the rows/columns, i.e., in the case of rows, 'Rowv <-
     rowMeans(x, na.rm=na.rm)'. If either is 'NULL', no reordering
     will be done for the corresponding side.

     By default ('scale = "row"') the rows are scaled to have mean zero
     and standard deviation one.  There is some empirical evidence from
     genomic plotting that this is useful.

     The default colors are not pretty.  Consider using enhancements
     such as the 'RColorBrewer' package.
}
\value{
     Invisibly, a list with components
  \item{rowInd }{*r*ow index permutation vector as returned by
          'order.dendrogram'.}
  \item{colInd }{*c*olumn index permutation vector.}
  \item{Rowv }{the row dendrogram; only if input 'Rowv' was not NA and
          'keep.dendro' is true.}
  \item{Colv }{the column dendrogram; only if input 'Colv' was not NA and
          'keep.dendro' is true.}
}
\author{  Andy Liaw, original; R. Gentleman, M. Maechler, W. Huber, E. Whalen,
     revisions.}
\note{ 
     Unless 'Rowv = NA' (or 'Colw = NA'), the original rows and columns
     are reordered in any case to match the dendrogram, e.g., the
     rows by 'order.dendrogram(Rowv)' where 'Rowv' is the (possibly
     'reorder()'ed) row dendrogram.

     'heatmap()' uses 'layout' and draws the 'image' in the lower right
     corner of a 2x2 layout. Consequentially, it can *not* be used in a
     multi column/row layout, i.e., when 'par(mfrow= *)' or '(mfcol=
     *)' has been called.

     If RowSideColors or ColSideColors consist of more than one variable, the
     first variable in RowSideColors will be shown on the left and the first
     variable in ColSideColors will be shown on the bottom (closest to the
     heatmap image).
}
\seealso{ 
  \code{\link{heatmap}}, 
  \code{\link{image}},
  \code{\link{hclust}}
}
\examples{
     require(graphics)
     x  <- as.matrix(mtcars)
     rc1 <- rainbow(nrow(x), start=0, end=.3)
     rc2 <- rainbow(nrow(x), start=0.3, end=0.6)
     cc1 <- rainbow(ncol(x), start=0, end=.3)
     cc2 <- rainbow(ncol(x), start=0.3, end=0.6)
     hv <- heatmapNI(x, col = cm.colors(256), scale="column",
                   RowSideColors = rbind(rc1, rc2), 
                   ColSideColors = cbind(cc1, cc2), margin=c(5,10),
                   xlab = "specification variables", ylab= "Car Models",
                   main = "heatmap(<Mtcars data>, ..., scale = \"column\")")

}
\keyword{ aplot }

